/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample_common.c                                           */
/*  Abstract     : I/O toolkit sample program                                */
/*  Author     NEC                                                           */
/*  Date       2011/03/01                                                    */
/*  Version    1.09                                                          */
/*  Notice                                                                   */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sample_common.h"


/** Get memory size.
 * @param  p_dimsize  [IN]Dimension size (int[3])
 * @return Memory size
 */
size_t get_memory_size(const int *p_dimsize)
{
	return (0 < p_dimsize[0] ? p_dimsize[0] : 1) *
		(0 < p_dimsize[1] ? p_dimsize[1] : 1) *
		(0 < p_dimsize[2] ? p_dimsize[2] : 1);
}

/** Get metadata(sample).
 * @param  p_filepath   [IN] Sample file path
 * @param  p_meta_name  [OUT]Metadata name
 * @param  p_meta_value [OUT]Metadata value
 * @param  p_meta_num   [OUT]Number of metadata
 * @return RET_NORMAL
 * @return RET_ERROR
 */
int get_metadata_sample(
	const char *p_filepath,
	char (*p_meta_name)[META_DATA_LEN_MAX],
	char (*p_meta_value)[META_DATA_LEN_MAX],
	int *p_meta_num)
{
	FILE *p_file = NULL;

	int meta_num = 0;
	/* int index = 0, max_size = 0; */
	char buffer[META_DATA_LEN_MAX] = "";
	char *p_token = NULL;
	const char sep[] = "\t";
	
	p_file = fopen(p_filepath, "r");
	if (NULL == p_file)
	{
		/* error */
		E_MSG("Metadata sample open error.[%s]\n", p_filepath);
		return RET_ERROR;
	}
	
	/** read metadata from file */
	while (NULL != fgets(buffer, META_DATA_LEN_MAX, p_file))
	{
		/* delete '\n' */
		char *p_lf = strchr(buffer, '\n');
		if (p_lf) *p_lf = '\0';
		
		p_token = strtok(buffer, sep);
		if (NULL == p_token)
		{
			/* error */
			E_MSG("Metadata format error.[%s(LINE:%d)]\n", p_filepath,
				meta_num + 1);
			return RET_ERROR;
		}
		/* unused */
		/* index = atoi(p_token); */
		
		p_token = strtok(NULL, sep);
		if (NULL == p_token)
		{
			/* error */
			E_MSG("Metadata format error.[%s(LINE:%d)]\n", p_filepath,
				meta_num + 1);
			return RET_ERROR;
		}
		strcpy(&p_meta_name[meta_num][0], p_token);
		
		p_token = strtok(NULL, sep);
		if (NULL == p_token)
		{
			/* error */
			E_MSG("Metadata format error.[%s(LINE:%d)]\n", p_filepath,
				meta_num + 1);
			return RET_ERROR;
		}
		/* unused */
		/* max_size = atoi(p_token); */
		
		p_token = strtok(NULL, sep);
		if (NULL == p_token)
		{
			/* error */
			E_MSG("Metadata format error.[%s(LINE:%d)]\n", p_filepath,
				meta_num + 1);
			return RET_ERROR;
		}
		strcpy(&p_meta_value[meta_num][0], p_token);
		
		meta_num++;
	}
	
	fclose(p_file);
	
	/* set number of metadata */
	*p_meta_num = meta_num;
	
	if (0 == *p_meta_num)
	{
		/* error. */
		E_MSG("Metadata sample file error.[%s]\n", p_filepath);
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


