C*****************************************************************************
C*  Copyright (C) NEC Corporation 2010 All rights reserved.                  *
C*  File Name    : sample3_make_L2Lproduct.f                                 *
C*  Abstract     : I/O toolkit sample program                                *
C*  Function List                                                            *
C*      main                         : Sample program main                   *
C*                                                                           *
C*  Author     NEC                                                           *
C*  Date       2011/03/01                                                    *
C*  Version    1.09                                                          *
C*  Notice                                                                   *
C*****************************************************************************
      Program main

      Implicit NONE

      include 'AMTK_f.h'

      character*40 fname
      data fname/'../data/sample3_IOToolKit_l2l_f.h5'/
      character*40 meta
      data meta/'../data/metadata_sample_L2L.txt'/

C     Scene Scan number
      integer, PARAMETER :: SCENE_SCAN_NUM = 2000
C     Overlap Scan Number
      integer,Parameter :: OVERLAP_SCAN_NUM = 0
C     Meta_data number maximum
      integer,PARAMETER :: META_DATA_NUM_MAX = 200
C     Number of Scans
      integer, PARAMETER :: NUM_SCANS = 2000
C     Number of Geophysical Data
      integer, PARAMETER :: NUM_GEO = 3
      
C     Sample meta data read
      character*512 buffer
      integer*1 datax(512)
      
      character*512 label(4)
      integer*1 idx(512)
      integer*1 metadata_name(512)
      integer*1 max_size(512)
      integer*1 metadata_value(512)

      Integer file_id
      Integer meta_data_num
      Integer scan_num
      Integer pixel_num
      Integer geo_num

      Integer status
      Integer size(3)
      Integer i
      Integer j
      equivalence(buffer, datax)
      equivalence(label(1), idx)
      equivalence(label(2), metadata_name)
      equivalence(label(3), max_size)
      equivalence(label(4), metadata_value)

      Integer lines
      Integer scan_size
      Integer scan_start
      Integer scan_end

      character*3 buf_itoa

C--------------------------------------------------------------------C
C    *definition of data set variable.                               C
C--------------------------------------------------------------------C

C     Dimension size <-> Array size
C     Elements of the array is reversed.
C      Example)
C       AMTK_getDimSize(*, *, dim_size)
C       dim_size(1) : Scan
C       dim_size(2) : Pixel
C       dim_size(3) : Layer
C       -> array(Layer, Pixel, Scan)

C    *Scan Time
      type(AM2_COMMON_SCANTIME) scan_time(NUM_SCANS)

C    *Position in Orbit
      Real*8 pos_orbit(NUM_SCANS)

C    *Geophysical Data (Layer: 1...3)
C    * (Number of scans * 243) */
      Real geo_data_1(AM2_DEF_SNUM_LO, NUM_SCANS)
      Real geo_data_2(AM2_DEF_SNUM_LO, NUM_SCANS)
      Real geo_data_3(AM2_DEF_SNUM_LO, NUM_SCANS)

C    *Latitude of Observation Point
C    *Longitude of Observation Point
      type(AM2_COMMON_LATLON) latlon(AM2_DEF_SNUM_LO, NUM_SCANS)

C    *Pixel Data Quality
C    * (Number of Geophysical Data[1...3] * Numbef of scans * 243)
      Character pixqual(AM2_DEF_SNUM_LO, NUM_SCANS, NUM_GEO)


C--------------------------------------------------------------------C
C    *init data set                                                  C
C--------------------------------------------------------------------C

      scan_size = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM * 2
      scan_start = 1 - OVERLAP_SCAN_NUM
      scan_end = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM

C     HDF product file open (write mode)
      file_id = AMTK_openH5_Write(fname, AM2_CREATE_MODE)
      if (file_id.lt.0) then
          write(*,*) 'AMTK_openH5_Write ERROR' ,file_id
          stop
      endif
      
C     sample meta data read
C     Read Metadata from Metadata file

      lines = 0
      
      open(10, file=meta, status='old')

  555 read(10, 100, end=999) buffer
  100 format(A512)
c
      lines = lines + 1
c
      do i=1,512
          idx(i) = 32
          metadata_name(i) = 32
          max_size(i) = 32
          metadata_value(i) = 32
      end do
c
      i=1
      j=1
   10 if( datax(i).ne.9 ) then
          idx(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 10
      endif
c
      i=i+1
      j=1
   20 if( datax(i).ne.9 ) then
          metadata_name(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 20
      endif
c
      i=i+1
      j=1
   30 if( datax(i).ne.9 ) then
          max_size(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 30
      endif
c
      i=i+1
      j=1
   40 if( i.lt.512 .and. datax(i).ne.0 .and. datax(i).ne.9) then
          metadata_value(j) = datax(i)
          i = i + 1
          j = j + 1
          goto 40
      endif
c

C     Meta data write
      status = AMTK_setMetaDataName(file_id, label(2), label(4))
      if (status.lt.0) then
          write(*,*) 'AMTK_setMetaDataName ERROR' ,status
          close(10)
          stop
      endif
      goto 555

  999 meta_data_num = lines

      close(10)


      Do scan_num =1, scan_size

C         Scan Time
          scan_time(scan_num)%year = 2010
          scan_time(scan_num)%month = mod((scan_num - 1), 12)
          if(scan_time(scan_num)%month.eq.0) then
              scan_time(scan_num)%month = 1
          endif
          scan_time(scan_num)%day = mod((scan_num - 1), 28)
          if(scan_time(scan_num)%day.eq.0) then
              scan_time(scan_num)%day = 1
          endif
          scan_time(scan_num)%hour   = mod((scan_num - 1), 24)
          scan_time(scan_num)%minute = mod((scan_num - 1), 60)
          scan_time(scan_num)%second = mod((scan_num - 1), 60)
          scan_time(scan_num)%ms     = mod((scan_num - 1), 100)
          
C         Position in Orbit
          pos_orbit(scan_num) = 0.001 * scan_num
          
          Do pixel_num =1, AM2_DEF_SNUM_LO
          
C             Geophysical Data (Layer: 1...3)
              geo_data_1(pixel_num, scan_num) = 0.001*(pixel_num-1)
              geo_data_2(pixel_num, scan_num) = 0.001*(pixel_num)
              geo_data_3(pixel_num, scan_num) = 0.001*(pixel_num+1)
              
C             Latitude of Observation Point
C             Longitude of Observation Point
              latlon(pixel_num, scan_num)%lat =
     *            real(mod((pixel_num-1)-90, 91))
              latlon(pixel_num, scan_num)%lon =
     *            real(mod((pixel_num-1)-180, 181))
              
C             Pixel Data Quality
              Do geo_num = 1, NUM_GEO
                  buf_itoa =
     *                char(mod(scan_num + pixel_num + geo_num, 256))
                  pixqual(pixel_num, scan_num, geo_num) = buf_itoa
              End do
              
          End do
          
      End do


C--------------------------------------------------------------------C
C    *output data                                                    C
C--------------------------------------------------------------------C

C     Dimension size <-> Array size
C     Elements of the array is reversed.
C      Example)
C       AMTK_getDimSize(*, *, dim_size)
C       dim_size(1) : Scan
C       dim_size(2) : Pixel
C       dim_size(3) : Layer
C       -> array(Layer, Pixel, Scan)

C     Scan Time
      size(1) = scan_size
      size(2) = 0
      size(3) = 0
      status = AMTK_setDimSize(file_id, AM2_SCAN_TIME_DEF, size)
      if (status.lt.0) then
          write(*,*) 'AMTK_setDimSize ERROR' ,status
          stop
      endif
      status = AMTK_setScanTime(file_id, scan_time,
     *    scan_start, scan_end)
      if (status.lt.0) then
          write(*,*) 'AMTK_setScanTime ERROR' ,status
          stop
      endif

C     Position in Orbit
      size(1) = scan_size
      size(2) = 0
      size(3) = 0
      status = AMTK_setDimSize(file_id, AM2_POS_ORBIT, size)
      if (status.lt.0) then
          write(*,*) 'AMTK_setDimSize ERROR' ,status
          stop
      endif
      status = AMTK_set_SwathDouble(file_id, pos_orbit, scan_start,
     *    scan_end, AM2_POS_ORBIT)
      if (status.lt.0) then
          write(*,*) 'AMTK_set_SwathDouble ERROR' ,status
          stop
      endif

C     Geophysical Data (Layer: 1...3)
      size(1) = scan_size
      size(2) = AM2_DEF_SNUM_LO
      size(3) = NUM_GEO
      
C     Layer: ALL -> AM2_SWATH_GEOA
      status = AMTK_setDimSize(file_id, AM2_SWATH_GEOA, size)
      if (status.lt.0) then
          write(*,*) 'AMTK_setDimSize ERROR' ,status
          stop
      endif
      
C     Layer: 1...3 -> AM2_SWATH_GEO1 ... AM2_SWATH_GEO3
      status = AMTK_set_SwathFloat(file_id, geo_data_1, scan_start,
     *    scan_end, AM2_SWATH_GEO1)
      if (status.lt.0) then
          write(*,*) 'AMTK_set_SwathFloat ERROR' ,status
          stop
      endif
      status = AMTK_set_SwathFloat(file_id, geo_data_2, scan_start,
     *    scan_end, AM2_SWATH_GEO2)
      if (status.lt.0) then
          write(*,*) 'AMTK_set_SwathFloat ERROR' ,status
          stop
      endif
      status = AMTK_set_SwathFloat(file_id, geo_data_3, scan_start,
     *    scan_end, AM2_SWATH_GEO3)
      if (status.lt.0) then
          write(*,*) 'AMTK_set_SwathFloat ERROR' ,status
          stop
      endif
      
C     Latitude of Observation Point
C     Longitude of Observation Point
      size(1) = scan_size
      size(2) = AM2_DEF_SNUM_LO
      size(3) = 0
      status = AMTK_setDimSize(file_id, AM2_LATLON_L2_LO, size)
      if (status.lt.0) then
          write(*,*) 'AMTK_setDimSize ERROR' ,status
          stop
      endif
      status = AMTK_setLatLon(file_id, latlon, scan_start,
     *    scan_end, AM2_LATLON_L2_LO)
      if (status.lt.0) then
          write(*,*) 'AMTK_setLatLon ERROR' ,status
          stop
      endif

C     Pixel Data Quality
      size(1) = NUM_GEO
      size(2) = scan_size
      size(3) = AM2_DEF_SNUM_LO
      status = AMTK_setDimSize(file_id, AM2_PIX_QUAL, size)
      if (status.lt.0) then
          write(*,*) 'AMTK_setDimSize ERROR' ,status
          stop
      endif
      status = AMTK_set_SwathUChar(file_id, pixqual, scan_start,
     *    scan_end, AM2_PIX_QUAL)
      if (status.lt.0) then
          write(*,*) 'AMTK_set_SwathUChar ERROR' ,status
          stop
      endif
      
C     HDF product file close (write mode)
      status = AMTK_closeH5_Write(file_id)
      if (status.lt.0) then
          write(*,*) 'AMTK_closeH5_Write ERROR' ,status
          stop
      endif

      Write(*,*) 'L2(Low) product created'

      Stop
      End


