C*****************************************************************************
C*  Copyright (C) NEC Corporation 2010 All rights reserved.                  *
C*  File Name    : sample3_make_L2Lproduct.f                                 *
C*  Abstract     : I/O toolkit sample program                                *
C*  Function List                                                            *
C*      main                         : Sample program main                   *
C*                                                                           *
C*  Author     NEC                                                           *
C*  Date       2011/03/01                                                    *
C*  Version    1.09                                                          *
C*  Notice                                                                   *
C*****************************************************************************
      Program main

      Implicit NONE

      include 'AMTK_f.h'

      character*40 fname
      data fname/'../data/sample5_IOToolKit_l3d_f.h5'/
      character*40 meta
      data meta/'../data/metadata_sample_L3_day.txt'/

C--------------------------------------------------------------------C
C     GeophysicalName: Cloud Liquid Water                            C
C     Projection     : EQR                                           C
C     Resolution     : 0.25deg                                       C
C--------------------------------------------------------------------C

C     Number of Geophysical Data
      integer, PARAMETER :: NUM_GEO = 1
      
C     Sample meta data read
      character*512 buffer
      integer*1 datax(512)
      
      character*512 label(4)
      integer*1 idx(512)
      integer*1 metadata_name(512)
      integer*1 max_size(512)
      integer*1 metadata_value(512)

      Integer file_id
      Integer meta_data_num
      Integer x
      Integer y

      Integer status
      Integer size(3)
      Integer i
      Integer j
      equivalence(buffer, datax)
      equivalence(label(1), idx)
      equivalence(label(2), metadata_name)
      equivalence(label(3), max_size)
      equivalence(label(4), metadata_value)

      Integer lines

      Integer x_pixsize
      Integer y_pixsize

C--------------------------------------------------------------------C
C    *definition of data set variable.                               C
C--------------------------------------------------------------------C

C     Dimension size <-> Array size
C     Elements of the array is reversed.
C      Example)
C       AMTK_getDimSize(*, *, dim_size)
C       dim_size(1) : Scan
C       dim_size(2) : Pixel
C       dim_size(3) : Layer
C       -> array(Layer, Pixel, Scan)

C    *Geophysical Data (Layer: 1...3)
C    * (X pixel * Y pixel) */
      Real geo_data_1(AM2_DEF_L3L_EQ_Y, AM2_DEF_L3L_EQ_X)

C    *Time Infomation
C    * (X pixel * Y pixel)
      Integer time_info(AM2_DEF_L3L_EQ_Y, AM2_DEF_L3L_EQ_X)


C--------------------------------------------------------------------C
C    *init data set                                                  C
C--------------------------------------------------------------------C
C     Number of pixels in X direction(longitude): 1440
      x_pixsize = AM2_DEF_L3L_EQ_X
C     Number of pixels in Y direction(latitude ): 720
      y_pixsize = AM2_DEF_L3L_EQ_Y

C     HDF product file open (write mode)
      file_id = AMTK_openH5_Write(fname, AM2_CREATE_MODE)
      if (file_id.lt.0) then
          write(*,*) 'AMTK_openH5_Write ERROR' ,file_id
          stop
      endif
      
C     sample meta data read
C     Read Metadata from Metadata file

      lines = 0
      
      open(10, file=meta, status='old')

  555 read(10, 100, end=999) buffer
  100 format(A512)
c
      lines = lines + 1
c
      do i=1,512
          idx(i) = 32
          metadata_name(i) = 32
          max_size(i) = 32
          metadata_value(i) = 32
      end do
c
      i=1
      j=1
   10 if( datax(i).ne.9 ) then
          idx(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 10
      endif
c
      i=i+1
      j=1
   20 if( datax(i).ne.9 ) then
          metadata_name(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 20
      endif
c
      i=i+1
      j=1
   30 if( datax(i).ne.9 ) then
          max_size(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 30
      endif
c
      i=i+1
      j=1
   40 if( i.lt.512 .and. datax(i).ne.0 .and. datax(i).ne.9) then
          metadata_value(j) = datax(i)
          i = i + 1
          j = j + 1
          goto 40
      endif
c

C     Meta data write
      status = AMTK_setMetaDataName(file_id, label(2), label(4))
      if (status.lt.0) then
          write(*,*) 'AMTK_setMetaDataName ERROR' ,status
          close(10)
          stop
      endif
      goto 555

  999 meta_data_num = lines

      close(10)


C     Init data set
      Do y = 1, y_pixsize

          Do x = 1, x_pixsize
              
C             Geophysical Data (Layer: 1...3)
              geo_data_1(y, x) = 0.001 * (x + y - 1)
              
C             Time Infomation
              time_info(y, x) = x + y
              
          End do
          
      End do


C--------------------------------------------------------------------C
C    *output data                                                    C
C--------------------------------------------------------------------C

C     Dimension size <-> Array size
C     Elements of the array is reversed.
C      Example)
C       AMTK_getDimSize(*, *, dim_size)
C       dim_size(1) : Scan
C       dim_size(2) : Pixel
C       dim_size(3) : Layer
C       -> array(Layer, Pixel, Scan)

C     Geophysical Data (Layer: 1...3)
      size(1) = x_pixsize
      size(2) = y_pixsize
      size(3) = NUM_GEO
      
C     Layer: ALL -> AM2_GRID_GEOA
      status = AMTK_setDimSize(file_id, AM2_GRID_GEOA, size)
      if (status.lt.0) then
          write(*,*) 'AMTK_setDimSize ERROR' ,status
          stop
      endif
      
C     Layer: 1...3 -> AM2_GRID_GEO1 ... AM2_GRID_GEO3
      status = AMTK_set_GridFloat(file_id, geo_data_1, AM2_GRID_GEO1)
      if (status.lt.0) then
          write(*,*) 'AMTK_set_GridFloat ERROR' ,status
          stop
      endif

C     Time Information
      size(1) = x_pixsize
      size(2) = y_pixsize
      size(3) = 0

      status = AMTK_setDimSize(file_id, AM2_GRID_TIME, size)
      if (status.lt.0) then
          write(*,*) 'AMTK_setDimSize ERROR' ,status
          stop
      endif
      status = AMTK_set_GridInt(file_id, time_info, AM2_GRID_TIME)
      if (status.lt.0) then
          write(*,*) 'AMTK_set_GridInt ERROR' ,status
          stop
      endif
      
C     HDF product file close (write mode)
      status = AMTK_closeH5_Write(file_id)
      if (status.lt.0) then
          write(*,*) 'AMTK_closeH5_Write ERROR' ,status
          stop
      endif

      Write(*,*) 'L3(Day) product created'

      Stop
      End


