C*****************************************************************************
C*  Copyright (C) NEC Corporation 2010 All rights reserved.                  *
C*  File Name    : sample8_read_L2Lproduct.f                                 *
C*  Abstract     : I/O toolkit sample program                                *
C*  Function List                                                            *
C*      main                         : Sample program main                   *
C*                                                                           *
C*  Author     NEC                                                           *
C*  Date       2011/03/01                                                    *
C*  Version    1.09                                                          *
C*  Notice                                                                   *
C*****************************************************************************
      Program main

      Implicit NONE

      include 'AMTK_f.h'


C     PRINT_SCAN_NUM defines print data number
      Integer, PARAMETER ::  PRINT_SCAN_NUM = 2
C     Number of Scans(Variable size)
      integer, PARAMETER :: NUM_SCANS_VAR = 2000
C     Number of Geophysical Data(Variable size)
      integer, PARAMETER :: NUM_GEO_VAR = 3

      character*40 fname
      data fname/'../data/sample3_IOToolKit_l2l_f.h5'/
      character*40 meta
      data meta/'../data/metadata_sample_L2L.txt'/

C     Sample meta data read
      character*512 buffer
      integer*1 datax(512)
      
      character*512  label(4)
      integer*1 idx(512)
      integer*1 metadata_name(512)
      integer*1 max_size(512)
      integer*1 metadata_value(512)

      character*1024 metadata_area_buff
      integer*1 metadata_area(1024)
      equivalence(metadata_area_buff, metadata_area)
c

      Integer file_id
      Integer meta_data_num
      Integer scan_num
      Integer pixel_num
      Integer geo_num

      Integer status
      Integer i
      Integer j
      equivalence(buffer, datax)
      equivalence(label(1), idx)
      equivalence(label(2), metadata_name)
      equivalence(label(3), max_size)
      equivalence(label(4), metadata_value)

      Integer dim_size(3)

      Integer lines
      Integer scan_size
      Integer scan_start
      Integer scan_end
      Integer geo_size
      
      Integer num_scans
      Integer num_overlap_scans
      Character*4 buf_num_scans
      Character*1 buf_num_overlap_scans

C--------------------------------------------------------------------C
C    *definition of data set variable.                               C
C--------------------------------------------------------------------C

C     Dimension size <-> Array size
C     Elements of the array is reversed.
C      Example)
C       AMTK_getDimSize(*, *, dim_size)
C       dim_size(1) : Scan
C       dim_size(2) : Pixel
C       dim_size(3) : Layer
C       -> array(Layer, Pixel, Scan)

C    *Scan Time
      type(AM2_COMMON_SCANTIME) scan_time(NUM_SCANS_VAR)

C    *Position in Orbit
      Real*8 pos_orbit(NUM_SCANS_VAR)

C    *Geophysical Data (Layer: 1...3)
C    * (Number of scans * 243) */
      Real geo_data_1(AM2_DEF_SNUM_LO, NUM_SCANS_VAR)
      Real geo_data_2(AM2_DEF_SNUM_LO, NUM_SCANS_VAR)
      Real geo_data_3(AM2_DEF_SNUM_LO, NUM_SCANS_VAR)

C    *Latitude of Observation Point
C    *Longitude of Observation Point
      type(AM2_COMMON_LATLON) latlon(AM2_DEF_SNUM_LO, NUM_SCANS_VAR)

C    *Pixel Data Quality
C    * (Number of Geophysical Data[1...3] * Numbef of scans * 243)
      Character pixqual(AM2_DEF_SNUM_LO, NUM_SCANS_VAR, NUM_GEO_VAR)

C--------------------------------------------------------------------C
C    *input data                                                     C
C--------------------------------------------------------------------C

      lines = 0

C     HDF product file open
      file_id = AMTK_openH5(fname)
      if (file_id.lt.0) then
          write(*,*) 'AMTK_openH5 ERROR' ,file_id
          stop
      endif

C     sample meta data read
C     Read Metadata from Metadata file
      open(10, file=meta, status='old')

  555 read(10, 100, end=999) buffer
  100 format(A512)
c
      lines = lines + 1
c
      do i=1,512
          idx(i) = 32
          metadata_name(i) = 32
          max_size(i) = 32
          metadata_value(i) = 32
      end do
c
      i=1
      j=1
   10 if( datax(i).ne.9 ) then
          idx(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 10
      endif
c
      i=i+1
      j=1
   20 if( datax(i).ne.9 ) then
          metadata_name(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 20
      endif
c
      i=i+1
      j=1
   30 if( datax(i).ne.9 ) then
          max_size(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 30
      endif
c
      i=i+1
      j=1
   40 if( i.lt.512 .and. datax(i).ne.0 .and. datax(i).ne.9) then
          metadata_value(j) = datax(i)
          i = i + 1
          j = j + 1
          goto 40
      endif

      do i=1,1024
          metadata_area(i) = 32
      end do

      status = AMTK_getMetaDataName(file_id, 
     *    label(2),metadata_area_buff)
      if (status.lt.0) then
          write(*,*) 'AMTK_getMetaDataName ERROR' ,status
          stop
      endif
      if( status.ne.0) then
         write(*,*) 'No=', lines, 'Name=',trim(label(2)),
     *    ' Value=',metadata_area_buff(1:status)
      endif

      goto 555

  999 meta_data_num = lines

      Close(10)

C     data set read
C     Get scan size.
      buf_num_scans = ''
      status = AMTK_getMetaDataName(file_id, 'NumberOfScans',
     *    buf_num_scans)
      if (status.lt.0) then
          write(*,*) 'AMTK_getMetaDataName ERROR' ,status
          stop
      endif
      READ(buf_num_scans, *) num_scans
      
      buf_num_overlap_scans = ''
      status = AMTK_getMetaDataName(file_id, 'OverlapScans',
     *    buf_num_overlap_scans)
      if (status.lt.0) then
          write(*,*) 'AMTK_getMetaDataName ERROR' ,status
          stop
      endif
      READ(buf_num_overlap_scans, *) num_overlap_scans

      scan_size = num_scans + num_overlap_scans * 2
      scan_start = 1 - num_overlap_scans
      scan_end = num_scans + num_overlap_scans

C     Scan Time
      status = AMTK_getDimSize(file_id, AM2_SCAN_TIME_DEF, dim_size)
      if (status.lt.0) then
          write(*,*) 'AMTK_getDimSize ERROR' ,status
          stop
      endif
      status = AMTK_getScanTime(file_id, scan_start,
     *    scan_end, scan_time)
      if (status.lt.0) then
          write(*,*) 'AMTK_getScanTime ERROR' ,status
          stop
      endif
      write(*,*) status
      Do scan_num =1, PRINT_SCAN_NUM
          write(*,'(8(A,I4,A))') 'Scan Time No=', scan_num,'  ',
     *        '[' ,scan_time( scan_num )%year,']  ',
     *        '[' ,scan_time( scan_num )%month,'/',
     *        ''  ,scan_time( scan_num )%day,  '] ',
     *        '[' ,scan_time( scan_num )%hour, ':',
     *        ''  ,scan_time( scan_num )%minute,':',
     *        ''  ,scan_time( scan_num )%second,'.',
     *        ''  ,scan_time( scan_num )%ms,']'
      End do

C     Position in Orbit
      status = AMTK_get_SwathDouble(file_id, pos_orbit, scan_start,
     *                                   scan_end, AM2_POS_ORBIT)
      if (status.lt.0) then
          write(*,*) 'AMTK_get_SwathDouble ERROR' ,status
          stop
      endif

C     Geophysical Data (Layer: 1...3)
C     Layer: ALL -> AM2_SWATH_GEOA
      status = AMTK_getDimSize(file_id, AM2_SWATH_GEOA, dim_size)
      if (status.lt.0) then
          write(*,*) 'AMTK_getDimSize ERROR' ,status
          stop
      endif
      
C     * get Number of Geophysical Data
C     # dim_size(1) : Number of Scans
C     # dim_size(2) : Number of Pixel
C     # dim_size(3) : Number of Geophysical Data
      geo_size = dim_size(3)
      write(*,*) 'Number of Geophysical Data(Layer) = ', geo_size
      
C     Layer: 1 -> AM2_SWATH_GEO1
      if (geo_size.ge.1) then
          status = AMTK_get_SwathFloat(file_id, geo_data_1,
     *        scan_start, scan_end, AM2_SWATH_GEO1)
          if (status.lt.0) then
              write(*,*) 'AMTK_get_SwathFloat ERROR' ,status
              stop
          endif
      endif
C     Layer: 2 -> AM2_SWATH_GEO2
      if (geo_size.ge.2) then
          status = AMTK_get_SwathFloat(file_id, geo_data_2,
     *        scan_start, scan_end, AM2_SWATH_GEO2)
          if (status.lt.0) then
              write(*,*) 'AMTK_get_SwathFloat ERROR' ,status
              stop
          endif
      endif
C     Layer: 3 -> AM2_SWATH_GEO2
      if (geo_size.ge.3) then
          status = AMTK_get_SwathFloat(file_id, geo_data_3,
     *        scan_start, scan_end, AM2_SWATH_GEO3)
          if (status.lt.0) then
              write(*,*) 'AMTK_get_SwathFloat ERROR' ,status
              stop
          endif
      endif

C     Latitude of Observation Point
C     Longitude of Observation Point
      status = AMTK_getLatLon(file_id, latlon, scan_start, scan_end,
     *    AM2_LATLON_L2_LO)
      if (status.lt.0) then
          write(*,*) 'AMTK_getLatLon ERROR' ,status
          stop
      endif

C     Pixel Data Quality
      status = AMTK_get_SwathUChar(file_id, pixqual, scan_start,
     *    scan_end, AM2_PIX_QUAL)
      if (status.lt.0) then
          write(*,*) 'AMTK_get_SwathUChar ERROR' ,status
          stop
      endif

      Do scan_num =1, PRINT_SCAN_NUM

C         first pixel
          pixel_num = 1
          write(*,'(2(A,I4))') 'Scan = ', scan_num, ' Pixel = ',
     *        pixel_num

C         Geophysical Data
          write(*,'(3(A,F8.3))')
     *        'Geophysical Data (Layer:1) = ',
     *            geo_data_1(pixel_num, scan_num),
     *        ' (Layer:2) = ',
     *            geo_data_2(pixel_num, scan_num),
     *        ' (Layer:3) = ',
     *            geo_data_3(pixel_num, scan_num)

C         Latitude/Longitude of Observation Point
          write(*,'(2(A,F8.2))')
     *        'Latitude = ', latlon(pixel_num, scan_num)%lat,
     *        '  Longitude = ' , latlon(pixel_num, scan_num)%lon

C         Pixel Data Quality
          Do geo_num = 1, geo_size
              write(*,*) 'Pixel Data Quality (Layer:', geo_num,
     *            ') = ',
     *            ICHAR(pixqual(pixel_num, scan_num, geo_num))
          End do

C         last pixel
          pixel_num = AM2_DEF_SNUM_LO
          write(*,'(2(A,I4))') 'Scan = ', scan_num, ' Pixel = ',
     *        pixel_num

C         Geophysical Data
          write(*,'(3(A,F8.3))')
     *        'Geophysical Data (Layer:1) = ',
     *            geo_data_1(pixel_num, scan_num),
     *        ' (Layer:2) = ',
     *            geo_data_2(pixel_num, scan_num),
     *        ' (Layer:3) = ',
     *            geo_data_3(pixel_num, scan_num)
     
C         Latitude/Longitude of Observation Point
          write(*,'(2(A,F8.2))')
     *        'Latitude = ', latlon(pixel_num, scan_num)%lat,
     *        '  Longitude = ' , latlon(pixel_num, scan_num)%lon

C         Pixel Data Quality
          Do geo_num = 1, geo_size
              write(*,*) 'Pixel Data Quality (Layer:', geo_num,
     *            ') = ',
     *            ICHAR(pixqual(pixel_num, scan_num, geo_num))
          End do

      End do

C     HDF product file close (write mode)
      status = AMTK_closeH5(file_id)
      if (status.lt.0) then
          write(*,*) 'AMTK_closeH5 ERROR' ,status
          stop
      endif
      
      Write(*,*) 'L2(Low) product read end'

      Stop
      End
      
