C*****************************************************************************
C*  Copyright (C) NEC Corporation 2010 All rights reserved.                  *
C*  File Name    : sample1.f                                                 *
C*  Abstract     : I/O toolkit sample program                                *
C*  Function List                                                            *
C*      main                         : Sample program main                   *
C*                                                                           *
C*  Author     NEC                                                           *
C*  Date       2011/02/01                                                    *
C*  Version    1.08                                                          *
C*  Notice                                                                   *
C*****************************************************************************
      Program main

      Implicit NONE

      include 'AMTK_f77.h'

      character*40 fname
      data fname/'../data/sample1_IOToolKit_f77.h5'/
      Integer file_id
      Integer status
      
      character*11 name_productname
      data name_productname/'ProductName'/
      character*9 val_productname
      data val_productname/'AMSR2-L1B'/
      
      character*12 name_overlapscans
      data name_overlapscans/'OverlapScans'/
      character*2 val_overlapscans
      data val_overlapscans/'20'/
      
      character*20 meta_value
      data meta_value/''/
      Integer size(3)
      Integer i
      Integer j
      
      real navi(6, 2040)
      
C     Scene Scan number
      integer SCENE_SCAN_NUM
      PARAMETER (SCENE_SCAN_NUM=2000)
C     Overlap Scan Number
      integer OVERLAP_SCAN_NUM
      PARAMETER (OVERLAP_SCAN_NUM=20)
      
      Integer scan_start
      Integer scan_end
      Integer scan_size
      
      scan_size = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM * 2
      scan_start = 1 - OVERLAP_SCAN_NUM
      scan_end = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM
      
C--------------------------------------------------------------------C
C    *Output product                                                 C
C--------------------------------------------------------------------C
C     file open
      file_id = AMTK_openH5_Write(fname , AM2_CREATE_MODE)
      if (file_id.lt.0) then
          write(*,*) 'AMTK_openH5_Write ERROR' ,file_id
          stop
      endif
      
C     set metadata
      status = AMTK_setMetaDataName(file_id, name_productname,
     *    val_productname)
      if (status.lt.0) then
          write(*,*) 'AMTK_setMetaDataName ERROR' ,status
          stop
      endif
      
      status = AMTK_setMetaDataName(file_id, name_overlapscans,
     *    val_overlapscans)
      if (status.lt.0) then
          write(*,*) 'AMTK_setMetaDataName ERROR' ,status
          stop
      endif
      
C     set dataset

C     Dimension size <-> Array size
C     Elements of the array is reversed.
C      Example)
C       AMTK_getDimSize(*, *, dim_size)
C       dim_size(1) : Scan
C       dim_size(2) : Pixel
C       -> array(Pixel, Scan)

      size(1) = scan_size
      size(2) = 6
      size(3) = 0
      
      status = AMTK_setDimSize(file_id, AM2_NAVI, size)
      if (status.lt.0) then
          write(*,*) 'AMTK_setDimSize ERROR' ,status
          stop
      endif
      
      Do j = 1, scan_size
          Do i = 1, 6
              navi(i, j) = (j - OVERLAP_SCAN_NUM - 1) * 0.1
          End do
      End do
      
      status = AMTK_set_SwathFloat(file_id, navi, scan_start,
     *    scan_end, AM2_NAVI)
      if (status.lt.0) then
          write(*,*) 'AMTK_set_SwathFloat ERROR' ,status
          stop
      endif
      
C     file close
      status = AMTK_closeH5_Write(file_id)
      if (status.lt.0) then
          write(*,*) 'AMTK_closeH5_Write ERROR' ,status
          stop
      endif
      
C--------------------------------------------------------------------C
C    *Input product                                                  C
C--------------------------------------------------------------------C
C     file open
      file_id = AMTK_openH5(fname)
      if (file_id.lt.0) then
          write(*,*) 'AMTK_openH5 ERROR' ,file_id
          stop
      endif
      
C     get metadata
      status = AMTK_getMetaDataName(file_id, name_productname,
     *    meta_value)
      if (status.lt.0) then
          write(*,*) 'AMTK_getMetaDataName ERROR' ,status
          stop
      endif
      
C     get dataset
      status = AMTK_get_SwathFloat(file_id, navi, scan_start,
     *    scan_end, AM2_NAVI)
      if (status.lt.0) then
          write(*,*) 'AMTK_get_SwathFloat ERROR' ,status
          stop
      endif
      
C     file close
      status = AMTK_closeH5(file_id)
      if (status.lt.0) then
          write(*,*) 'AMTK_closeH5 ERROR' ,status
          stop
      endif
      
      Write(*,*) 'ProductName = ', meta_value,
     *    ' Navigation Data(1, 1) = ', navi(1, 1)

      Stop
      End

