C*****************************************************************************
C*  Copyright (C) NEC Corporation 2010 All rights reserved.                  *
C*  File Name    : sample10_read_L3Dproduct.f                                 *
C*  Abstract     : I/O toolkit sample program                                *
C*  Function List                                                            *
C*      main                         : Sample program main                   *
C*                                                                           *
C*  Author     NEC                                                           *
C*  Date       2011/03/01                                                    *
C*  Version    1.09                                                          *
C*  Notice                                                                   *
C*****************************************************************************
      Program main

      Implicit NONE

      include 'AMTK_f77.h'

      character*40 fname
      data fname/'../data/sample5_IOToolKit_l3d_f77.h5'/
      character*40 meta
      data meta/'../data/metadata_sample_L3_day.txt'/

C--------------------------------------------------------------------C
C     GeophysicalName: Cloud Liquid Water                            C
C     Projection     : EQR                                           C
C     Resolution     : 0.25deg                                       C
C--------------------------------------------------------------------C

C     Number of Geophysical Data
      Integer NUM_GEO
      PARAMETER (NUM_GEO = 1)
C     PRINT_SCAN_NUM defines print data number
      Integer PRINT_SCAN_NUM
      PARAMETER (PRINT_SCAN_NUM = 2)

C     Sample meta data read
      character*512 buffer
      integer*1 datax(512)
      
      character*512  label(4)
      integer*1 idx(512)
      integer*1 metadata_name(512)
      integer*1 max_size(512)
      integer*1 metadata_value(512)

      character*1024 metadata_area_buff
      integer*1 metadata_area(1024)
      equivalence(metadata_area_buff, metadata_area)
c

      Integer file_id
      Integer meta_data_num
      Integer x
      Integer y

      Integer status
      Integer i
      Integer j
      equivalence(buffer, datax)
      equivalence(label(1), idx)
      equivalence(label(2), metadata_name)
      equivalence(label(3), max_size)
      equivalence(label(4), metadata_value)

      Integer dim_size(3)

      Integer lines

      Integer x_pixsize
      Integer y_pixsize
      Integer geo_size

C--------------------------------------------------------------------C
C    *definition of data set variable.                               C
C--------------------------------------------------------------------C

C     Dimension size <-> Array size
C     Elements of the array is reversed.
C      Example)
C       AMTK_getDimSize(*, *, dim_size)
C       dim_size(1) : Scan
C       dim_size(2) : Pixel
C       dim_size(3) : Layer
C       -> array(Layer, Pixel, Scan)

C    *Geophysical Data (Layer: 1...3)
C    * (X pixel * Y pixel) */
      Real geo_data_1(AM2_DEF_L3L_EQ_Y, AM2_DEF_L3L_EQ_X)

C    *Time Infomation
C    * (X pixel * Y pixel)
      Integer time_info(AM2_DEF_L3L_EQ_Y, AM2_DEF_L3L_EQ_X)

C     Latitude of Observation Point
C     Longitude of Observation Point
C     # L3LATLONFILE: data(X pixel, Y pixel)
C     # Example) data(1440, 720)
C     # -> X pixel and Y pixel is reversed. 
      COMMON /latlon/ latlon
      character*8 latlon(AM2_DEF_L3L_EQ_X, AM2_DEF_L3L_EQ_Y)
      
C--------------------------------------------------------------------C
C    *input data                                                     C
C--------------------------------------------------------------------C

      lines = 0

C     HDF product file open
      file_id = AMTK_openH5(fname)
      if (file_id.lt.0) then
          write(*,*) 'AMTK_openH5 ERROR' ,file_id
          stop
      endif

C     sample meta data read
C     Read Metadata from Metadata file
      open(10, file=meta, status='old')

  555 read(10, 100, end=999) buffer
  100 format(A512)
c
      lines = lines + 1
c
      do i=1,512
          idx(i) = 32
          metadata_name(i) = 32
          max_size(i) = 32
          metadata_value(i) = 32
      end do
c
      i=1
      j=1
   10 if( datax(i).ne.9 ) then
          idx(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 10
      endif
c
      i=i+1
      j=1
   20 if( datax(i).ne.9 ) then
          metadata_name(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 20
      endif
c
      i=i+1
      j=1
   30 if( datax(i).ne.9 ) then
          max_size(j) = datax(i)
          i = i + 1
          j = j + 1
          if (i.ge.512) then
              write(*,*) 'Format ERROR:', meta, ' LINE:', lines
              close(10)
              stop
          endif
          goto 30
      endif
c
      i=i+1
      j=1
   40 if( i.lt.512 .and. datax(i).ne.0 .and. datax(i).ne.9) then
          metadata_value(j) = datax(i)
          i = i + 1
          j = j + 1
          goto 40
      endif

      do i=1,1024
          metadata_area(i) = 32
      end do

      status = AMTK_getMetaDataName(file_id, 
     *    label(2),metadata_area_buff)
      if (status.lt.0) then
          write(*,*) 'AMTK_getMetaDataName ERROR' ,status
          stop
      endif
      if( status.ne.0) then
         write(*,*) 'No=', lines, 'Name=', label(2),
     *    ' Value=',metadata_area_buff(1:status)
      endif

      goto 555

  999 meta_data_num = lines

      Close(10)

C     data set read

C     Geophysical Data (1...3)
C     Layer: ALL -> AM2_GRID_GEOA
      status = AMTK_getDimSize(file_id, AM2_GRID_GEOA, dim_size)
      if (status.lt.0) then
          write(*,*) 'AMTK_getDimSize ERROR' ,status
          stop
      endif

C     get size
C     # dim_size(1) : Number of pixels in X direction(longitude)
C     # dim_size(2) : Number of pixels in Y direction(latitude )
C     # dim_size(3) : Number of Geophysical Data
      x_pixsize = dim_size(1)
      y_pixsize = dim_size(2)
      geo_size = dim_size(3)

      write(*,*) 'Number of pixels in X direction(longitude) = ',
     *    x_pixsize
      write(*,*) 'Number of pixels in Y direction(latitude ) = ',
     *    y_pixsize
      write(*,*) 'Number of Geophysical Data(Layer) = ', geo_size

C     Layer: 1 -> AM2_GRID_GEO1
      if (geo_size.ge.1) then
          status = AMTK_get_GridFloat(file_id, geo_data_1,
     *        AM2_GRID_GEO1)
          if (status.lt.0) then
              write(*,*) 'AMTK_get_GridFloat ERROR' ,status
              stop
          endif
      endif

C     Time Infomation
      status = AMTK_get_GridInt(file_id, time_info, AM2_GRID_TIME)
      if (status.lt.0) then
          write(*,*) 'AMTK_get_GridInt ERROR' ,status
          stop
      endif

C     Latitude of Observation Point
C     Longitude of Observation Point
      status = AMTK_getLatLon(file_id, latlon, AM2_SCANNUM_NOCHK,
     *    AM2_SCANNUM_NOCHK, AM2_GRID_LATLON)
      if (status.lt.0) then
          write(*,*) 'AMTK_getLatLon ERROR' ,status
          stop
      endif

      Do x = 1, PRINT_SCAN_NUM

C         first pixel
          y = 1
          write(*,'(2(A,I4))') 'X = ', x, ' Y = ', y

C         Geophysical Data
          write(*,'(A,F8.3)') 'Geophysical Data (Layer:1) = ',
     *        geo_data_1(y, x)

C         Time Infomation
          write(*,'(A,I4)') 'Time Infomation = ', time_info(y, x)
          
C         Latitude/Longitude of Observation Point
C         # L3LATLONFILE: data(X pixel, Y pixel)
C         # Example) data(1440, 720)
C         # -> X pixel and Y pixel is reversed. 
          AM2_COMMON_LATLON = latlon(x, y)
          write(*,'(2(A,F8.2))') 'Latitude  = ', lat,
     *        '  Longitude = ' , lon

C         last pixel
          y = y_pixsize
          write(*,'(2(A,I4))') 'X = ', x, ' Y = ', y

C         Geophysical Data
          write(*,'(A,F8.3)') 'Geophysical Data (Layer:1) = ',
     *        geo_data_1(y, x)

C         Time Infomation
          write(*,'(A,I4)') 'Time Infomation = ', time_info(y, x)

C         Latitude/Longitude of Observation Point
C         # L3LATLONFILE: data(X pixel, Y pixel)
C         # Example) data(1440, 720)
C         # -> X pixel and Y pixel is reversed. 
          AM2_COMMON_LATLON = latlon(x, y)
          write(*,'(2(A,F8.2))') 'Latitude  = ', lat,
     *        '  Longitude = ' , lon
      End do

C     HDF product file close (write mode)
      status = AMTK_closeH5(file_id)
      if (status.lt.0) then
          write(*,*) 'AMTK_closeH5 ERROR' ,status
          stop
      endif
      
      Write(*,*) 'L3(Day) product read end'

      Stop
      End
      
