/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : AMTK_PLUS.h                                               */
/*  Abstract     : AMTK header file                                          */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/06/14                                                    */
/*  Version    1.10                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#ifndef AMTK_PLUS_H
#define AMTK_PLUS_H

#define INC_HDF
#include "AMTK.h"

#include <hdf5.h>
/* #include "./AMSR_PLUS.h"*/
#ifdef LINUX
#include <byteswap.h>
#endif

#include "./IOTK_common.h"

/* Initial Value when AMTK_setDimSize */
#define AM2_DEF_I_INIT    -32766
#define AM2_DEF_UI_INIT    65533
#define AM2_DEF_C_INIT       253
#define AM2_DEF_R_INIT    -9997.0

/* Abnormal Value */
#define AM2_DEF_I_ABNML   -32767
#define AM2_DEF_UI_ABNML   65534
#define AM2_DEF_C_ABNML      254
#define AM2_DEF_R_ABNML    -9998.0

#define AM2_DEF_SIGNED_INT_ABNML         (-32761.0)
#define AM2_DEF_UNSIGNED_INT_ABNML       ( 65531.0)
#define AM2_DEF_POSITION_IN_ORBIT_ABNML  ( -9991.0)
#define AM2_DEF_LATLON_ABNML             ( -9991.0)
#define AM2_DEF_LAND_OCEAN_FLAG_ABNML    (   251.0)

/* Definition of Metadata. */
#define AM2_DEF_META_PRODUCTNAME         ("ProductName")
#define AM2_DEF_META_OVERLAPSCANS        ("OverlapScans")
#define AM2_DEF_META_GEOPHYSICALNAME     ("GeophysicalName")
#define AM2_DEF_META_GRANULEID           ("GranuleID")
#define AM2_DEF_META_PROJECTION          ("Projection")
#define AM2_DEF_META_ORBITDIRECTION      ("OrbitDirection")
#define AM2_DEF_META_PRODUCTIONDATETIME  ("ProductionDateTime")
#define AM2_DEF_META_COREGISTRATIONA1    ("CoRegistrationParameterA1")
#define AM2_DEF_META_COREGISTRATIONA2    ("CoRegistrationParameterA2")

/* Definition of Metadata value. */
#define PRODUCTNAME_L1A     ("AMSR2-L1A")
#define PRODUCTNAME_L1B     ("AMSR2-L1B")
#define PRODUCTNAME_L1R     ("AMSR2-L1R")
#define PRODUCTNAME_L2      ("AMSR2-L2")
#define PRODUCTNAME_L3      ("AMSR2-L3")

#define GRANULEID_SPD_POS   (18)
#define GRANULEID_SPD_DAY   ('D')
#define GRANULEID_SPD_MON   ('M')
#define GRANULEID_RES_POS   (32)
#define GRANULEID_RES_HI    ('H')
#define GRANULEID_RES_LO    ('L')

/*****************************************************************************/
/*    Acoefficient parameters                                                */
/*****************************************************************************/
typedef struct latlon_coef {
   double a1;		/* a1 */
   double a2;		/* a2 */
} LATLON_COEF ;

/*****************************************************************************/
/*    ACCESS_LABEL table structure                                           */
/*****************************************************************************/
typedef struct
{
	unsigned int     dataset_no;
	unsigned int     HDF_datatype;
	unsigned int     out_datatype1;   /* int      */
	unsigned int     out_datatype2;   /* float    */
	unsigned int     out_datatype3;   /* double   */
	unsigned int     out_datatype4;   /* uchar    */
	unsigned int     dimension;
	unsigned int     dimension1;
	unsigned int     dimension2;
	unsigned int     dimension3;
	unsigned int     HDF_ref_no;
	float            scale_factor;
	char*            p_unit;
}ACCESS_LABEL;

enum{
	DS_L1A_SCAN_TIME,
	DS_L1A_POS_ORBIT,
	DS_L1A_NAVI,
	DS_L1A_ATT,
	DS_L1A_OC06V,
	DS_L1A_OC06H,
	DS_L1A_OC07V,
	DS_L1A_OC07H,
	DS_L1A_OC10V,
	DS_L1A_OC10H,
	DS_L1A_OC18V,
	DS_L1A_OC18H,
	DS_L1A_OC23V,
	DS_L1A_OC23H,
	DS_L1A_OC36V,
	DS_L1A_OC36H,
	DS_L1A_OC89AV,
	DS_L1A_OC89AH,
	DS_L1A_OC89BV,
	DS_L1A_OC89BH,
	DS_L1A_OC_LO,
	DS_L1A_OC_89A,
	DS_L1A_OC_89B,
	DS_L1A_HTS_LO,
	DS_L1A_HTS_HI,
	DS_L1A_CSM_LO,
	DS_L1A_CSM_HI,
	DS_L1A_OFF_GAIN,
	DS_L1A_SUN_AZ,
	DS_L1A_SUN_EL,
	DS_L1A_EARTH_INC,
	DS_L1A_EARTH_AZ,
	DS_L1A_LOF_LO,
	DS_L1A_LOF_HI,
	DS_L1A_OB_SPL,
	DS_L1A_SPC_TEMP,
	DS_L1A_SPS_TEMP,
	DS_L1A_PCD,
	DS_L1A_SCAN_QUAL,
	DS_L1A_PIX_QUAL_LO,
	DS_L1A_PIX_QUAL_HI,
	DS_L1A_INTPL_LO,
	DS_L1A_INTPL_HI,
	DS_L1A_SPL_OV
};

enum{
	DS_L1B_SCAN_TIME,
	DS_L1B_POS_ORBIT,
	DS_L1B_NAVI,
	DS_L1B_ATT,
	DS_L1B_TB06V,
	DS_L1B_TB06H,
	DS_L1B_TB07V,
	DS_L1B_TB07H,
	DS_L1B_TB10V,
	DS_L1B_TB10H,
	DS_L1B_TB18V,
	DS_L1B_TB18H,
	DS_L1B_TB23V,
	DS_L1B_TB23H,
	DS_L1B_TB36V,
	DS_L1B_TB36H,
	DS_L1B_TB89AV,
	DS_L1B_TB89AH,
	DS_L1B_TB89BV,
	DS_L1B_TB89BH,
	DS_L1B_TB_LO,
	DS_L1B_TB_89A,
	DS_L1B_TB_89B,
	DS_L1B_HTS_LO,
	DS_L1B_HTS_HI,
	DS_L1B_CSM_LO,
	DS_L1B_CSM_HI,
	DS_L1B_OFF_GAIN,
	DS_L1B_SUN_AZ,
	DS_L1B_SUN_EL,
	DS_L1B_EARTH_INC,
	DS_L1B_EARTH_AZ,
	DS_L1B_LOF_LO,
	DS_L1B_LOF_HI,
	DS_L1B_OB_SPL,
	DS_L1B_SPC_TEMP,
	DS_L1B_SPS_TEMP,
	DS_L1B_PCD,
	DS_L1B_SCAN_QUAL,
	DS_L1B_PIX_QUAL_LO,
	DS_L1B_PIX_QUAL_HI,
	DS_L1B_INTPL_LO,
	DS_L1B_INTPL_HI,
	DS_L1B_SPL_OV
};

enum{
	DS_L1R_SCAN_TIME,
	DS_L1R_POS_ORBIT,
	DS_L1R_NAVI,
	DS_L1R_ATT,
	DS_L1R_RES06_TB06V,
	DS_L1R_RES06_TB06H,
	DS_L1R_RES06_TB07V,
	DS_L1R_RES06_TB07H,
	DS_L1R_RES06_TB10V,
	DS_L1R_RES06_TB10H,
	DS_L1R_RES06_TB18V,
	DS_L1R_RES06_TB18H,
	DS_L1R_RES06_TB23V,
	DS_L1R_RES06_TB23H,
	DS_L1R_RES06_TB36V,
	DS_L1R_RES06_TB36H,
	DS_L1R_RES06_TB89V,
	DS_L1R_RES06_TB89H,
	DS_L1R_RES06_TB_ALL,
	DS_L1R_RES10_TB10V,
	DS_L1R_RES10_TB10H,
	DS_L1R_RES10_TB18V,
	DS_L1R_RES10_TB18H,
	DS_L1R_RES10_TB23V,
	DS_L1R_RES10_TB23H,
	DS_L1R_RES10_TB36V,
	DS_L1R_RES10_TB36H,
	DS_L1R_RES10_TB89V,
	DS_L1R_RES10_TB89H,
	DS_L1R_RES10_TB_ALL,
	DS_L1R_RES23_TB18V,
	DS_L1R_RES23_TB18H,
	DS_L1R_RES23_TB23V,
	DS_L1R_RES23_TB23H,
	DS_L1R_RES23_TB36V,
	DS_L1R_RES23_TB36H,
	DS_L1R_RES23_TB89V,
	DS_L1R_RES23_TB89H,
	DS_L1R_RES23_TB_ALL,
	DS_L1R_RES36_TB36V,
	DS_L1R_RES36_TB36H,
	DS_L1R_RES36_TB89V,
	DS_L1R_RES36_TB89H,
	DS_L1R_RES36_TB_ALL,
	DS_L1R_TB89AV,
	DS_L1R_TB89AH,
	DS_L1R_TB89BV,
	DS_L1R_TB89BH,
	DS_L1R_TB_89A,
	DS_L1R_TB_89B,
	DS_L1R_MEAN_HEIGHT,
	DS_L1R_SUN_AZ,
	DS_L1R_SUN_EL,
	DS_L1R_EARTH_INC,
	DS_L1R_EARTH_AZ,
	DS_L1R_LOF_RES_LO,
	DS_L1R_LOF_RES_HI,
	DS_L1R_SCAN_QUAL,
	DS_L1R_PIX_QUAL_LO,
	DS_L1R_PIX_QUAL_HI
};

enum{
	DS_L2_SCAN_TIME,
	DS_L2_POS_ORBIT,
	DS_L2_SWATH_GEO1,
	DS_L2_SWATH_GEO2,
	DS_L2_SWATH_GEO3,
	DS_L2_SWATH_GEOA,
	DS_L2_PIX_QUAL,
	DS_L2_SWATHA_GEO1,
	DS_L2_SWATHA_GEO2,
	DS_L2_SWATHA_GEO3,
	DS_L2_SWATHA_GEOA,
	DS_L2_SWATHB_GEO1,
	DS_L2_SWATHB_GEO2,
	DS_L2_SWATHB_GEO3,
	DS_L2_SWATHB_GEOA,
	DS_L2_PIX_QUAL_A,
	DS_L2_PIX_QUAL_B
};

enum{
	DS_L3DH_GRID_TBV,
	DS_L3DH_GRID_TBH,
	DS_L3DH_GRID_TB,
	DS_L3DH_GRID_TIME,
	DS_L3DH_GRID_GEO1,
	DS_L3DH_GRID_GEO2,
	DS_L3DH_GRID_GEO3,
	DS_L3DH_GRID_GEOA
};

enum{
	DS_L3DL_GRID_TBV,
	DS_L3DL_GRID_TBH,
	DS_L3DL_GRID_TB,
	DS_L3DL_GRID_TIME,
	DS_L3DL_GRID_GEO1,
	DS_L3DL_GRID_GEO2,
	DS_L3DL_GRID_GEO3,
	DS_L3DL_GRID_GEOA
};

enum{
	DS_L3MH_GRID_TBV,
	DS_L3MH_GRID_TBH,
	DS_L3MH_GRID_TB,
	DS_L3MH_GRID_TBV_STD,
	DS_L3MH_GRID_TBH_STD,
	DS_L3MH_GRID_TB_STD,
	DS_L3MH_GRID_TBV_ANUM,
	DS_L3MH_GRID_TBH_ANUM,
	DS_L3MH_GRID_TB_ANUM,
	DS_L3MH_GRID_TBV_TNUM,
	DS_L3MH_GRID_TBH_TNUM,
	DS_L3MH_GRID_TB_TNUM,
	DS_L3MH_GRID_GEO1,
	DS_L3MH_GRID_GEO2,
	DS_L3MH_GRID_GEO3,
	DS_L3MH_GRID_GEOA,
	DS_L3MH_GRID_GEO1_STD,
	DS_L3MH_GRID_GEO2_STD,
	DS_L3MH_GRID_GEO3_STD,
	DS_L3MH_GRID_GEOA_STD,
	DS_L3MH_GRID_GEO1_ANUM,
	DS_L3MH_GRID_GEO2_ANUM,
	DS_L3MH_GRID_GEO3_ANUM,
	DS_L3MH_GRID_GEOA_ANUM,
	DS_L3MH_GRID_GEO1_TNUM,
	DS_L3MH_GRID_GEO2_TNUM,
	DS_L3MH_GRID_GEO3_TNUM,
	DS_L3MH_GRID_GEOA_TNUM
};

enum{
	DS_L3ML_GRID_TBV,
	DS_L3ML_GRID_TBH,
	DS_L3ML_GRID_TB,
	DS_L3ML_GRID_TBV_STD,
	DS_L3ML_GRID_TBH_STD,
	DS_L3ML_GRID_TB_STD,
	DS_L3ML_GRID_TBV_ANUM,
	DS_L3ML_GRID_TBH_ANUM,
	DS_L3ML_GRID_TB_ANUM,
	DS_L3ML_GRID_TBV_TNUM,
	DS_L3ML_GRID_TBH_TNUM,
	DS_L3ML_GRID_TB_TNUM,
	DS_L3ML_GRID_GEO1,
	DS_L3ML_GRID_GEO2,
	DS_L3ML_GRID_GEO3,
	DS_L3ML_GRID_GEOA,
	DS_L3ML_GRID_GEO1_STD,
	DS_L3ML_GRID_GEO2_STD,
	DS_L3ML_GRID_GEO3_STD,
	DS_L3ML_GRID_GEOA_STD,
	DS_L3ML_GRID_GEO1_ANUM,
	DS_L3ML_GRID_GEO2_ANUM,
	DS_L3ML_GRID_GEO3_ANUM,
	DS_L3ML_GRID_GEOA_ANUM,
	DS_L3ML_GRID_GEO1_TNUM,
	DS_L3ML_GRID_GEO2_TNUM,
	DS_L3ML_GRID_GEO3_TNUM,
	DS_L3ML_GRID_GEOA_TNUM
};

enum{
	DS_LATLON_06,
	DS_LATLON_07,
	DS_LATLON_10,
	DS_LATLON_18,
	DS_LATLON_23,
	DS_LATLON_36,
	DS_LATLON_89A,
	DS_LATLON_89B,
	DS_LATLON_LO,
	DS_LATLON_RS_89A,
	DS_LATLON_RS_89B,
	DS_LATLON_RS_LO,
	DS_LATLON_L2_LO,
	DS_LATLON_L2_89A,
	DS_LATLON_L2_89B,
	DS_GRID_LATLON
};

#define INT 1
#define OUT1 1
#define OUT2 2
#define AM2_ 1
#define NATIVE_DOUBLE 1
#define NATIVE_FLOAT  2
#define NATIVE_INT    3
#define NATIVE_UCHAR  4

#define OFF 0
#define ON  1
#define AM2_ 1

#define H5U_STD_I16LE	1	/* int */
#define H5U_STD_U16LE	2	/* unsigned int */
#define H5U_IEEE_F32LE	3	/* float */
#define H5U_IEEE_F64LE	4	/* double */
#define H5U_STD_I8LE	5	/* signed char */
#define H5U_STD_U8LE	6	/* unsigned char */
#define H5U_STD_B8LE	7	/* (NATIVE_UCHAR) */
#define H5U_STD_B16LE	8	/* (NATIVE_UCHAR) */
#define H5U_STD_B64LE	9   /* (NATIVE_UCHAR) */
#define H5U_STD_B512LE	10	/* (UINT) */

enum{
	DS_89A_LAT,
	DS_89A_LON
};
enum{
	DS_89B_LAT,
	DS_89B_LON
};

/*****************************************************************************/
/*    LeapSec                                                                */
/*****************************************************************************/
typedef struct leapsec {
    int     iYear;
    int     i_Month;
    int     i_Day;
    double  d_Delat;
    double  drift[2];
    double  just_Sec;
} LEAPSEC;

#define IYV (2009)
#define NERA1 (14)

/*****************************************************************************/
/*    HDF                                                                    */
/*****************************************************************************/
#define	AM2_GETHDFDATA_MAX	15

typedef struct
{
	int		dataset_no;
	int		datasetsize;
	int		datasize;
	int		rank;
	int		ntype;
	int     size[3];
} HDFDATA;

typedef struct
{
	int		datacnt;
	HDFDATA	data[AM2_GETHDFDATA_MAX];
} GET_HDFDATA;

/*****************************************************************************/
/*    Function prototypes                                                    */
/*****************************************************************************/

ACCESS_LABEL *search_table(hid_t HDF_file_id, int dataset_no, int *table_num);
int AccessLabelTypeCheck(hid_t HDF_file_id, unsigned int  datatype, int dataset_no);
int getScaleFactor(hid_t dataset,float *scale);
int setUnit(hid_t dataset, char* unit);
int getL3Scalefactor(hid_t HDF_file_id, int dataset_no, float* scale, char* unit);
int getGeophysicalInfo(char* geophysicalName,  float* scale, char* unit);
int checkWriteNG(int dataset_no);

/* For latlon */
int get_coef(hid_t *HDF_file_id, LATLON_COEF COEF[]);

void checkByteOrder( unsigned char* org_value, unsigned char* check_value, int isLittleEndian);

/*************************************************************/
int IOTKprintf(const char *form, ...);
int CommonPrmCheck(hid_t HDF_file_id, int from_scan, int to_scan, int dataset_no);
int getAttribute(hid_t attr, char **value);
int getDataSet(hid_t HDF_file_id, void **data, int from_scan, int to_scan, int dataset_no, int datatype);
int setDataSet(hid_t HDF_file_id, void *data, int from_scan, int to_scan, int dataset_no, int datatype);
int getMemorySize(int *size, int rank, int datasize, int from_scan, int to_scan);
void *getMemoryAlloc(int *size, int rank, int datasize, int from_scan, int to_scan, int *getmemsize);
int setScaleFactor(hid_t dataset, float value);
int getMetaDataInside(hid_t HDF_file_id, char *p_name, char **p_value, int open_err_no, int hdf_err_no);
int getOverLapScans(hid_t HDF_file_id, int *overlapscan);
int setHyperslab(hid_t dataset, int from_scan, int to_scan, int	*size, int rank, int overlap, hid_t *memspace, hid_t *filespace);
int getDimSize(hid_t dataset, int *size);
int getLatLon(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no);
int setLatLon(hid_t HDF_file_id, AM2_COMMON_LATLON *latlon, int from_scan, int to_scan, int dataset_no);
int getRegistration(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no);
int getLowlatlonAverage(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no);
int getResamplingPoint(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no);
int getL3LatLon(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon);
int getLeapSec(LEAPSEC	*t_LeapSec, int year, int month, double *deltat );
int readLeapSec(LEAPSEC *LeapSec);
void checkLeapSecDate(LEAPSEC *t_LeapSec, AM2_COMMON_SCANTIME *scan_time, double tai93sec);
int	getL3LatLonFileName(hid_t HDF_file_id, char *fname);
int getL3LatLonSize(char *fname, int *size);
int getGridDataSet(hid_t HDF_file_id, void **data, int from_scan, int to_scan, int dataset_no, int datatype);
int setGridDataSet(hid_t HDF_file_id, void *data, int from_scan, int to_scan, int dataset_no, int datatype);
int checkWriteNG(int dataset_no);
void checkAccessLabel(int dataset_no, GET_HDFDATA *hdfdata);
char *string_check_fortran(char* original, int length);
int	setGridHyperslab(hid_t dataset, int from_scan, int to_scan, int overlap, int rank, int getGrid, hid_t *memspace, hid_t *filespace);

#endif
