/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : amtk_set.c                                                */
/*  Abstract     : I/O toolkit get function                                  */
/*  Function List                                                            */
/*      AMTK_openH5_Write()          : HDF file open funtion for write       */
/*      AMTK_closeH5_Write()         : HDF file close funtion for write      */
/*      AMTK_setDimSize()            : Set HDF dataset dimension size        */
/*      AMTK_setMetaDataName()       : Set HDF Meta Data by Name             */
/*      AMTK_set_SwathInt()          : Set HDF dataset integer value         */
/*      AMTK_set_SwathFloat()        : Set HDF dataset float value           */
/*      AMTK_set_SwathDouble()       : Set HDF dataset double value          */
/*      AMTK_set_SwathUChar()        : Set HDF dataset unsigned char value   */
/*      AMTK_set_GridInt()           : Set HDF dataset integer value in grid */
/*      AMTK_set_GridFloat()         : Set HDF dataset float value in grid   */
/*      AMTK_set_ScanDataQuality()   : Set HDF Scan Data Quality Structure   */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/06/13                                                    */
/*  Version    1.10                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "./AMTK_PLUS.h"

extern char **name_table[];
extern char *latlon_name_lo[];
extern char *latlon_name_89a[];
extern char *latlon_name_89b[];

/** @file amtk_set.c
 * HDF file open funtion for write
 *
 * @author     NEC
 * @date       2010/06/30
 */
hid_t AMTK_openH5_Write(char *file_name, int mode)
{
	hid_t   file_id = 0;
	/******* Mute HDF error  ******/
	hid_t error_stack = H5E_DEFAULT;
	H5Eset_auto2(error_stack, NULL, NULL);
	/******* Mute HDF error  end******/

	/* DEBUG FLAG */
	IOTK_DebugCheck();
	/* DEBUG FLAG */

	if( mode == AM2_CREATE_MODE )
	{
		file_id = H5Fcreate(file_name, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
		if(file_id == FAIL)
		{
			return ERROR_HDF_FILE_OPEN;
		}
	}else if(mode == AM2_RW_MODE )
	{
		file_id = H5Fopen(file_name, H5F_ACC_RDWR, H5P_DEFAULT);
		if(file_id == FAIL)
		{
			return ERROR_HDF_FILE_OPEN;
		}
	}else
	{
			return ERROR_ILLEGAL_MODE;
	}
	return file_id;
}
hid_t amtk_openh5_write_(char *file_name, int *mode, int length)
{
    hid_t hDF_file_id = 0;
    char* p_string = NULL;

    p_string = string_check_fortran(file_name, length);
    hDF_file_id =  AMTK_openH5_Write(p_string, *mode);
    free(p_string);
    return hDF_file_id;
}
/* For g77 entry */
hid_t amtk_openh5_write__(char *file_name, int *mode, int length)
{
    return amtk_openh5_write_(file_name, mode, length);
}

/** @file amtk_set.c
 * HDF file close funtion for write
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_closeH5_Write(hid_t HDF_file_id)
{
	if( H5Fclose(HDF_file_id)==FAIL )
	{
		return ERROR_HDF_FILE_CLOSE;
	}
	return SUCCESS;
}
int amtk_closeh5_write_(hid_t *HDF_file_id)
{
    return AMTK_closeH5_Write(*HDF_file_id);
}
/* For g77 entry */
int amtk_closeh5_write__(hid_t *HDF_file_id)
{
    return amtk_closeh5_write_(HDF_file_id);
}

/** @file amtk_set.c
 * Set HDF dimension size
 *
 * @author     NEC
 * @date       2011/06/13
 */
int AMTK_setDimSize(hid_t HDF_file_id,int dataset_no , int *size)
{
	int				ret = 0;
	int				cnt = 0;
	int				loop = 0;
	int				rank = 0;
	int				datasize = 0;
	int				alldata = 0;
	int				table_num = 0;
	int				getmemsize = 0;
	ACCESS_LABEL	*p_table=NULL;
	hid_t			space = 0;
	hid_t			dataset = 0;
	void			*p_data = NULL;
	hid_t			ntype = 0;
	hsize_t			dsize[3];
	hid_t			datatype = 0;
	int 			setCnt = 0;
	char			setName[2][513];
	char			wrtflg = 0;
	float			scale = 0.0;
	char			p_unit[11];

	ret = CommonPrmCheck(HDF_file_id, AM2_SCANNUM_NOCHK, AM2_SCANNUM_NOCHK, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	if((size[0] == 0) && (size[1] == 0) && (size[2] == 0))
	{
		return ERROR_ILLEGAL_DIMS;
	}
	if(size[2] != 0)
	{
		if((size[0] == 0) || (size[1] == 0))
		{
			return ERROR_ILLEGAL_DIMS;
		}
		rank = 3;
	}
	else if(size[1] != 0)
	{
		if(size[0] == 0)
		{
			return ERROR_ILLEGAL_DIMS;
		}
		rank =2;
	} else {
		rank = 1;
	}

	dsize[0] = (hsize_t)size[0];
	dsize[1] = (hsize_t)size[1];
	dsize[2] = (hsize_t)size[2];

    p_table = search_table(HDF_file_id, dataset_no, &table_num);
	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	memset(&setName[0][0], 0x00, 513);
	memset(&setName[1][0], 0x00, 513);

	if(dataset_no == AM2_LATLON_L2_LO)
	{
		strcpy(&setName[0][0], latlon_name_lo[0]);
		strcpy(&setName[1][0], latlon_name_lo[1]);
		setCnt=2;
	}
	else if( (dataset_no == AM2_LATLON_89A)
		|| (dataset_no == AM2_LATLON_RS_89A)
		|| (dataset_no == AM2_LATLON_L2_89A) )
	{
		strcpy(&setName[0][0], latlon_name_89a[0]);
		strcpy(&setName[1][0], latlon_name_89a[1]);
		setCnt=2;
	}
	else if( (dataset_no == AM2_LATLON_89B)
		|| (dataset_no == AM2_LATLON_RS_89B)
		|| (dataset_no == AM2_LATLON_L2_89B) )
	{
		strcpy(&setName[0][0], latlon_name_89b[0]);
		strcpy(&setName[1][0], latlon_name_89b[1]);
		setCnt=2;
	}
	else
	{
		/* WriteNG */
		ret = checkWriteNG(dataset_no);
		if(ret == ERROR_ACCESSLABEL_WRITE_NG)
		{
			return ret;
		}
		strcpy(&setName[0][0], name_table[table_num][p_table->dataset_no]);
		setCnt=1;
	}
	for(loop = 0; loop < setCnt; loop++)
	{
		dataset  = H5Dopen (HDF_file_id, setName[loop], H5P_DEFAULT);
		if(dataset < 0)
		{
			space = H5Screate_simple (rank, &dsize[0], NULL);
			if(space < 0)
			{
				IOTK_printf("setDimSize ERROR - 1[%s]\n",setName[loop]);
				return ERROR_HDF_CREATE_SPACE;
			}
			switch(p_table->HDF_datatype)
			{
				case H5U_STD_I16LE:
					datatype = H5T_STD_I16LE;
					break;
				case H5U_STD_U16LE:
					datatype = H5T_STD_U16LE;
					break;
				case H5U_IEEE_F32LE:
					datatype = H5T_IEEE_F32LE;
					break;
				case H5U_IEEE_F64LE:
					datatype = H5T_IEEE_F64LE;
					break;
				case H5U_STD_I8LE:
					datatype = H5T_STD_I8LE;
					break;
				case H5U_STD_U8LE:
				default:
					datatype = H5T_STD_U8LE;
					break;
			}
			dataset = H5Dcreate (HDF_file_id, setName[loop], 
								datatype, space, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
			if(dataset < 0)
			{
				/* close dataspace */
				H5Sclose(space);
				return ERROR_HDF_DATASET_CREATE;
			}
		}
		else
		{
			/* close dataset */
			H5Dclose(dataset);
			return SUCCESS;
		}

		datasize = 0;
		switch(p_table->HDF_datatype)
		{
			case H5U_STD_I16LE:
				datasize = sizeof(short);
				ntype = H5T_NATIVE_SHORT;
				wrtflg = 1;
				break;
			case H5U_STD_U16LE:
				datasize = sizeof(unsigned short);
				ntype = H5T_NATIVE_USHORT;
				wrtflg = 5;
				break;
			case H5U_IEEE_F32LE:
				datasize = sizeof(float);
				ntype = H5T_NATIVE_FLOAT;
				wrtflg = 2;
				break;
			case H5U_IEEE_F64LE:
				datasize = sizeof(double);
				ntype = H5T_NATIVE_DOUBLE;
				wrtflg = 3;
				break;
			case H5U_STD_I8LE:
				datasize = sizeof(char);
				ntype = H5T_NATIVE_SCHAR;
				wrtflg = 4;
				break;
			case H5U_STD_U8LE:
			default:
				datasize = sizeof(unsigned char);
				ntype = H5T_NATIVE_UCHAR;
				wrtflg = 4;
				break;
		}

		p_data  = (void *)getMemoryAlloc(size, rank, datasize, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, &getmemsize);
		if(p_data == NULL)
		{
			/* close dataspace */
			H5Sclose(space);
			/* close dataset */
			H5Dclose(dataset);
			return	ERROR_MALLOC;
		}

		alldata = getmemsize/datasize;

		if(wrtflg == 1)            /* SHORT */
		{
			for(cnt = 0; cnt < alldata; cnt++)
			{
				*((short *)p_data+cnt) = AM2_DEF_I_INIT;
			}
		}
		else if(wrtflg == 2)       /* FLOAT */
		{
			for(cnt = 0; cnt < alldata; cnt++)
			{
				*((float *)p_data+cnt) = AM2_DEF_R_INIT;
			}
		}
		else if(wrtflg == 3)       /* DOUBLE */
		{
			for(cnt = 0; cnt < alldata; cnt++)
			{
				*((double *)p_data+cnt) = AM2_DEF_R_INIT;
			}
		}
		else if(wrtflg == 5)       /* USHORT */
		{
			for(cnt = 0; cnt < alldata; cnt++)
			{
				*((unsigned short *)p_data+cnt) = AM2_DEF_UI_INIT;
			}
		}
		else                       /* CHAR   */
		{
			for(cnt = 0; cnt < alldata; cnt++)
			{
				*((unsigned char *)p_data+cnt) = AM2_DEF_C_INIT;
			}
		}

		ret = H5Dwrite (dataset, ntype, space, H5S_ALL, H5P_DEFAULT, p_data);
		
		/* close dataspace */
		H5Sclose(space);
		
		if(ret < 0)
		{
			free(p_data);
			/* close dataset */
			H5Dclose(dataset);
			return	ERROR_HDF_DATASET_WRITE;
		}

		/* L2,L3 Scale factor depends on Geophysical data */
      if((dataset_no == AM2_SWATH_GEO1)
		|| (dataset_no == AM2_SWATH_GEO2)
		|| (dataset_no == AM2_SWATH_GEO3)
		|| (dataset_no == AM2_SWATH_GEOA)
		|| (dataset_no == AM2_SWATHA_GEO1)
		|| (dataset_no == AM2_SWATHA_GEO2)
		|| (dataset_no == AM2_SWATHA_GEO3)
		|| (dataset_no == AM2_SWATHA_GEOA)
		|| (dataset_no == AM2_SWATHB_GEO1)
		|| (dataset_no == AM2_SWATHB_GEO2)
		|| (dataset_no == AM2_SWATHB_GEO3)
		|| (dataset_no == AM2_SWATHB_GEOA)
		|| (dataset_no == AM2_GRID_GEO1)
		|| (dataset_no == AM2_GRID_GEO2)
		|| (dataset_no == AM2_GRID_GEO3)
		|| (dataset_no == AM2_GRID_GEOA) )
		{
			ret = getL3Scalefactor(HDF_file_id, dataset_no, &scale, &p_unit[0]);
			if(ret < 0)
			{
			    free(p_data);
				/* close dataset */
			    H5Dclose(dataset);
			    return	ret;
			}
		}else
		{
			scale = p_table->scale_factor;
			strcpy(p_unit , p_table->p_unit);
		}

		ret = setScaleFactor(dataset, scale);
		if(ret < 0)
		{
			free(p_data);
			/* close dataset */
			H5Dclose(dataset);
			return	ret;
		}
		ret = setUnit(dataset, p_unit);
		if(ret < 0)
		{
			free(p_data);
			/* close dataset */
			H5Dclose(dataset);
			return	ret;
		}
		/* close dataset */
		H5Dclose(dataset);

		free(p_data);
		p_data = NULL;
	}
	return SUCCESS;
}
int amtk_setdimsize_(hid_t *HDF_file_id,int *dataset_no , int *size)
{
    return AMTK_setDimSize(*HDF_file_id,*dataset_no , size);
}
/* For g77 entry */
int amtk_setdimsize__(hid_t *HDF_file_id,int *dataset_no , int *size)
{
    return AMTK_setDimSize(*HDF_file_id,*dataset_no , size);
}

/** @file amtk_set.c
 * Set HDF meta data name
 *
 * @author     NEC
 * @date       2011/06/13
 */
int AMTK_setMetaDataName(hid_t HDF_file_id, char *name, char *value)
{
	hid_t	attr = 0, memtype = 0, filetype = 0;  /* Attribute, datatype, dataspace identifiers */
	herr_t	ret = 0;
	hsize_t	hsize[1];
	hid_t	space = 0;
	char *p_value[1] = {NULL};

	if(HDF_file_id == 0)
	{
		return ERROR_ILLEGAL_HDF_FILEID;
	}

	if(name == NULL)
	{
		return ERROR_NULL_ADDRESS;
	}

	if(value == NULL)
	{
		return ERROR_NULL_ADDRESS;
	}
	
	/*  Open the attribute using its name.  */
	attr = H5Aopen_name(HDF_file_id, name);
	if (attr >= 0)
	{
		/* If meta data exist, delete the meta data.
		 HDF does not allow overwrite */
		
		/* Get String datatype. */
		memtype = H5Tcopy(H5T_C_S1);
		if (memtype < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 1\n");
			H5Aclose(attr);
			return ERROR_HDF_DT_COPY;
		}
		
		/* Reset datatype size. */
		ret = H5Tset_size(memtype, H5T_VARIABLE);
		if (ret < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 2\n");
			/* Close Datatype. */
			H5Tclose(memtype);
			/* Close Attribute. */
			H5Aclose(attr);
			return ERROR_HDF_DT_SET_SIZE;
		}
	}
	else
	{
		/* Create New Meta data. */
		attr  = 0;
		
		/* Get String datatype. */
		filetype = H5Tcopy(H5T_C_S1);
		if (filetype < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 1\n");
			return ERROR_HDF_DT_COPY;
		}
		
		/* Get String datatype. */
		memtype = H5Tcopy(H5T_C_S1);
		if (memtype < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 1\n");
			/* Close Datatype. */
			H5Tclose(filetype);
			return ERROR_HDF_DT_COPY;
		}

		/* Set new datatype size. */
		ret = H5Tset_size (filetype, H5T_VARIABLE);
		if (ret < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 2\n");
			/* Close Datatype. */
			H5Tclose(filetype);
			H5Tclose(memtype);
			return ERROR_HDF_DT_SET_SIZE;
		}
		
		/* Set new datatype size. */
		ret = H5Tset_size (memtype, H5T_VARIABLE);
		if (ret < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 2\n");
			/* Close Datatype. */
			H5Tclose(filetype);
			H5Tclose(memtype);
			return ERROR_HDF_DT_SET_SIZE;
		}
		
		/* Create dataspace. */
		hsize[0] = 1;
		space = H5Screate_simple (1, hsize, NULL);
		if(space < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 2\n");
			/* Close Datatype. */
			H5Tclose(filetype);
			H5Tclose(memtype);
			return ERROR_HDF_CREATE_SPACE;
		}
		
		/* Create attribute. */
		attr = H5Acreate (HDF_file_id, name, filetype, space, H5P_DEFAULT, H5P_DEFAULT);
		if (attr < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 3\n");
			/* Close Datatype. */
			H5Tclose(filetype);
			H5Tclose(memtype);
			/* Close Dataspace. */
			H5Sclose(space);
			return ERROR_HDF_ATTR_CREATE;
		}
		
		/* Close Dataspace. */
		H5Sclose(space);
	}

	if (H5T_STRING == H5Tget_class(memtype))
	{
		p_value[0] = value;
		ret = H5Awrite(attr, memtype, p_value);
		if (ret < 0)
		{
			IOTK_printf("setMetaDataName ERROR - 6\n");
			/* Close Datatype. */
			H5Tclose(memtype);
			H5Tclose(filetype);
			/* Close Attribute. */
			H5Aclose(attr);
			return ERROR_HDF_ATTR_WRITE;
		}
	}
	else
	{
		/* Close Datatype. */
		H5Tclose(memtype);
		H5Tclose(filetype);
		/* Close Attribute. */
		H5Aclose(attr);
		return ERROR_HDF_DT_GET_CLASS;
	}

	/* Close Datatype. */
	H5Tclose(memtype);
	H5Tclose(filetype);
	/* Close Attribute. */
	H5Aclose(attr);
	return SUCCESS;
}
int amtk_setmetadataname_(hid_t *HDF_file_id, char *name_org, char *value_org, int name_length, int value_length){
    char* p_name = NULL;
    char* p_value = NULL;
    int   ret = 0;

    p_name = string_check_fortran(name_org, name_length);
    p_value = string_check_fortran(value_org, value_length);
    ret = AMTK_setMetaDataName(*HDF_file_id, p_name, p_value);
    free( p_name );
    free( p_value );
    return ret;
}
/* For g77 entry */
int amtk_setmetadataname__(hid_t *HDF_file_id, char *name_org, char *value_org, int name_length, int value_length)
{
    return amtk_setmetadataname_(HDF_file_id, name_org, value_org, name_length, value_length);
}

/** @file amtk_set.c
 * Set HDF meta data
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_setMetaData(hid_t HDF_file_id, int index, char *value)
{
	hid_t	attr = 0, atype = 0;  /* Attribute, datatype, dataspace identifiers */
	herr_t	ret = 0;
	

	if(HDF_file_id == 0)
	{
		return ERROR_ILLEGAL_HDF_FILEID;
	}

	if(index == 0)
	{
		return ERROR_ILLEGAL_INDEX;
	}

	if(value == NULL)
	{
		return ERROR_NULL_ADDRESS;
	}

	/*  Open the attribute using its name.  */    
	attr = H5Aopen_idx(HDF_file_id, index);
	if( attr < 0)
	{
		IOTK_printf("setMetaData Open ERROR \n");
		return ERROR_HDF_ATTR_IDX_OPEN;
	}

    /* Get attribute datatype, dataspace, rank, and dimensions.  */
	atype  = H5Aget_type(attr);
	if( atype < 0)
	{
		IOTK_printf("setMetaData ERROR - 1\n");
		H5Aclose(attr);
		return ERROR_HDF_ATTR_GET_TYPE;
	}

	if (H5T_STRING == H5Tget_class (atype)) 
	{
		ret = H5Awrite(attr, atype, value);
		if(ret<0)
		{
			IOTK_printf("setMetaData ERROR - 3\n");
			H5Tclose(atype);
			H5Aclose(attr);
			return ERROR_HDF_ATTR_WRITE;
		}
	}
	else
	{
		IOTK_printf("setMetaData ERROR - 4\n");
		H5Tclose(atype);
		H5Aclose(attr);
		return ERROR_HDF_DT_GET_CLASS;
	}

	H5Tclose(atype);
	H5Aclose(attr);

	return SUCCESS;
}
int amtk_setmetadata_(hid_t *HDF_file_id, int *index, char *value)
{
    return AMTK_setMetaData(*HDF_file_id, *index, value);
}
/* For g77 entry */
int amtk_setmetadata__(hid_t *HDF_file_id, int *index, char *value)
{
    return amtk_setmetadata_(HDF_file_id, index, value);
}

/** @file amtk_set.c
 * Set HDF integer value
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_set_SwathInt(hid_t HDF_file_id, int *data, int from_scan, int to_scan, int dataset_no)
{
	int	ret = 0;

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	ret = checkWriteNG(dataset_no);
	if(ret == ERROR_ACCESSLABEL_WRITE_NG)
	{
		return ret;
	}

	if(data == NULL){
		return ERROR_NOT_WRITEDATA;
	}

	ret = setDataSet(HDF_file_id, data, from_scan, to_scan, dataset_no, AM2_DTYPE_INT);
	if(ret != SUCCESS)
	{
		return ret;
	}
	return SUCCESS;
}
int amtk_set_swathint_(hid_t *HDF_file_id, int *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return AMTK_set_SwathInt(*HDF_file_id, data, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_set_swathint__(hid_t *HDF_file_id, int *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return amtk_set_swathint_(HDF_file_id, data, from_scan, to_scan, dataset_no);
}

/** @file amtk_set.c
 * Set HDF float vale
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_set_SwathFloat(hid_t HDF_file_id, float *data, int from_scan, int to_scan, int dataset_no)
{
	int	ret = 0;

	if(data == NULL)
	{
		return ERROR_NOT_WRITEDATA;
	}

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	ret = checkWriteNG(dataset_no);
	if(ret == ERROR_ACCESSLABEL_WRITE_NG)
	{
		return ret;
	}

	ret = setDataSet(HDF_file_id, data, from_scan, to_scan, dataset_no, AM2_DTYPE_FLOAT);
	if(ret != SUCCESS)
	{
		return ret;
	}
	return SUCCESS;
}
int amtk_set_swathfloat_(hid_t *HDF_file_id, float *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return AMTK_set_SwathFloat(*HDF_file_id, data, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_set_swathfloat__(hid_t *HDF_file_id, float *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return amtk_set_swathfloat_(HDF_file_id, data, from_scan, to_scan, dataset_no);
}

/** @file amtk_set.c
 * Set HDF double value
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_set_SwathDouble(hid_t HDF_file_id, double *data, int from_scan, int to_scan, int dataset_no)
{
	int	ret = 0;

	if(data == NULL)
	{
		return ERROR_NOT_WRITEDATA;
	}

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	ret = checkWriteNG(dataset_no);
	if(ret == ERROR_ACCESSLABEL_WRITE_NG)
	{
		return ret;
	}

	ret = setDataSet(HDF_file_id, data, from_scan, to_scan, dataset_no, AM2_DTYPE_DOUBLE);
	if(ret != SUCCESS)
	{
		return ret;
	}
	return SUCCESS;
}
int amtk_set_swathdouble_(hid_t *HDF_file_id, double *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return AMTK_set_SwathDouble(*HDF_file_id, data, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_set_swathdouble__(hid_t *HDF_file_id, double *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return amtk_set_swathdouble_(HDF_file_id, data, from_scan, to_scan, dataset_no);
}

/** @file amtk_set.c
 * Set HDF unsigned char value
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_set_SwathUChar(hid_t HDF_file_id, unsigned char *data, int from_scan, int to_scan, int dataset_no)
{
	int	ret = 0;

	if(data == NULL){
		return ERROR_NOT_WRITEDATA;
	}

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	ret = checkWriteNG(dataset_no);
	if(ret == ERROR_ACCESSLABEL_WRITE_NG)
	{
		return ret;
	}

	ret = setDataSet(HDF_file_id, data, from_scan, to_scan, dataset_no, AM2_DTYPE_UCHAR);
	if(ret != SUCCESS){
		return ret;
	}
	return SUCCESS;
}
int amtk_set_swathuchar_(hid_t *HDF_file_id, unsigned char *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return AMTK_set_SwathUChar(*HDF_file_id, data,  *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_set_swathuchar__(hid_t *HDF_file_id, unsigned char *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return amtk_set_swathuchar_(HDF_file_id, data, from_scan, to_scan, dataset_no);
}

/** @file amtk_set.c
 * Set HDF integer in grid
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_set_GridInt(hid_t HDF_file_id, int *data, int dataset_no)
{
	int ret = 0;
	int	gridFlg = IOTK_OFF;

	if(data == NULL)
	{
		return ERROR_NOT_WRITEDATA;
	}

	ret = CommonPrmCheck(HDF_file_id, AM2_SCANNUM_NOCHK, AM2_SCANNUM_NOCHK, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	ret = checkWriteNG(dataset_no);
	if(ret == ERROR_ACCESSLABEL_WRITE_NG)
	{
		return ret;
	}

	switch(dataset_no)
	{
		case AM2_GRID_TBV_ANUM:
		case AM2_GRID_TBH_ANUM:
		case AM2_GRID_TB_ANUM:
		case AM2_GRID_TBV_TNUM:
		case AM2_GRID_TBH_TNUM:
		case AM2_GRID_TB_TNUM:
		case AM2_GRID_TIME:
			ret = SUCCESS;
			break;
		case AM2_GRID_GEO1_ANUM:
		case AM2_GRID_GEO2_ANUM:
		case AM2_GRID_GEO3_ANUM:
		case AM2_GRID_GEOA_ANUM:
		case AM2_GRID_GEO1_TNUM:
		case AM2_GRID_GEO2_TNUM:
		case AM2_GRID_GEO3_TNUM:
		case AM2_GRID_GEOA_TNUM:
			gridFlg = IOTK_ON;
			ret = SUCCESS;
			break;
		default:
			ret = ERROR_ACCESSLABEL_WRITE_NG;
			break;
	}

	if(ret == ERROR_ACCESSLABEL_WRITE_NG)
	{
		return ret;
	}

	if(gridFlg == IOTK_OFF)
	{
		ret = setDataSet(HDF_file_id, data, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, dataset_no, AM2_DTYPE_INT);
	}
	else
	{
		ret = setGridDataSet(HDF_file_id, data, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, dataset_no, AM2_DTYPE_INT);
	}
	if(ret != SUCCESS)
	{
		return ret;
	}

	return SUCCESS;
}
int amtk_set_gridint_(hid_t *HDF_file_id, int *data, int *dataset_no)
{
    return AMTK_set_GridInt(*HDF_file_id, data, *dataset_no);
}
/* For g77 entry */
int amtk_set_gridint__(hid_t *HDF_file_id, int *data, int *dataset_no)
{
    return amtk_set_gridint_(HDF_file_id, data, dataset_no);
}

/** @file amtk_set.c
 * Set HDF float in grid
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_set_GridFloat(hid_t HDF_file_id, float *data, int dataset_no)
{

	int ret = 0;
	int	gridFlg = IOTK_OFF;

	if(data == NULL)
	{
		return ERROR_NOT_WRITEDATA;
	}

	ret = CommonPrmCheck(HDF_file_id, AM2_SCANNUM_NOCHK, AM2_SCANNUM_NOCHK, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	ret = checkWriteNG(dataset_no);
	if(ret == ERROR_ACCESSLABEL_WRITE_NG)
	{
		return ret;
	}

	switch(dataset_no)
	{
		case AM2_GRID_TBV:
		case AM2_GRID_TBH:
		case AM2_GRID_TB:
		case AM2_GRID_TBV_STD:
		case AM2_GRID_TBH_STD:
		case AM2_GRID_TB_STD:
			ret = SUCCESS;
			break;
		case AM2_GRID_GEO1:
		case AM2_GRID_GEO2:
		case AM2_GRID_GEO3:
		case AM2_GRID_GEOA:
		case AM2_GRID_GEO1_STD:
		case AM2_GRID_GEO2_STD:
		case AM2_GRID_GEO3_STD:
		case AM2_GRID_GEOA_STD:
			gridFlg = IOTK_ON;
			ret = SUCCESS;
			break;
		default:
			ret = ERROR_ACCESSLABEL_WRITE_NG;
			break;
	}
	if(ret == ERROR_ACCESSLABEL_WRITE_NG)
	{
		return ret;
	}

	if(gridFlg == IOTK_OFF)
	{
		ret = setDataSet(HDF_file_id, data, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, dataset_no, AM2_DTYPE_FLOAT);
	}
	else
	{
		ret = setGridDataSet(HDF_file_id, data, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, dataset_no, AM2_DTYPE_FLOAT);
	}
	if(ret != SUCCESS)
	{
		return ret;
	}

	return SUCCESS;
}
int amtk_set_gridfloat_(hid_t *HDF_file_id, float *data, int *dataset_no)
{
    return AMTK_set_GridFloat(*HDF_file_id, data, *dataset_no);
}
/* For g77 entry */
int amtk_set_gridfloat__(hid_t *HDF_file_id, float *data, int *dataset_no)
{
    return amtk_set_gridfloat_(HDF_file_id, data, dataset_no);
}

/** @file amtk_set.c
 * Set HDF Scan data quality Byte order
 *
 * @author     NEC
 * @date       2010/06/30
 */
void checkByteOrder( unsigned char* org_value, unsigned char* check_value, int isLittleEndian)
{
    if(isLittleEndian == 0)
    {
        *check_value     = *(org_value+3);
        *(check_value+1) = *(org_value+2);
        *(check_value+2) = *(org_value+1);
        *(check_value+3) = *(org_value);
    }
    else
    {
        *check_value     = *(org_value);
        *(check_value+1) = *(org_value+1);
        *(check_value+2) = *(org_value+2);
        *(check_value+3) = *(org_value+3);
    }
}
/** @file amtk_set.c
 * Set HDF Scan data quality structure
 *
 * @author     NEC
 * @date       2011/06/13
 */
int AMTK_set_ScanDataQuality(hid_t HDF_file_id, AMTK_SCAN_DATA_QUALITY *quality, int from_scan, int to_scan, int dataset_no)
{
	ACCESS_LABEL	*p_table=NULL;
	int				size[3] = {0,0,0};
	int				ret = 0;
	int				rank = 0;
	int				table_num = 0;
	hid_t			dataset = 0;
	int				datasize = 0;
	int				getmemsize = 0;
	int				overlap = 0;
	hid_t			memspace = 0;
	hid_t			filespace = 0;
	unsigned char	*p_wdata = NULL;
	int				writecnt=0;
	int				cnt = 0;
	int				cnt1 = 0;
	int				cnt2 = 0;
	int				isLittleEndian = 0;

	/* check system byte order */
	isLittleEndian = 1;
	isLittleEndian = *(char *)(&isLittleEndian);

	if(quality == NULL){
		return ERROR_NOT_WRITEDATA;
	}

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}
	
	p_table = search_table(HDF_file_id, dataset_no, &table_num);
	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	ret = getOverLapScans( HDF_file_id, &overlap);
	if(ret != SUCCESS)
	{
		return ret;
	}

	rank = AMTK_getDimSize( HDF_file_id, dataset_no , &size[0]);
	if(rank < 0)
	{
		return ret;
	}

    datasize = sizeof(unsigned char);
	p_wdata  = (unsigned char *)getMemoryAlloc(&size[0], rank, datasize, from_scan, to_scan, &getmemsize);
	if(p_wdata == NULL)
	{
		return ERROR_MALLOC;
	}

	dataset = H5Dopen (HDF_file_id, name_table[table_num][p_table->dataset_no], H5P_DEFAULT);
	if(dataset < 0)
	{
		free(p_wdata);
		return ERROR_HDF_DATASET_OPEN;
	}

	ret = setHyperslab(dataset, from_scan, to_scan, &size[0], rank, overlap, &memspace, &filespace);
	if(ret < 0)
	{
		free(p_wdata);
		/* close dataset */
		H5Dclose(dataset);
		return ret;
	}

    writecnt = getmemsize/sizeof(AMTK_SCAN_DATA_QUALITY);

	for(cnt = 0; cnt < writecnt; cnt++)
	{
        cnt2 = 0;
        for(cnt1 = 0; cnt1 < 2; cnt1++)
        {
            checkByteOrder( (unsigned char*)&((quality+cnt)->angular_direction[cnt1]), 
                            (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
        	cnt2++;
        }
        checkByteOrder( (unsigned char*)&((quality+cnt)->quality_of_packet), 
                        (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
		cnt2++;
        checkByteOrder( (unsigned char*)&((quality+cnt)->angular_velocity), 
                        (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
		cnt2++;
        for(cnt1 = 0; cnt1 < 64; cnt1++)
        {
            checkByteOrder( (unsigned char*)&((quality+cnt)->calibration[cnt1]), 
                            (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
        	cnt2++;
        }
        checkByteOrder( (unsigned char*)&((quality+cnt)->spc_sps_error), 
                        (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
        cnt2++;
        for(cnt1 = 0; cnt1 < 16; cnt1++)
        {
            checkByteOrder( (unsigned char*)&((quality+cnt)->hts_temperature[cnt1]), 
                            (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
        	cnt2++;
        }
        for(cnt1 = 0; cnt1 < 33; cnt1++)
        {
             checkByteOrder( (unsigned char*)&((quality+cnt)->parity_error[cnt1]), 
                            (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
        	cnt2++;
        }
        for(cnt1 = 0; cnt1 < 4; cnt1++)
        {
            checkByteOrder( (unsigned char*)&((quality+cnt)->quality_info[cnt1]), 
                            (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
        	cnt2++;
        }
        for(cnt1 = 0; cnt1 < 6; cnt1++)
        {
            checkByteOrder( (unsigned char*)&((quality+cnt)->spare[cnt1]), 
                            (unsigned char*)(p_wdata+(512*cnt)+cnt2*4),  isLittleEndian);
        	cnt2++;
        }
	}
	ret = H5Dwrite( dataset,			/* hid_t dset_id        */
					H5T_NATIVE_UCHAR,	/* hid_t mem_type_id    */
					memspace,			/* hid_t mem_space_id   */
					filespace,			/* hid_t file_space_id  */
					H5P_DEFAULT,		/* hid_t plist_id       */
					p_wdata);			    /* void *buf   *out*    */
	
	free(p_wdata);
	/* close dataspace */
	H5Sclose(memspace);
	H5Sclose(filespace);
	/* close dataset */
	H5Dclose(dataset);
	
	if(ret < 0)
	{
		return ERROR_HDF_DATASET_WRITE;
	}

	return SUCCESS;
}


int amtk_set_scandataquality_(hid_t *HDF_file_id, AMTK_SCAN_DATA_QUALITY *quality, int *from_scan, int *to_scan, int *dataset_no)
{
	return AMTK_set_ScanDataQuality(*HDF_file_id, quality, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_set_scandataquality__(hid_t *HDF_file_id, AMTK_SCAN_DATA_QUALITY *quality, int *from_scan, int *to_scan, int *dataset_no)
{
	return amtk_set_scandataquality_(HDF_file_id, quality, from_scan, to_scan, dataset_no);
}
/*********************************************************************************/
