/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : amtk_hdf.c                                                */
/*  Abstract     : I/O toolkit HDF access function                           */
/*  Function List                                                            */
/*      get_L3_access_table()   : Get L3 Access label tabl                   */
/*      get_access_table()      : Get Access label table                     */
/*      search_table()          : Access label table search                  */
/*      getScaleFactor()        : Get scale factor                           */
/*      checkWriteNG()          : Check write NG                             */
/*      CommonPrmCheck()        : Common parameter check                     */
/*      checkAccessLabel()      : Check access label                         */
/*      getMetaDataInside()     : Get meta data (AMTK inside)                */
/*      getOverLapScans()       : Get overlap scan                           */
/*      getAttribute()          : Get attribute                              */
/*      getDataSet()            : Get dataset                                */
/*      setDataSet()            : Set dataset                                */
/*      getMemorySize()         : Get memory size                            */
/*      getMemoryAlloc()        : Get memory allocation                      */
/*      getGeophysicalInfo()    : Get Geophysical Infomation                 */
/*      getL3Scalefactor()      : Get scale factor for L3                    */
/*      setScaleFactor()        : function3 is sample                        */
/*      setUnit()               : Set unit                                   */
/*      getGridDataSet()        : get grid dataset                           */
/*      setGridDataSet()        : Set grid dataset                           */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2010/06/13                                                    */
/*  Version    1.10                                                          */
/*             1.11 2015/01/05 Add handling SensorShortName data for AMSR-E  */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include "./AMTK_PLUS.h"
#include "./IOTK_common.h"

#define DS_END    9999

union unit1{
	unsigned char	a[4];
	float	b;
	} ;

union unit_v{
	char	ch;
	int		in;
	float	fl;
	double	dou;
	} ;

/* ACCESS LABEL TABLE */
char *dataset_name_l1a[] ={
	"Scan Time",                                                /*  DS_SCAN_TIME */
	"Position in Orbit",                                        /*  DS_POS_ORBIT */
	"Navigation Data",                                          /*  DS_NAVI */
	"Attitude Data",                                            /*  DS_ATT */
	"Observation Count (6.9GHz,V)",                             /*  DS_OC06V */
	"Observation Count (6.9GHz,H)",                             /*  DS_OC06H */
	"Observation Count (7.3GHz,V)",                             /*  DS_OC07V */
	"Observation Count (7.3GHz,H)",                             /*  DS_OC07H */
	"Observation Count (10.7GHz,V)",                            /*  DS_OC10V */
	"Observation Count (10.7GHz,H)",                            /*  DS_OC10H */
	"Observation Count (18.7GHz,V)",                            /*  DS_OC18V */
	"Observation Count (18.7GHz,H)",                            /*  DS_OC18H */
	"Observation Count (23.8GHz,V)",                            /*  DS_OC23V */
	"Observation Count (23.8GHz,H)",                            /*  DS_OC23H */
	"Observation Count (36.5GHz,V)",                            /*  DS_OC36V */
	"Observation Count (36.5GHz,H)",                            /*  DS_OC36H */
	"Observation Count (89.0GHz-A,V)",                          /*  DS_OC89AV */
	"Observation Count (89.0GHz-A,H)",                          /*  DS_OC89AH */
	"Observation Count (89.0GHz-B,V)",                          /*  DS_OC89BV */
	"Observation Count (89.0GHz-B,H)",                          /*  DS_OC89BH */
	"Observation Count (6.9GHz-36.5GHz, V&H)",                  /*  DS_OC_LO */
	"Observation Count (89GHz-A, V&H)",                         /*  DS_OC_89A */
	"Observation Count (89GHz-B, V&H)",                         /*  DS_OC_89B */
	"Hot Load Count 6 to 36",                                   /*  DS_HTS_LO */
	"Hot Load Count 89",                                        /*  DS_HTS_HI */
	"Cold Sky Mirror Count 6 to 36",                            /*  DS_CSM_LO */
	"Cold Sky Mirror Count 89",                                 /*  DS_CSM_HI */
	"Rx Offset_Gain Count",                                     /*  DS_OFF_GAIN */
	"Sun Azimuth",                                              /*  DS_SUN_AZ */
	"Sun Elevation",                                            /*  DS_SUN_EL */
	"Earth Incidence",                                          /*  DS_EARTH_INC */
	"Earth Azimuth",                                            /*  DS_EARTH_AZ */
	"Land_Ocean Flag 6 to 36",                                  /*  DS_LOF_LO */
	"Land_Ocean Flag 89",                                       /*  DS_LOF_HI */
	"Observation Supplement",                                   /*  DS_OB_SPL */
	"SPC Temperature Count",                                    /*  DS_SPC_TEMP */
	"SPS Temperature Count",                                    /*  DS_SPS_TEMP */
	"PCD Data",                                                 /*  DS_PCD */
	"Scan Data Quality",                                        /*  DS_SCAN_QUAL */
	"Pixel Data Quality 6 to 36",                               /*  DS_PIX_QUAL_LO */
	"Pixel Data Quality 89",                                    /*  DS_PIX_QUAL_HI */
	"Interpolation Flag 6 to 36",                               /*  DS_INTPL_LO */
	"Interpolation Flag 89",                                    /*  DS_INTPL_HI */
	"Spill Over"                                                /*  DS_SPL_OV */
};

char *dataset_name_l1b[] ={
	"Scan Time",                                                /*  DS_SCAN_TIME */
	"Position in Orbit",                                        /*  DS_POS_ORBIT */
	"Navigation Data",                                          /*  DS_NAVI */
	"Attitude Data",                                            /*  DS_ATT */
	"Brightness Temperature (6.9GHz,V)",                        /*  DS_TB06V */
	"Brightness Temperature (6.9GHz,H)",                        /*  DS_TB06H */
	"Brightness Temperature (7.3GHz,V)",                        /*  DS_TB07V */
	"Brightness Temperature (7.3GHz,H)",                        /*  DS_TB07H */
	"Brightness Temperature (10.7GHz,V)",                       /*  DS_TB10V */
	"Brightness Temperature (10.7GHz,H)",                       /*  DS_TB10H */
	"Brightness Temperature (18.7GHz,V)",                       /*  DS_TB18V */
	"Brightness Temperature (18.7GHz,H)",                       /*  DS_TB18H */
	"Brightness Temperature (23.8GHz,V)",                       /*  DS_TB23V */
	"Brightness Temperature (23.8GHz,H)",                       /*  DS_TB23H */
	"Brightness Temperature (36.5GHz,V)",                       /*  DS_TB36V */
	"Brightness Temperature (36.5GHz,H)",                       /*  DS_TB36H */
	"Brightness Temperature (89.0GHz-A,V)",                     /*  DS_TB89AV */
	"Brightness Temperature (89.0GHz-A,H)",                     /*  DS_TB89AH */
	"Brightness Temperature (89.0GHz-B,V)",                     /*  DS_TB89BV */
	"Brightness Temperature (89.0GHz-B,H)",                     /*  DS_TB89BH */
	"Brightness Temperature (6.9GHz-36.5GHz, V&H)",             /*  DS_TB_LO */
	"Brightness Temperature (89GHz-A, V&H)",                    /*  DS_TB_89A */
	"Brightness Temperature (89GHz-B, V&H)",                    /*  DS_TB_89B */
	"Hot Load Count 6 to 36",                                   /*  DS_HTS_LO */
	"Hot Load Count 89",                                        /*  DS_HTS_HI */
	"Cold Sky Mirror Count 6 to 36",                            /*  DS_CSM_LO */
	"Cold Sky Mirror Count 89",                                 /*  DS_CSM_HI */
	"Rx Offset_Gain Count",                                     /*  DS_OFF_GAIN */
	"Sun Azimuth",                                              /*  DS_SUN_AZ */
	"Sun Elevation",                                            /*  DS_SUN_EL */
	"Earth Incidence",                                          /*  DS_EARTH_INC */
	"Earth Azimuth",                                            /*  DS_EARTH_AZ */
	"Land_Ocean Flag 6 to 36",                                  /*  DS_LOF_LO */
	"Land_Ocean Flag 89",                                       /*  DS_LOF_HI */
	"Observation Supplement",                                   /*  DS_OB_SPL */
	"SPC Temperature Count",                                    /*  DS_SPC_TEMP */
	"SPS Temperature Count",                                    /*  DS_SPS_TEMP */
	"PCD Data",                                                 /*  DS_PCD */
	"Scan Data Quality",                                        /*  DS_SCAN_QUAL */
	"Pixel Data Quality 6 to 36",                               /*  DS_PIX_QUAL_LO */
	"Pixel Data Quality 89",                                    /*  DS_PIX_QUAL_HI */
	"Interpolation Flag 6 to 36",                               /*  DS_INTPL_LO */
	"Interpolation Flag 89",                                    /*  DS_INTPL_HI */
	"Spill Over"                                                /*  DS_SPL_OV */
};

char *dataset_name_l1r[] ={
	"Scan Time",                                                /*  DS_SCAN_TIME */
	"Position in Orbit",                                        /*  DS_POS_ORBIT */
	"Navigation Data",                                          /*  DS_NAVI */
	"Attitude Data",                                            /*  DS_ATT */
	"Brightness Temperature (res06,6.9GHz,V)",                  /*  DS_RES06_TB06V */
	"Brightness Temperature (res06,6.9GHz,H)",                  /*  DS_RES06_TB06H */
	"Brightness Temperature (res06,7.3GHz,V)",                  /*  DS_RES06_TB07V */
	"Brightness Temperature (res06,7.3GHz,H)",                  /*  DS_RES06_TB07H */
	"Brightness Temperature (res06,10.7GHz,V)",                 /*  DS_RES06_TB10V */
	"Brightness Temperature (res06,10.7GHz,H)",                 /*  DS_RES06_TB10H */
	"Brightness Temperature (res06,18.7GHz,V)",                 /*  DS_RES06_TB18V */
	"Brightness Temperature (res06,18.7GHz,H)",                 /*  DS_RES06_TB18H */
	"Brightness Temperature (res06,23.8GHz,V)",                 /*  DS_RES06_TB23V */
	"Brightness Temperature (res06,23.8GHz,H)",                 /*  DS_RES06_TB23H */
	"Brightness Temperature (res06,36.5GHz,V)",                 /*  DS_RES06_TB36V */
	"Brightness Temperature (res06,36.5GHz,H)",                 /*  DS_RES06_TB36H */
	"Brightness Temperature (res06,89.0GHz,V)",                 /*  DS_RES06_TB89V */
	"Brightness Temperature (res06,89.0GHz,H)",                 /*  DS_RES06_TB89H */
	"Brightness Temperature (res06,all)",                       /*  DS_RES06_TB_ALL */
	"Brightness Temperature (res10,10.7GHz,V)",                 /*  DS_RES10_TB10V */
	"Brightness Temperature (res10,10.7GHz,H)",                 /*  DS_RES10_TB10H */
	"Brightness Temperature (res10,18.7GHz,V)",                 /*  DS_RES10_TB18V */
	"Brightness Temperature (res10,18.7GHz,H)",                 /*  DS_RES10_TB18H */
	"Brightness Temperature (res10,23.8GHz,V)",                 /*  DS_RES10_TB23V */
	"Brightness Temperature (res10,23.8GHz,H)",                 /*  DS_RES10_TB23H */
	"Brightness Temperature (res10,36.5GHz,V)",                 /*  DS_RES10_TB36V */
	"Brightness Temperature (res10,36.5GHz,H)",                 /*  DS_RES10_TB36H */
	"Brightness Temperature (res10,89.0GHz,V)",                 /*  DS_RES10_TB89V */
	"Brightness Temperature (res10,89.0GHz,H)",                 /*  DS_RES10_TB89H */
	"Brightness Temperature (res10,all)",                       /*  DS_RES10_TB_ALL */
	"Brightness Temperature (res23,18.7GHz,V)",                 /*  DS_RES23_TB18V */
	"Brightness Temperature (res23,18.7GHz,H)",                 /*  DS_RES23_TB18H */
	"Brightness Temperature (res23,23.8GHz,V)",                 /*  DS_RES23_TB23V */
	"Brightness Temperature (res23,23.8GHz,H)",                 /*  DS_RES23_TB23H */
	"Brightness Temperature (res23,36.5GHz,V)",                 /*  DS_RES23_TB36V */
	"Brightness Temperature (res23,36.5GHz,H)",                 /*  DS_RES23_TB36H */
	"Brightness Temperature (res23,89.0GHz,V)",                 /*  DS_RES23_TB89V */
	"Brightness Temperature (res23,89.0GHz,H)",                 /*  DS_RES23_TB89H */
	"Brightness Temperature (res23,all)",                       /*  DS_RES23_TB_ALL */
	"Brightness Temperature (res36,36.5GHz,V)",                 /*  DS_RES36_TB36V */
	"Brightness Temperature (res36,36.5GHz,H)",                 /*  DS_RES36_TB36H */
	"Brightness Temperature (res36,89.0GHz,V)",                 /*  DS_RES36_TB89V */
	"Brightness Temperature (res36,89.0GHz,H)",                 /*  DS_RES36_TB89H */
	"Brightness Temperature (res36,all)",                       /*  DS_RES36_TB_ALL */
	"Brightness Temperature (original,89GHz-A,V)",              /*  DS_TB89AV */
	"Brightness Temperature (original,89GHz-A,H)",              /*  DS_TB89AH */
	"Brightness Temperature (original,89GHz-B,V)",              /*  DS_TB89BV */
	"Brightness Temperature (original,89GHz-B,H)",              /*  DS_TB89BH */
	"Brightness Temperature (original,89GHz-A,V&H)",            /*  DS_TB_89A */
	"Brightness Temperature (original,89GHz-B,V&H)",            /*  DS_TB_89B */
	"Area Mean Height",                                         /*  DS_MEAN_HEIGHT */
	"Sun Azimuth",                                              /*  DS_SUN_AZ */
	"Sun Elevation",                                            /*  DS_SUN_EL */
	"Earth Incidence",                                          /*  DS_EARTH_INC */
	"Earth Azimuth",                                            /*  DS_EARTH_AZ */
	"Land_Ocean Flag 6 to 36",                                  /*  DS_LOF_RES_LO */
	"Land_Ocean Flag 89",                                       /*  DS_LOF_RES_HI */
	"Scan Data Quality",                                        /*  DS_SCAN_QUAL */
	"Pixel Data Quality 6 to 36",                               /*  DS_PIX_QUAL_LO */
	"Pixel Data Quality 89"                                     /*  DS_PIX_QUAL_HI */
};

char *dataset_name_l2[] ={
	"Scan Time",                                                /*  DS_SCAN_TIME */
	"Position in Orbit",                                        /*  DS_POS_ORBIT */
	"Geophysical Data",                                         /*  DS_SWATH_GEO1 */
	"Geophysical Data",                                         /*  DS_SWATH_GEO2 */
	"Geophysical Data",                                         /*  DS_SWATH_GEO3 */
	"Geophysical Data",                                         /*  DS_SWATH_GEOA */

	"Pixel Data Quality",                                       /*  DS_PIX_QUAL */

	"Geophysical Data for 89A",                                 /*  DS_SWATHA_GEO1 */
	"Geophysical Data for 89A",                                 /*  DS_SWATHA_GEO2 */
	"Geophysical Data for 89A",                                 /*  DS_SWATHA_GEO3 */
	"Geophysical Data for 89A",                                 /*  DS_SWATHA_GEOA */
	"Geophysical Data for 89B",                                 /*  DS_SWATHB_GEO1 */
	"Geophysical Data for 89B",                                 /*  DS_SWATHB_GEO2 */
	"Geophysical Data for 89B",                                 /*  DS_SWATHB_GEO3 */
	"Geophysical Data for 89B",                                 /*  DS_SWATHB_GEOA */

	"Pixel Data Quality for 89A",                               /*  DS_PIX_QUAL_A */
	"Pixel Data Quality for 89B"                                /*  DS_PIX_QUAL_B */
};

char *dataset_name_l3dh[] ={
	"Brightness Temperature (V)",                               /*  DS_GRID_TBV */
	"Brightness Temperature (H)",                               /*  DS_GRID_TBH */
	"Brightness Temperature (V&H)",                             /*  DS_GRID_TB */
	"Time Information",                                         /*  DS_GRID_TIME */
	"Geophysical Data",                                         /*  DS_GRID_GEO1 */
	"Geophysical Data",                                         /*  DS_GRID_GEO2 */
	"Geophysical Data",                                         /*  DS_GRID_GEO3 */
	"Geophysical Data",                                         /*  DS_GRID_GEOA */
	"Time Information",                                         /*  DS_GRID_TIME */
};

char *dataset_name_l3dl[] ={
	"Brightness Temperature (V)",                               /*  DS_GRID_TBV */
	"Brightness Temperature (H)",                               /*  DS_GRID_TBH */
	"Brightness Temperature (V&H)",                             /*  DS_GRID_TB */
	"Time Information",                                         /*  DS_GRID_TIME */
	"Geophysical Data",                                         /*  DS_GRID_GEO1 */
	"Geophysical Data",                                         /*  DS_GRID_GEO2 */
	"Geophysical Data",                                         /*  DS_GRID_GEO3 */
	"Geophysical Data",                                         /*  DS_GRID_GEOA */
	"Time Information",                                         /*  DS_GRID_TIME */
};

char *dataset_name_l3mh[] ={
	"Brightness Temperature (V)",                               /*  DS_GRID_TBV */
	"Brightness Temperature (H)",                               /*  DS_GRID_TBH */
	"Brightness Temperature (V&H)",                             /*  DS_GRID_TB */
	"Standard Deviation (V)",                                   /*  DS_GRID_TBV_STD */
	"Standard Deviation (H)",                                   /*  DS_GRID_TBH_STD */
	"Standard Deviation (V&H)",                                 /*  DS_GRID_TB_STD */
	"Average Number (V)",                                       /*  DS_GRID_TBV_ANUM */
	"Average Number (H)",                                       /*  DS_GRID_TBH_ANUM */
	"Average Number (V&H)",                                     /*  DS_GRID_TB_ANUM */
	"Total Number (V)",                                         /*  DS_GRID_TBV_TNUM */
	"Total Number (H)",                                         /*  DS_GRID_TBH_TNUM */
	"Total Number (V&H)",                                       /*  DS_GRID_TB_TNUM */
	"Geophysical Data",                                         /*  DS_GRID_GEO1 */
	"Geophysical Data",                                         /*  DS_GRID_GEO2 */
	"Geophysical Data",                                         /*  DS_GRID_GEO3 */
	"Geophysical Data",                                         /*  DS_GRID_GEOA */
	"Standard Deviation",                                       /*  DS_GRID_GEO1_STD */
	"Standard Deviation",                                       /*  DS_GRID_GEO2_STD */
	"Standard Deviation",                                       /*  DS_GRID_GEO3_STD */
	"Standard Deviation",                                       /*  DS_GRID_GEOA_STD */
	"Average Number",                                           /*  DS_GRID_GEO1_ANUM */
	"Average Number",                                           /*  DS_GRID_GEO2_ANUM */
	"Average Number",                                           /*  DS_GRID_GEO3_ANUM */
	"Average Number",                                           /*  DS_GRID_GEOA_ANUM */
	"Total Number",                                             /*  DS_GRID_GEO1_TNUM */
	"Total Number",                                             /*  DS_GRID_GEO2_TNUM */
	"Total Number",                                             /*  DS_GRID_GEO3_TNUM */
	"Total Number",                                             /*  DS_GRID_GEOA_TNUM */
};

char *dataset_name_l3ml[] ={
	"Brightness Temperature (V)",                               /*  DS_GRID_TBV */
	"Brightness Temperature (H)",                               /*  DS_GRID_TBH */
	"Brightness Temperature (V&H)",                             /*  DS_GRID_TB */
	"Standard Deviation (V)",                                   /*  DS_GRID_TBV_STD */
	"Standard Deviation (H)",                                   /*  DS_GRID_TBH_STD */
	"Standard Deviation (V&H)",                                 /*  DS_GRID_TB_STD */
	"Average Number (V)",                                       /*  DS_GRID_TBV_ANUM */
	"Average Number (H)",                                       /*  DS_GRID_TBH_ANUM */
	"Average Number (V&H)",                                     /*  DS_GRID_TB_ANUM */
	"Total Number (V)",                                         /*  DS_GRID_TBV_TNUM */
	"Total Number (H)",                                         /*  DS_GRID_TBH_TNUM */
	"Total Number (V&H)",                                       /*  DS_GRID_TB_TNUM */
	"Geophysical Data",                                         /*  DS_GRID_GEO1 */
	"Geophysical Data",                                         /*  DS_GRID_GEO2 */
	"Geophysical Data",                                         /*  DS_GRID_GEO3 */
	"Geophysical Data",                                         /*  DS_GRID_GEOA */
	"Standard Deviation",                                       /*  DS_GRID_GEO1_STD */
	"Standard Deviation",                                       /*  DS_GRID_GEO2_STD */
	"Standard Deviation",                                       /*  DS_GRID_GEO3_STD */
	"Standard Deviation",                                       /*  DS_GRID_GEOA_STD */
	"Average Number",                                           /*  DS_GRID_GEO1_ANUM */
	"Average Number",                                           /*  DS_GRID_GEO2_ANUM */
	"Average Number",                                           /*  DS_GRID_GEO3_ANUM */
	"Average Number",                                           /*  DS_GRID_GEOA_ANUM */
	"Total Number",                                             /*  DS_GRID_GEO1_TNUM */
	"Total Number",                                             /*  DS_GRID_GEO2_TNUM */
	"Total Number",                                             /*  DS_GRID_GEO3_TNUM */
	"Total Number",                                             /*  DS_GRID_GEOA_TNUM */
};

char *latlon_name_lo[] ={
	"Latitude of Observation Point",
	"Longitude of Observation Point"
};

char *latlon_name_89a[] ={
	"Latitude of Observation Point for 89A",
	"Longitude of Observation Point for 89A"
};

char *latlon_name_89b[] ={
	"Latitude of Observation Point for 89B",
	"Longitude of Observation Point for 89B"
};

/* ACCESS LABEL TABLE */

ACCESS_LABEL l1a_table[]={
	{DS_L1A_SCAN_TIME   ,H5U_IEEE_F64LE,OFF,OFF,ON ,OFF, 1,2018,   0,   0,AM2_SCAN_TIME_DEF,(float)1.00, "sec"},
	{DS_L1A_POS_ORBIT   ,H5U_IEEE_F64LE,OFF,OFF,ON ,OFF, 1,2018,   0,   0,AM2_POS_ORBIT    ,(float)1.00, ""},
	{DS_L1A_NAVI        ,H5U_IEEE_F32LE,OFF,ON ,OFF,OFF, 2,2018,   6,   0,AM2_NAVI         ,(float)1.00, "m,m/s"},
	{DS_L1A_ATT         ,H5U_IEEE_F32LE,OFF,ON ,OFF,OFF, 2,2018,   3,   0,AM2_ATT          ,(float)1.00, "deg"},
	{DS_L1A_OC06V       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC06V        ,(float)1.00, "Count"},
	{DS_L1A_OC06H       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC06H        ,(float)1.00, "Count"},
	{DS_L1A_OC07V       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC07V        ,(float)1.00, "Count"},
	{DS_L1A_OC07H       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC07H        ,(float)1.00, "Count"},
	{DS_L1A_OC10V       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC10V        ,(float)1.00, "Count"},
	{DS_L1A_OC10H       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC10H        ,(float)1.00, "Count"},
	{DS_L1A_OC18V       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC18V        ,(float)1.00, "Count"},
	{DS_L1A_OC18H       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC18H        ,(float)1.00, "Count"},
	{DS_L1A_OC23V       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC23V        ,(float)1.00, "Count"},
	{DS_L1A_OC23H       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC23H        ,(float)1.00, "Count"},
	{DS_L1A_OC36V       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC36V        ,(float)1.00, "Count"},
	{DS_L1A_OC36H       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC36H        ,(float)1.00, "Count"},
	{DS_L1A_OC89AV      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC89AV       ,(float)1.00, "Count"},
	{DS_L1A_OC89AH      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC89AH       ,(float)1.00, "Count"},
	{DS_L1A_OC89BV      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC89BV       ,(float)1.00, "Count"},
	{DS_L1A_OC89BH      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC89BH       ,(float)1.00, "Count"},
	{DS_L1A_OC_LO       ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC_LO        ,(float)1.00, "Count"},
	{DS_L1A_OC_89A      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC_89A       ,(float)1.00, "Count"},
	{DS_L1A_OC_89B      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,2018, 243,   0,AM2_OC_89B       ,(float)1.00, "Count"},
	{DS_L1A_HTS_LO      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 3,  12,2018,  16,AM2_HTS_LO       ,(float)1.00, "Count"},
	{DS_L1A_HTS_HI      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 3,   4,2018,  32,AM2_HTS_HI       ,(float)1.00, "Count"},
	{DS_L1A_CSM_LO      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 3,  12,2018,  16,AM2_CSM_LO       ,(float)1.00, "Count"},
	{DS_L1A_CSM_HI      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 3,   4,2018,  32,AM2_CSM_HI       ,(float)1.00, "Count"},
	{DS_L1A_OFF_GAIN    ,H5U_STD_U16LE ,ON ,OFF,OFF,OFF, 2,2018,  32,   0,AM2_OFF_GAIN     ,(float)1.00, "Count"},
	{DS_L1A_SUN_AZ      ,H5U_STD_I16LE ,OFF,ON, OFF,OFF, 2,2018, 243,   0,AM2_SUN_AZ       ,(float)0.01, "deg"},
	{DS_L1A_SUN_EL      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_SUN_EL       ,(float)0.01, "deg"},
	{DS_L1A_EARTH_INC   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_EARTH_INC    ,(float)0.01, "deg"},
	{DS_L1A_EARTH_AZ    ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_EARTH_AZ     ,(float)0.01, "deg"},
	{DS_L1A_LOF_LO      ,H5U_STD_U8LE  ,ON ,ON ,OFF,OFF, 3,   6,2018, 243,AM2_LOF_LO       ,(float)1.00, "%"},
	{DS_L1A_LOF_HI      ,H5U_STD_U8LE  ,ON ,ON ,OFF,OFF, 3,   2,2018, 486,AM2_LOF_HI       ,(float)1.00, "%"},
	{DS_L1A_OB_SPL      ,H5U_STD_U8LE  ,OFF,OFF,OFF,ON , 2,2018,  54,   0,AM2_OB_SPL       ,(float)1.00, ""},
	{DS_L1A_SPC_TEMP    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018,  20,   0,AM2_SPC_TEMP     ,(float)1.00, "Count"},
	{DS_L1A_SPS_TEMP    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018,  32,   0,AM2_SPS_TEMP     ,(float)1.00, "Count"},
	{DS_L1A_PCD         ,H5U_STD_U8LE  ,OFF,OFF,OFF,ON , 2,2018,  64,   0,AM2_PCD          ,(float)1.00, ""},
	{DS_L1A_SCAN_QUAL   ,H5U_STD_U8LE  ,OFF,OFF,OFF,OFF, 2,2018, 512,   0,AM2_SCAN_QUAL    ,(float)1.00, ""},
	{DS_L1A_PIX_QUAL_LO ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,2018, 486,   0,AM2_PIX_QUAL_LO  ,(float)1.00, ""},
	{DS_L1A_PIX_QUAL_HI ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,2018, 486,   0,AM2_PIX_QUAL_HI  ,(float)1.00, ""},
	{DS_L1A_INTPL_LO    ,H5U_STD_U8LE  ,OFF,OFF,OFF,ON , 3,  12,2018,  16,AM2_INTPL_LO     ,(float)1.00, ""},
	{DS_L1A_INTPL_HI    ,H5U_STD_U8LE  ,OFF,OFF,OFF,ON , 3,   4,2018,  32,AM2_INTPL_HI     ,(float)1.00, ""},
	{DS_L1A_SPL_OV      ,H5U_IEEE_F32LE,OFF,ON ,OFF,OFF, 3,   2, 200, 243,AM2_SPL_OV       ,(float)1.00, "mV"},
	{DS_END             ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0                ,(float)1.00, ""}                   /* Stopper */
};

ACCESS_LABEL l1b_table[]= {
	{DS_L1B_SCAN_TIME   ,H5U_IEEE_F64LE,OFF,OFF,ON ,OFF, 1,2018,   0,   0,AM2_SCAN_TIME_DEF,(float)1.00, "sec"},
	{DS_L1B_POS_ORBIT   ,H5U_IEEE_F64LE,OFF,OFF,ON ,OFF, 1,2018,   0,   0,AM2_POS_ORBIT    ,(float)1.00, ""},
	{DS_L1B_NAVI        ,H5U_IEEE_F32LE,OFF,ON ,OFF,OFF, 2,2018,   6,   0,AM2_NAVI         ,(float)1.00, "m,m/s"},
	{DS_L1B_ATT         ,H5U_IEEE_F32LE,OFF,ON ,OFF,OFF, 2,2018,   3,   0,AM2_ATT          ,(float)1.00, "deg"},
	{DS_L1B_TB06V       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB06V        ,(float)0.01, "K"},
	{DS_L1B_TB06H       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB06H        ,(float)0.01, "K"},
	{DS_L1B_TB07V       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB07V        ,(float)0.01, "K"},
	{DS_L1B_TB07H       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB07H        ,(float)0.01, "K"},
	{DS_L1B_TB10V       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB10V        ,(float)0.01, "K"},
	{DS_L1B_TB10H       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB10H        ,(float)0.01, "K"},
	{DS_L1B_TB18V       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB18V        ,(float)0.01, "K"},
	{DS_L1B_TB18H       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB18H        ,(float)0.01, "K"},
	{DS_L1B_TB23V       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB23V        ,(float)0.01, "K"},
	{DS_L1B_TB23H       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB23H        ,(float)0.01, "K"},
	{DS_L1B_TB36V       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB36V        ,(float)0.01, "K"},
	{DS_L1B_TB36H       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_TB36H        ,(float)0.01, "K"},
	{DS_L1B_TB89AV      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB89AV       ,(float)0.01, "K"},
	{DS_L1B_TB89AH      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB89AH       ,(float)0.01, "K"},
	{DS_L1B_TB89BV      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB89BV       ,(float)0.01, "K"},
	{DS_L1B_TB89BH      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB89BH       ,(float)0.01, "K"},
	{DS_L1B_TB_LO       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB_LO        ,(float)0.01, "K"},
	{DS_L1B_TB_89A      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB_89A       ,(float)0.01, "K"},
	{DS_L1B_TB_89B      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB_89B       ,(float)0.01, "K"},
	{DS_L1B_HTS_LO      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 3,  12,2018,  16,AM2_HTS_LO       ,(float)1.00, "Count"},
	{DS_L1B_HTS_HI      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 3,   4,2018,  32,AM2_HTS_HI       ,(float)1.00, "Count"},
	{DS_L1B_CSM_LO      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 3,  12,2018,  16,AM2_CSM_LO       ,(float)1.00, "Count"},
	{DS_L1B_CSM_HI      ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 3,   4,2018,  32,AM2_CSM_HI       ,(float)1.00, "Count"},
	{DS_L1B_OFF_GAIN    ,H5U_STD_U16LE ,ON ,OFF,OFF,OFF, 2,2018,  32,   0,AM2_OFF_GAIN     ,(float)1.00, "Count"},
	{DS_L1B_SUN_AZ      ,H5U_STD_I16LE ,OFF,ON, OFF,OFF, 2,2018, 243,   0,AM2_SUN_AZ       ,(float)0.01, "deg"},
	{DS_L1B_SUN_EL      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_SUN_EL       ,(float)0.01, "deg"},
	{DS_L1B_EARTH_INC   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_EARTH_INC    ,(float)0.01, "deg"},
	{DS_L1B_EARTH_AZ    ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_EARTH_AZ     ,(float)0.01, "deg"},
	{DS_L1B_LOF_LO      ,H5U_STD_U8LE  ,ON ,ON ,OFF,OFF, 3,   6,2018, 243,AM2_LOF_LO       ,(float)1.00, "%"},
	{DS_L1B_LOF_HI      ,H5U_STD_U8LE  ,ON ,ON ,OFF,OFF, 3,   2,2018, 486,AM2_LOF_HI       ,(float)1.00, "%"},
	{DS_L1B_OB_SPL      ,H5U_STD_U8LE  ,OFF,OFF,OFF,ON , 2,2018,  54,   0,AM2_OB_SPL       ,(float)1.00, ""},
	{DS_L1B_SPC_TEMP    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018,  20,   0,AM2_SPC_TEMP     ,(float)1.00, "Count"},
	{DS_L1B_SPS_TEMP    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018,  32,   0,AM2_SPS_TEMP     ,(float)1.00, "Count"},
	{DS_L1B_PCD         ,H5U_STD_U8LE  ,OFF,OFF,OFF,ON , 2,2018,  64,   0,AM2_PCD          ,(float)1.00, ""},
	{DS_L1B_SCAN_QUAL   ,H5U_STD_U8LE  ,OFF,OFF,OFF,OFF, 2,2018, 512,   0,AM2_SCAN_QUAL    ,(float)1.00, ""},
	{DS_L1B_PIX_QUAL_LO ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,2018, 486,   0,AM2_PIX_QUAL_LO  ,(float)1.00, ""},
	{DS_L1B_PIX_QUAL_HI ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,2018, 486,   0,AM2_PIX_QUAL_HI  ,(float)1.00, ""},
	{DS_L1B_INTPL_LO    ,H5U_STD_U8LE  ,OFF,OFF,OFF,ON , 3,  12,2018,  16,AM2_INTPL_LO     ,(float)1.00, ""},
	{DS_L1B_INTPL_HI    ,H5U_STD_U8LE  ,OFF,OFF,OFF,ON , 3,   4,2018,  32,AM2_INTPL_HI     ,(float)1.00, ""},
	{DS_L1B_SPL_OV      ,H5U_IEEE_F32LE,OFF,ON ,OFF,OFF, 3,   2, 200, 243,AM2_SPL_OV       ,(float)1.00, "mV"},
	{DS_END             ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0                ,(float)1.00, ""}                   /* Stopper */
};

ACCESS_LABEL l1r_table[]={
	{DS_L1R_SCAN_TIME   ,H5U_IEEE_F64LE,OFF,OFF,ON ,OFF, 1,2018,   0,   0,AM2_SCAN_TIME_DEF,(float)1.00, "sec"},
	{DS_L1R_POS_ORBIT   ,H5U_IEEE_F64LE,OFF,OFF,ON ,OFF, 1,2018,   0,   0,AM2_POS_ORBIT    ,(float)1.00, ""},
	{DS_L1R_NAVI        ,H5U_IEEE_F32LE,OFF,ON ,OFF,OFF, 2,2018,   6,   0,AM2_NAVI         ,(float)1.00, "m,m/s"},
	{DS_L1R_ATT         ,H5U_IEEE_F32LE,OFF,ON ,OFF,OFF, 2,2018,   3,   0,AM2_ATT          ,(float)1.00, "deg"},

	{DS_L1R_RES06_TB06V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB06V,(float)0.01, "K"},
	{DS_L1R_RES06_TB06H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB06H,(float)0.01, "K"},
	{DS_L1R_RES06_TB07V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB07V,(float)0.01, "K"},
	{DS_L1R_RES06_TB07H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB07H,(float)0.01, "K"},
	{DS_L1R_RES06_TB10V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB10V,(float)0.01, "K"},
	{DS_L1R_RES06_TB10H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB10H,(float)0.01, "K"},
	{DS_L1R_RES06_TB18V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB18V,(float)0.01, "K"},
	{DS_L1R_RES06_TB18H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB18H,(float)0.01, "K"},
	{DS_L1R_RES06_TB23V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB23V,(float)0.01, "K"},
	{DS_L1R_RES06_TB23H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB23H,(float)0.01, "K"},
	{DS_L1R_RES06_TB36V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB36V,(float)0.01, "K"},
	{DS_L1R_RES06_TB36H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB36H,(float)0.01, "K"},
	{DS_L1R_RES06_TB89V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB89V,(float)0.01, "K"},
	{DS_L1R_RES06_TB89H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB89H,(float)0.01, "K"},
	{DS_L1R_RES06_TB_ALL,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES06_TB_ALL,(float)0.01, "K"},

	{DS_L1R_RES10_TB10V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB10V,(float)0.01, "K"},
	{DS_L1R_RES10_TB10H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB10H,(float)0.01, "K"},
	{DS_L1R_RES10_TB18V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB18V,(float)0.01, "K"},
	{DS_L1R_RES10_TB18H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB18H,(float)0.01, "K"},
	{DS_L1R_RES10_TB23V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB23V,(float)0.01, "K"},
	{DS_L1R_RES10_TB23H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB23H,(float)0.01, "K"},
	{DS_L1R_RES10_TB36V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB36V,(float)0.01, "K"},
	{DS_L1R_RES10_TB36H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB36H,(float)0.01, "K"},
	{DS_L1R_RES10_TB89V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB89V,(float)0.01, "K"},
	{DS_L1R_RES10_TB89H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB89H,(float)0.01, "K"},
	{DS_L1R_RES10_TB_ALL,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES10_TB_ALL,(float)0.01, "K"},

	{DS_L1R_RES23_TB18V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB18V,(float)0.01, "K"},
	{DS_L1R_RES23_TB18H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB18H,(float)0.01, "K"},
	{DS_L1R_RES23_TB23V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB23V,(float)0.01, "K"},
	{DS_L1R_RES23_TB23H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB23H,(float)0.01, "K"},
	{DS_L1R_RES23_TB36V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB36V,(float)0.01, "K"},
	{DS_L1R_RES23_TB36H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB36H,(float)0.01, "K"},
	{DS_L1R_RES23_TB89V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB89V,(float)0.01, "K"},
	{DS_L1R_RES23_TB89H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB89H,(float)0.01, "K"},
	{DS_L1R_RES23_TB_ALL,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES23_TB_ALL,(float)0.01, "K"},

	{DS_L1R_RES36_TB36V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES36_TB36V,(float)0.01, "K"},
	{DS_L1R_RES36_TB36H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES36_TB36H,(float)0.01, "K"},
	{DS_L1R_RES36_TB89V ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES36_TB89V,(float)0.01, "K"},
	{DS_L1R_RES36_TB89H ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES36_TB89H,(float)0.01, "K"},
	{DS_L1R_RES36_TB_ALL,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_RES36_TB_ALL,(float)0.01, "K"},
	
	{DS_L1R_TB89AV      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB89AV       ,(float)0.01, "K"},
	{DS_L1R_TB89AH      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB89AH       ,(float)0.01, "K"},
	{DS_L1R_TB89BV      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB89BV       ,(float)0.01, "K"},
	{DS_L1R_TB89BH      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB89BH       ,(float)0.01, "K"},
	{DS_L1R_TB_89A      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB_89A       ,(float)0.01, "K"},
	{DS_L1R_TB_89B      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,2018, 486,   0,AM2_TB_89B       ,(float)0.01, "K"},

	{DS_L1R_MEAN_HEIGHT ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_MEAN_HEIGHT  ,(float)1.00, "m"},
	{DS_L1R_SUN_AZ      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_SUN_AZ       ,(float)0.01, "deg"},
	{DS_L1R_SUN_EL      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_SUN_EL       ,(float)0.01, "deg"},
	{DS_L1R_EARTH_INC   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_EARTH_INC    ,(float)0.01, "deg"},
	{DS_L1R_EARTH_AZ    ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,2018, 243,   0,AM2_EARTH_AZ     ,(float)0.01, "deg"},
	{DS_L1R_LOF_RES_LO  ,H5U_STD_U8LE  ,ON ,ON ,OFF,OFF, 3,   4,2018, 243,AM2_LOF_RES_LO   ,(float)1.00, "%"},
	{DS_L1R_LOF_RES_HI  ,H5U_STD_U8LE  ,ON ,ON ,OFF,OFF, 3,   2,2018, 486,AM2_LOF_RES_HI   ,(float)1.00, "%"},
	{DS_L1R_SCAN_QUAL   ,H5U_STD_U8LE  ,OFF,OFF,OFF,OFF, 2,2018, 512,   0,AM2_SCAN_QUAL    ,(float)1.00, ""},
	{DS_L1R_PIX_QUAL_LO ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,2018, 486,   0,AM2_PIX_QUAL_LO  ,(float)1.00, ""},
	{DS_L1R_PIX_QUAL_HI ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,2018, 486,   0,AM2_PIX_QUAL_HI  ,(float)1.00, ""},
	{DS_END             ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0                ,(float)1.00, ""}     /* Stopper */
};

ACCESS_LABEL l2_table[]={
	{DS_L2_SCAN_TIME    ,H5U_IEEE_F64LE,OFF,OFF,ON ,OFF, 1,1978,   0,   0,AM2_SCAN_TIME_DEF,(float)1.00, "sec"},
	{DS_L2_POS_ORBIT    ,H5U_IEEE_F64LE,OFF,OFF,ON ,OFF, 1,1978,   0,   0,AM2_POS_ORBIT    ,(float)1.00, ""},

	{DS_L2_SWATH_GEO1   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 243,   0,AM2_SWATH_GEO1   ,(float)0.01, ""},
	{DS_L2_SWATH_GEO2   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 243,   0,AM2_SWATH_GEO2   ,(float)0.01, ""},
	{DS_L2_SWATH_GEO3   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 243,   0,AM2_SWATH_GEO3   ,(float)0.01, ""},
	{DS_L2_SWATH_GEOA   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 3,   3,1978, 243,AM2_SWATH_GEOA   ,(float)0.01, ""},

	{DS_L2_PIX_QUAL     ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,1978, 243,   0,AM2_PIX_QUAL     ,(float)1.00, ""},

	{DS_L2_SWATHA_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 486,   0,AM2_SWATHA_GEO1  ,(float)0.10, ""},
	{DS_L2_SWATHA_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 486,   0,AM2_SWATHA_GEO2  ,(float)0.10, ""},
	{DS_L2_SWATHA_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 486,   0,AM2_SWATHA_GEO3  ,(float)0.10, ""},
	{DS_L2_SWATHA_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 3,   3,1978, 486,AM2_SWATHA_GEOA  ,(float)0.10, ""},

	{DS_L2_SWATHB_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 486,   0,AM2_SWATHB_GEO1  ,(float)0.10, ""},
	{DS_L2_SWATHB_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 486,   0,AM2_SWATHB_GEO2  ,(float)0.10, ""},
	{DS_L2_SWATHB_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1978, 486,   0,AM2_SWATHB_GEO3  ,(float)0.10, ""},
	{DS_L2_SWATHB_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 3,   3,1978, 486,AM2_SWATHB_GEOA  ,(float)0.10, ""},
	{DS_L2_PIX_QUAL_A   ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,1978, 486,   0,AM2_PIX_QUAL_A   ,(float)1.00, ""},
	{DS_L2_PIX_QUAL_B   ,H5U_STD_U8LE  ,ON ,OFF,OFF,ON , 2,1978, 486,   0,AM2_PIX_QUAL_B   ,(float)1.00, ""},
	{DS_END             ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0                ,(float)0.00, ""}   /* Stopper */
};

ACCESS_LABEL l3dh_table[]={
	{DS_L3DH_GRID_TBV   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3DH_GRID_TBH   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3DH_GRID_TB    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3DH_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DH_GRID_TBV   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3DH_GRID_TBH   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3DH_GRID_TB    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3DH_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DH_GRID_TBV   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3DH_GRID_TBH   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3DH_GRID_TB    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3DH_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DH_GRID_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO1    ,(float)0.1, "K"},
	{DS_L3DH_GRID_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO2    ,(float)0.1, "K"},
	{DS_L3DH_GRID_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO3    ,(float)0.1, "K"},
	{DS_L3DH_GRID_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEOA    ,(float)0.1, "K"},
	{DS_L3DH_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DH_GRID_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO1    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO2    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO3    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEOA    ,(float)0.1, ""},
	{DS_L3DH_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DH_GRID_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO1    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO2    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO3    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEOA    ,(float)0.1, ""},
	{DS_L3DH_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DH_GRID_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO1    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO2    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO3    ,(float)0.1, ""},
	{DS_L3DH_GRID_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEOA    ,(float)0.1, ""},
	{DS_L3DH_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_TIME    ,(float)1.00, "min"},
	{DS_END             ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0                ,(float)0.00, ""}  /* Stopper */
};

ACCESS_LABEL l3dl_table[]={
	{DS_L3DL_GRID_TBV   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3DL_GRID_TBH   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3DL_GRID_TB    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3DL_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DL_GRID_TBV   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3DL_GRID_TBH   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3DL_GRID_TB    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3DL_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DL_GRID_TBV   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3DL_GRID_TBH   ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3DL_GRID_TB    ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3DL_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DL_GRID_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO1    ,(float)0.1, "K"},
	{DS_L3DL_GRID_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO2    ,(float)0.1, "K"},
	{DS_L3DL_GRID_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO3    ,(float)0.1, "K"},
	{DS_L3DL_GRID_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEOA    ,(float)0.1, "K"},
	{DS_L3DL_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DL_GRID_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO1    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO2    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO3    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEOA    ,(float)0.1, ""},
	{DS_L3DL_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DL_GRID_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO1    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO2    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO3    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEOA    ,(float)0.1, ""},
	{DS_L3DL_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TIME    ,(float)1.00, "min"},

	{DS_L3DL_GRID_GEO1  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO1    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEO2  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO2    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEO3  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO3    ,(float)0.1, ""},
	{DS_L3DL_GRID_GEOA  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEOA    ,(float)0.1, ""},
	{DS_L3DL_GRID_TIME  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_TIME    ,(float)1.00, "min"},
	{DS_END             ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0                ,(float)0.01, ""}   /* Stopper */
};

ACCESS_LABEL l3mh_table[]={
	{DS_L3MH_GRID_TBV     ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3MH_GRID_TBH     ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3MH_GRID_TB      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3MH_GRID_TBV_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBV_STD ,(float)0.01, ""},
	{DS_L3MH_GRID_TBH_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBH_STD ,(float)0.01, ""},
	{DS_L3MH_GRID_TB_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TB_STD  ,(float)0.01, ""},
	{DS_L3MH_GRID_TBV_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBV_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_TBH_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBH_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_TB_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TB_ANUM ,(float)1.00, ""},
	{DS_L3MH_GRID_TBV_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBV_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_TBH_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TBH_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_TB_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_TB_TNUM ,(float)1.00, ""},

	{DS_L3MH_GRID_TBV     ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3MH_GRID_TBH     ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3MH_GRID_TB      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3MH_GRID_TBV_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBV_STD ,(float)0.01, ""},
	{DS_L3MH_GRID_TBH_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBH_STD ,(float)0.01, ""},
	{DS_L3MH_GRID_TB_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TB_STD  ,(float)0.01, ""},
	{DS_L3MH_GRID_TBV_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBV_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_TBH_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBH_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_TB_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TB_ANUM ,(float)1.00, ""},
	{DS_L3MH_GRID_TBV_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBV_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_TBH_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TBH_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_TB_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_TB_TNUM ,(float)1.00, ""},

	{DS_L3MH_GRID_TBV     ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3MH_GRID_TBH     ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3MH_GRID_TB      ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3MH_GRID_TBV_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBV_STD ,(float)0.01, ""},
	{DS_L3MH_GRID_TBH_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBH_STD ,(float)0.01, ""},
	{DS_L3MH_GRID_TB_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TB_STD  ,(float)0.01, ""},
	{DS_L3MH_GRID_TBV_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBV_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_TBH_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBH_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_TB_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TB_ANUM ,(float)1.00, ""},
	{DS_L3MH_GRID_TBV_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBV_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_TBH_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TBH_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_TB_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_TB_TNUM ,(float)1.00, ""},

	{DS_L3MH_GRID_GEO1     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO1,(float)0.1, ""},
	{DS_L3MH_GRID_GEO2     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO2,(float)0.1, ""},
	{DS_L3MH_GRID_GEO3     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO3,(float)0.1, ""},
	{DS_L3MH_GRID_GEOA     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEOA,(float)0.1, ""},
	{DS_L3MH_GRID_GEO1_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO1_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO2_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO2_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO3_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO3_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEOA_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEOA_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO1_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO1_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO2_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO2_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO3_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO3_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEOA_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEOA_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO1_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO1_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO2_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO2_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO3_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEO3_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEOA_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1800,3600,   0,AM2_GRID_GEOA_TNUM,(float)1.00, ""},

	{DS_L3MH_GRID_GEO1     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO1,(float)0.1, ""},
	{DS_L3MH_GRID_GEO2     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO2,(float)0.1, ""},
	{DS_L3MH_GRID_GEO3     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO3,(float)0.1, ""},
	{DS_L3MH_GRID_GEOA     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEOA,(float)0.1, ""},
	{DS_L3MH_GRID_GEO1_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO1_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO2_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO2_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO3_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO3_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEOA_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEOA_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO1_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO1_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO2_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO2_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO3_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO3_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEOA_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEOA_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO1_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO1_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO2_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO2_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO3_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEO3_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEOA_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1120, 760,   0,AM2_GRID_GEOA_TNUM,(float)1.00, ""},

	{DS_L3MH_GRID_GEO1     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO1,(float)0.1, ""},
	{DS_L3MH_GRID_GEO2     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO2,(float)0.1, ""},
	{DS_L3MH_GRID_GEO3     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO3,(float)0.1, ""},
	{DS_L3MH_GRID_GEOA     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEOA,(float)0.1, ""},
	{DS_L3MH_GRID_GEO1_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO1_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO2_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO2_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO3_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO3_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEOA_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEOA_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO1_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO1_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO2_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO2_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO3_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO3_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEOA_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEOA_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO1_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO1_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO2_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO2_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO3_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEO3_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEOA_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 830, 790,   0,AM2_GRID_GEOA_TNUM,(float)1.00, ""},

	{DS_L3MH_GRID_GEO1     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO1,(float)0.1, ""},
	{DS_L3MH_GRID_GEO2     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO2,(float)0.1, ""},
	{DS_L3MH_GRID_GEO3     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO3,(float)0.1, ""},
	{DS_L3MH_GRID_GEOA     ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEOA,(float)0.1, ""},
	{DS_L3MH_GRID_GEO1_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO1_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO2_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO2_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO3_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO3_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEOA_STD ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEOA_STD,(float)0.01, ""},
	{DS_L3MH_GRID_GEO1_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO1_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO2_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO2_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO3_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO3_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEOA_ANUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEOA_ANUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO1_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO1_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO2_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO2_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEO3_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEO3_TNUM,(float)1.00, ""},
	{DS_L3MH_GRID_GEOA_TNUM,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2,1432,1078,   0,AM2_GRID_GEOA_TNUM,(float)1.00, ""},
	{DS_END                ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0    ,(float)0.01, ""}   /* Stopper */
};

ACCESS_LABEL l3ml_table[]={
	{DS_L3ML_GRID_TBV       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3ML_GRID_TBH       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3ML_GRID_TB        ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3ML_GRID_TBV_STD   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBV_STD ,(float)0.01, ""},
	{DS_L3ML_GRID_TBH_STD   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBH_STD ,(float)0.01, ""},
	{DS_L3ML_GRID_TB_STD    ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TB_STD  ,(float)0.01, ""},
	{DS_L3ML_GRID_TBV_ANUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBV_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_TBH_ANUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBH_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_TB_ANUM   ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TB_ANUM ,(float)1.00, ""},
	{DS_L3ML_GRID_TBV_TNUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBV_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_TBH_TNUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TBH_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_TB_TNUM   ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_TB_TNUM ,(float)1.00, ""},

	{DS_L3ML_GRID_TBV       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3ML_GRID_TBH       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3ML_GRID_TB        ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3ML_GRID_TBV_STD   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBV_STD ,(float)0.01, ""},
	{DS_L3ML_GRID_TBH_STD   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBH_STD ,(float)0.01, ""},
	{DS_L3ML_GRID_TB_STD    ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TB_STD  ,(float)0.01, ""},
	{DS_L3ML_GRID_TBV_ANUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBV_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_TBH_ANUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBH_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_TB_ANUM   ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TB_ANUM ,(float)1.00, ""},
	{DS_L3ML_GRID_TBV_TNUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBV_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_TBH_TNUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TBH_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_TB_TNUM   ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_TB_TNUM ,(float)1.00, ""},

	{DS_L3ML_GRID_TBV       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBV     ,(float)0.01, "K"},
	{DS_L3ML_GRID_TBH       ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBH     ,(float)0.01, "K"},
	{DS_L3ML_GRID_TB        ,H5U_STD_U16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TB      ,(float)0.01, "K"},
	{DS_L3ML_GRID_TBV_STD   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBV_STD ,(float)0.01, ""},
	{DS_L3ML_GRID_TBH_STD   ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBH_STD ,(float)0.01, ""},
	{DS_L3ML_GRID_TB_STD    ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TB_STD  ,(float)0.01, ""},
	{DS_L3ML_GRID_TBV_ANUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBV_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_TBH_ANUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBH_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_TB_ANUM   ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TB_ANUM ,(float)1.00, ""},
	{DS_L3ML_GRID_TBV_TNUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBV_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_TBH_TNUM  ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TBH_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_TB_TNUM   ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_TB_TNUM ,(float)1.00, ""},

	{DS_L3ML_GRID_GEO1      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO1,(float)0.1, ""},
	{DS_L3ML_GRID_GEO2      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO2,(float)0.1, ""},
	{DS_L3ML_GRID_GEO3      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO3,(float)0.1, ""},
	{DS_L3ML_GRID_GEOA      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEOA,(float)0.1, ""},
	{DS_L3ML_GRID_GEO1_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO1_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO2_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO2_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO3_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO3_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEOA_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEOA_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO1_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO1_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO2_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO2_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO3_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO3_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEOA_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEOA_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO1_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO1_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO2_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO2_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO3_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEO3_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEOA_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 720,1440,   0,AM2_GRID_GEOA_TNUM,(float)1.00, ""},

	{DS_L3ML_GRID_GEO1      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO1,(float)0.1, ""},
	{DS_L3ML_GRID_GEO2      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO2,(float)0.1, ""},
	{DS_L3ML_GRID_GEO3      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO3,(float)0.1, ""},
	{DS_L3ML_GRID_GEOA      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEOA,(float)0.1, ""},
	{DS_L3ML_GRID_GEO1_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO1_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO2_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO2_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO3_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO3_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEOA_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEOA_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO1_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO1_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO2_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO2_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO3_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO3_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEOA_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEOA_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO1_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO1_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO2_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO2_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO3_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEO3_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEOA_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 448, 304,   0,AM2_GRID_GEOA_TNUM,(float)1.00, ""},

	{DS_L3ML_GRID_GEO1      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO1,(float)0.1, ""},
	{DS_L3ML_GRID_GEO2      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO2,(float)0.1, ""},
	{DS_L3ML_GRID_GEO3      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO3,(float)0.1, ""},
	{DS_L3ML_GRID_GEOA      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEOA,(float)0.1, ""},
	{DS_L3ML_GRID_GEO1_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO1_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO2_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO2_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO3_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO3_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEOA_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEOA_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO1_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO1_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO2_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO2_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO3_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO3_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEOA_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEOA_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO1_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO1_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO2_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO2_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO3_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEO3_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEOA_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 332, 316,   0,AM2_GRID_GEOA_TNUM,(float)1.00, ""},

	{DS_L3ML_GRID_GEO1      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO1,(float)0.1, ""},
	{DS_L3ML_GRID_GEO2      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO2,(float)0.1, ""},
	{DS_L3ML_GRID_GEO3      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO3,(float)0.1, ""},
	{DS_L3ML_GRID_GEOA      ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEOA,(float)0.1, ""},
	{DS_L3ML_GRID_GEO1_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO1_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO2_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO2_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO3_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO3_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEOA_STD  ,H5U_STD_I16LE ,OFF,ON ,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEOA_STD,(float)0.01, ""},
	{DS_L3ML_GRID_GEO1_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO1_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO2_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO2_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO3_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO3_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEOA_ANUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEOA_ANUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO1_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO1_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO2_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO2_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEO3_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEO3_TNUM,(float)1.00, ""},
	{DS_L3ML_GRID_GEOA_TNUM ,H5U_STD_I16LE ,ON ,OFF,OFF,OFF, 2, 573, 431,   0,AM2_GRID_GEOA_TNUM,(float)1.00, ""},
	{DS_END                 ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0                  ,(float)0.01, ""} /* Stopper */
};

ACCESS_LABEL latlon_table[]={
	{DS_LATLON_06          ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_LATLON_06    ,(float)1.00, "deg"},
	{DS_LATLON_07          ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_LATLON_07    ,(float)1.00, "deg"},
	{DS_LATLON_10          ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_LATLON_10    ,(float)1.00, "deg"},
	{DS_LATLON_18          ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_LATLON_18    ,(float)1.00, "deg"},
	{DS_LATLON_23          ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_LATLON_23    ,(float)1.00, "deg"},
	{DS_LATLON_36          ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_LATLON_36    ,(float)1.00, "deg"},
	{DS_LATLON_89A         ,H5U_IEEE_F32LE,OFF,ON, OFF,OFF, 2,2018, 486,   0, AM2_LATLON_89A   ,(float)1.00, "deg"},
	{DS_LATLON_89B         ,H5U_IEEE_F32LE,OFF,ON, OFF,OFF, 2,2018, 486,   0, AM2_LATLON_89B   ,(float)1.00, "deg"},
	{DS_LATLON_LO          ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_LATLON_LO    ,(float)1.00, "deg"},
	{DS_LATLON_RS_89A      ,H5U_IEEE_F32LE,OFF,ON, OFF,OFF, 2,2018, 486,   0, AM2_LATLON_RS_89A,(float)1.00, "deg"},
	{DS_LATLON_RS_89B      ,H5U_IEEE_F32LE,OFF,ON, OFF,OFF, 2,2018, 486,   0, AM2_LATLON_RS_89B,(float)1.00, "deg"},
	{DS_LATLON_RS_LO       ,H5U_IEEE_F32LE,OFF,ON, OFF,OFF, 2,2018, 486,   0, AM2_LATLON_RS_LO ,(float)1.00, "deg"},
	{DS_LATLON_L2_LO       ,H5U_IEEE_F32LE,OFF,ON, OFF,OFF, 2,2018, 486,   0, AM2_LATLON_L2_LO ,(float)1.00, "deg"},
	{DS_LATLON_L2_89A      ,H5U_IEEE_F32LE,OFF,ON, OFF,OFF, 2,2018, 486,   0, AM2_LATLON_L2_89A,(float)1.00, "deg"},
	{DS_LATLON_L2_89B      ,H5U_IEEE_F32LE,OFF,ON, OFF,OFF, 2,2018, 486,   0, AM2_LATLON_L2_89B,(float)1.00, "deg"},
	{DS_GRID_LATLON        ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_GRID_LATLON  ,(float)1.00, "deg"},
	{DS_GRID_LATLON        ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_GRID_LATLON  ,(float)1.00, "deg"},
	{DS_GRID_LATLON        ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_GRID_LATLON  ,(float)1.00, "deg"},
	{DS_GRID_LATLON        ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_GRID_LATLON  ,(float)1.00, "deg"},
	{DS_GRID_LATLON        ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_GRID_LATLON  ,(float)1.00, "deg"},
	{DS_GRID_LATLON        ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0, AM2_GRID_LATLON  ,(float)1.00, "deg"},
	{DS_END                ,0             ,0  ,0  ,0  ,0  , 0,   0,   0,   0,0               ,(float)0.00, ""} /* Stopper */
};

/** Access label table. */
ACCESS_LABEL *label_table[] = 
{
	l1a_table,    /* DS_TABLE_L1A */
	l1b_table,    /* DS_TABLE_L1B */
	l1r_table,    /* DS_TABLE_L1R */
	l2_table,     /* DS_TABLE_L2 */
	l3dh_table,   /* DS_TABLE_L3D_H */
	l3dl_table,   /* DS_TABLE_L3D_L */
	l3mh_table,   /* DS_TABLE_L3M_H */
	l3ml_table,   /* DS_TABLE_L3M_L */
	latlon_table  /* unused. -> @see AMTK_setDimSize(), AMTK_getDimSize() */
};

/** Dataset name table. */
char **name_table[] = 
{
	dataset_name_l1a,
	dataset_name_l1b,
	dataset_name_l1r,
	dataset_name_l2,
	dataset_name_l3dh,
	dataset_name_l3dl,
	dataset_name_l3mh,
	dataset_name_l3ml,
	latlon_name_lo  /* unused. -> @see AMTK_setDimSize(), AMTK_getDimSize() */
};

/** Access label table index. */
enum DS_TABLE_INDEX
{
	DS_TABLE_L1A,
	DS_TABLE_L1B,
	DS_TABLE_L1R,
	DS_TABLE_L2,
	DS_TABLE_L3D_H,
	DS_TABLE_L3D_L,
	DS_TABLE_L3M_H,
	DS_TABLE_L3M_L,
	DS_TABLE_LATLON  /* unused. -> @see AMTK_setDimSize(), AMTK_getDimSize() */
};


/** @file amtk_hdf.c
 * Get L3 Access label table.
 *
 * @author     NEC
 * @date       2010/11/30
 * @return     Access Label(NULL : not found)
 */
static ACCESS_LABEL * get_L3_access_table(hid_t HDF_file_id, int *p_table_num)
{
	/* Return code. */
	int result = 0;
	/* Value of GranuleID(auto alloc). */
	char *p_granuleid = NULL;
	/* Access label table. */
	ACCESS_LABEL *p_table = NULL;
	
	
	/* Get meta "GranuleID". */
	result = getMetaDataInside(HDF_file_id, AM2_DEF_META_GRANULEID, &p_granuleid,
		ERROR_HDF_METAOP_GRANULE, ERROR_HDF_METANG_GRANULE);
	if (0 > result)
	{
		/* Not found. */
		free(p_granuleid);
		*p_table_num = -1;
		return NULL;
	}
	else if (GRANULEID_RES_POS >= strlen(p_granuleid))
	{
		/* NG data. */
		free(p_granuleid);
		*p_table_num = -1;
		return NULL;
	}
	
	/* Check daily/monthly, high res/low res */
	if (GRANULEID_SPD_DAY == p_granuleid[GRANULEID_SPD_POS])
	{
		if (GRANULEID_RES_HI == p_granuleid[GRANULEID_RES_POS])
		{
			/* L3 Data (Dayly, High res) */
			p_table = l3dh_table;
			*p_table_num = DS_TABLE_L3D_H;
		}
		else if (GRANULEID_RES_LO == p_granuleid[GRANULEID_RES_POS])
		{
			/* L3 Data (Dayly, Low res) */
			p_table = l3dl_table;
			*p_table_num = DS_TABLE_L3D_L;
		}
		else
		{
			/* NG data. */
			p_table = NULL;
			*p_table_num = -1;
		}
	}
	else if (GRANULEID_SPD_MON == p_granuleid[GRANULEID_SPD_POS])
	{
		if (GRANULEID_RES_HI == p_granuleid[GRANULEID_RES_POS])
		{
			/* L3 Data (Monthly, High res) */
			p_table = l3mh_table;
			*p_table_num = DS_TABLE_L3M_H;
		}
		else if (GRANULEID_RES_LO == p_granuleid[GRANULEID_RES_POS])
		{
			/* L3 Data (Monthly, Low res) */
			p_table = l3ml_table;
			*p_table_num = DS_TABLE_L3M_L;
		}
		else
		{
			/* NG data. */
			p_table = NULL;
			*p_table_num = -1;
		}
	}
	
	free(p_granuleid);
	
	/* Return global pointer. */
	return p_table;

}


/** @file amtk_hdf.c
 * Get Access label table.
 *
 * @author     NEC
 * @date       2010/11/30
 * @return     Access Label(NULL : not found)
 *
 * 2014/05     RESTEC/NEC modified
 */
static ACCESS_LABEL * get_access_table(hid_t HDF_file_id, int dataset_no, int *p_table_num)
{
	/* Counter. */
	int i = 0;
	/* Return code. */
	int result = 0;
	/* Value of ProductName(auto alloc). */
	char *p_product_name = NULL;
	/* Access label table. */
	ACCESS_LABEL *p_table = NULL;
	
	/* Get meta "ProductName". */
	result = getMetaDataInside(HDF_file_id, AM2_DEF_META_PRODUCTNAME, &p_product_name,
		ERROR_HDF_METAOP_PRDCTNAME, ERROR_HDF_METANG_PRDCTNAME);
	if (0 > result)
	{
		/* Not found. */
		free(p_product_name);
		*p_table_num = -1;
		return NULL;
	}
	
	if (0 == strcmp(p_product_name, PRODUCTNAME_L1A) ||
            0 == strncmp(&p_product_name[strlen(p_product_name)-3], "L1A",3))  /* 2014/05 RESTEC/NEC modified */
	{
		/* L1A Data */
		p_table = l1a_table;
		*p_table_num = DS_TABLE_L1A;
	}
	else if (0 == strcmp(p_product_name, PRODUCTNAME_L1B) ||
                 0 == strncmp(&p_product_name[strlen(p_product_name)-3], "L1B",3))  /* 2014/05 RESTEC/NEC modified */
	{
		/* L1B Data */
		p_table = l1b_table;
		*p_table_num = DS_TABLE_L1B;
	}
	else if (0 == strcmp(p_product_name, PRODUCTNAME_L1R) ||
                 0 == strncmp(&p_product_name[strlen(p_product_name)-3], "L1R",3))  /* 2014/05 RESTEC/NEC modified */
	{
		/* L1R Data */
		p_table = l1r_table;
		*p_table_num = DS_TABLE_L1R;
	}
	else if (0 == strcmp(p_product_name, PRODUCTNAME_L2) ||
                 0 == strncmp(&p_product_name[strlen(p_product_name)-2], "L2",2))   /* 2014/05 RESTEC/NEC modified */
	{
		/* L2 Data */
		p_table = l2_table;
		*p_table_num = DS_TABLE_L2;
	}
	else if (0 == strcmp(p_product_name, PRODUCTNAME_L3) ||
                 0 == strncmp(&p_product_name[strlen(p_product_name)-2], "L3",2))   /* 2014/05 RESTEC/NEC modified */
	{
		/* L3 Data */
		p_table = get_L3_access_table(HDF_file_id, p_table_num);
	}
	else
	{
		/* Not found. */
		free(p_product_name);
		*p_table_num = -1;
		return NULL;
	}
	
	/* Free memory. */
	free(p_product_name);
	p_product_name = NULL;
	
	/* Search dataset. */
	if (NULL != p_table)
	{
		for (i = 0; DS_END != p_table[i].dataset_no; i++)
		{
			if (dataset_no == p_table[i].HDF_ref_no)
			{
				/* Found. */
				/* Return global pointer. */
				return &p_table[i];
			}
		}
	}
	
	/* Not found. */
	
	/* Search Lat&Lon dataset */
	for (i = 0; DS_END != latlon_table[i].dataset_no; i++)
	{
		if (dataset_no == latlon_table[i].HDF_ref_no)
		{
			/* Found. */
			*p_table_num = DS_TABLE_LATLON;
			/* Return global pointer. */
			return &latlon_table[i];
		}
	}
	
	/* Not found. */
	*p_table_num = -1;
	return NULL;
}


/** @file amtk_hdf.c
 * Access label table search
 *
 * @author     NEC
 * @date       2010/11/30
 */
ACCESS_LABEL *search_table(hid_t HDF_file_id, int dataset_no, int *table_num)
{
	/* Counter. */
	int i = 0, j = 0;
	/* Access label table. */
	ACCESS_LABEL *p_table = NULL;
	/* Size of access label table. */
	const size_t table_size = sizeof(label_table) / sizeof(label_table[0]);
	
	/* Get index of Access label table. */
	p_table = get_access_table(HDF_file_id, dataset_no, table_num);
	if (NULL != p_table)
	{
		/* Found. */
		/* Return global pointer. */
		return p_table;
	}
	
	/* Not found. -> Check all product table. */
	p_table = NULL;
	
	for (i = 0; i < table_size; i++)
	{
		p_table = label_table[i];
		for (j = 0; DS_END != p_table[j].dataset_no; j++)
		{
			if (dataset_no == p_table[j].HDF_ref_no)
			{
				/* Found. */
				*table_num = i;
				/* Return global pointer. */
				return &p_table[j];
			}
		}
	}
	
	/* Not found. */
	return NULL;
}

/** @file amtk_hdf.c
 * Get scale factor
 *
 * @author     NEC
 * @date       2011/06/13
 */
int getScaleFactor(hid_t dataset, float *scale)
{
	hid_t			attr = 0;
	hid_t			atype = 0;
	herr_t			ret = 0;
	union unit1		sf;

	attr = H5Aopen_name(dataset, "SCALE FACTOR");
	if(attr < 0)
	{
		IOTK_printf("SF ERROR : 1\n");
		return ERROR_HDF_ATTR_NAME_OPEN;
	}

	atype = H5Aget_type(attr);
	if(H5T_FLOAT == H5Tget_class(atype))
	{
		ret = H5Aread(attr, H5T_NATIVE_FLOAT, &sf);
		if(ret < 0)
		{
			IOTK_printf("SF ERROR : 3\n");
			/* close datatype */
			H5Tclose(atype);
			/* close attribute */
			H5Aclose(attr);
			return ERROR_HDF_ATTR_READ;
		}
	}
	/* close datatype */
	H5Tclose(atype);
	/* close attribute */
	H5Aclose(attr);
	*scale = sf.b;
	return SUCCESS;
}

/** @file amtk_hdf.c
 * Check write NG
 *
 * @author     NEC
 * @date       2010/06/30
 */
int checkWriteNG(int dataset_no) 
{
	int	ret = 0;

	switch(dataset_no)
	{
		case AM2_OC_LO:
		case AM2_OC_89A:
		case AM2_OC_89B:
		case AM2_TB_LO:
		case AM2_TB_89A:
		case AM2_TB_89B:
		case AM2_RES06_TB_ALL:
		case AM2_RES10_TB_ALL:
		case AM2_RES23_TB_ALL:
		case AM2_RES36_TB_ALL:
		case AM2_GRID_TB:
		case AM2_GRID_TB_STD:
		case AM2_GRID_TB_ANUM:
		case AM2_GRID_TB_TNUM:
		case AM2_GRID_LATLON:
		case AM2_LATLON_LO:
		case AM2_LATLON_06:
		case AM2_LATLON_07:
		case AM2_LATLON_10:
		case AM2_LATLON_18:
		case AM2_LATLON_23:
		case AM2_LATLON_36:
		case AM2_LATLON_RS_LO:
			ret = ERROR_ACCESSLABEL_WRITE_NG;
			break;
		default:
			ret = SUCCESS;
			break;
	}
	return ret;
}

/** @file amtk_hdf.c
 * AccessLabel datatype check
 *
 * @author     NEC
 * @date       2010/07/30
 */
int AccessLabelTypeCheck(hid_t HDF_file_id, unsigned int  datatype, int dataset_no)
{
	ACCESS_LABEL	*p_table  = NULL;
	int          table_num = 0;
	unsigned int checkType = 0;

	p_table = search_table(HDF_file_id, dataset_no, &table_num);
	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	switch(datatype)
	{
		case AM2_DTYPE_INT:
			checkType = p_table->out_datatype1;
			break;
		case AM2_DTYPE_FLOAT:
			checkType = p_table->out_datatype2;
			break;
		case AM2_DTYPE_DOUBLE:
			checkType = p_table->out_datatype3;
			break;
		case AM2_DTYPE_UCHAR:
			checkType = p_table->out_datatype4;
			break;
		default:
			break;
	}
	if(checkType == OFF)
	{
			return ERROR_ACCESSLABEL_UNMATCH;
	}
	return SUCCESS;
}

/** @file amtk_hdf.c
 * Common parameter check
 *
 * @author     NEC
 * @date       2010/06/30
 */
int CommonPrmCheck(hid_t HDF_file_id, int from_scan, int to_scan, int dataset_no)
{
	if((from_scan != AM2_SCANNUM_NOCHK) && (to_scan != AM2_SCANNUM_NOCHK))
	{
		if(from_scan > to_scan)
		{
			return ERROR_ILLEGAL_SCANNUM_S;
		}
	}
	if(HDF_file_id == 0)
	{
		return ERROR_ILLEGAL_HDF_FILEID;
	}
	if(dataset_no == 0)
	{
		return ERROR_NOT_ACCESSLABEL;
	}
	return SUCCESS;
}

/** @file amtk_hdf.c
 * Check access label
 *
 * @author     NEC
 * @date       2010/06/30
 */
void checkAccessLabel(int dataset_no, GET_HDFDATA *hdfdata)
{
	switch(dataset_no)
	{
		case AM2_OC_LO:
			hdfdata->datacnt = 12;
			hdfdata->data[0].dataset_no  = AM2_OC06V;
			hdfdata->data[1].dataset_no  = AM2_OC06H;
			hdfdata->data[2].dataset_no  = AM2_OC07V;
			hdfdata->data[3].dataset_no  = AM2_OC07H;
			hdfdata->data[4].dataset_no  = AM2_OC10V;
			hdfdata->data[5].dataset_no  = AM2_OC10H;
			hdfdata->data[6].dataset_no  = AM2_OC18V;
			hdfdata->data[7].dataset_no  = AM2_OC18H;
			hdfdata->data[8].dataset_no  = AM2_OC23V;
			hdfdata->data[9].dataset_no  = AM2_OC23H;
			hdfdata->data[10].dataset_no = AM2_OC36V;
			hdfdata->data[11].dataset_no = AM2_OC36H;
			break;
		case AM2_OC_89A:
			hdfdata->datacnt = 2;
			hdfdata->data[0].dataset_no = AM2_OC89AV;
			hdfdata->data[1].dataset_no = AM2_OC89AH;
			break;
		case AM2_OC_89B:
			hdfdata->datacnt = 2;
			hdfdata->data[0].dataset_no = AM2_OC89BV;
			hdfdata->data[1].dataset_no = AM2_OC89BH;
			break;
		case AM2_TB_LO:
			hdfdata->datacnt = 12;
			hdfdata->data[0].dataset_no  = AM2_TB06V;
			hdfdata->data[1].dataset_no  = AM2_TB06H;
			hdfdata->data[2].dataset_no  = AM2_TB07V;
			hdfdata->data[3].dataset_no  = AM2_TB07H;
			hdfdata->data[4].dataset_no  = AM2_TB10V;
			hdfdata->data[5].dataset_no  = AM2_TB10H;
			hdfdata->data[6].dataset_no  = AM2_TB18V;
			hdfdata->data[7].dataset_no  = AM2_TB18H;
			hdfdata->data[8].dataset_no  = AM2_TB23V;
			hdfdata->data[9].dataset_no  = AM2_TB23H;
			hdfdata->data[10].dataset_no = AM2_TB36V;
			hdfdata->data[11].dataset_no = AM2_TB36H;
			break;
		case AM2_TB_89A:
			hdfdata->datacnt = 2;
			hdfdata->data[0].dataset_no = AM2_TB89AV;
			hdfdata->data[1].dataset_no = AM2_TB89AH;
			break;
		case AM2_TB_89B:
			hdfdata->datacnt = 2;
			hdfdata->data[0].dataset_no = AM2_TB89BV;
			hdfdata->data[1].dataset_no = AM2_TB89BH;
			break;
		case AM2_RES06_TB_ALL:
			hdfdata->datacnt = 14;
			hdfdata->data[0].dataset_no  = AM2_RES06_TB06V;
			hdfdata->data[1].dataset_no  = AM2_RES06_TB06H;
			hdfdata->data[2].dataset_no  = AM2_RES06_TB07V;
			hdfdata->data[3].dataset_no  = AM2_RES06_TB07H;
			hdfdata->data[4].dataset_no  = AM2_RES06_TB10V;
			hdfdata->data[5].dataset_no  = AM2_RES06_TB10H;
			hdfdata->data[6].dataset_no  = AM2_RES06_TB18V;
			hdfdata->data[7].dataset_no  = AM2_RES06_TB18H;
			hdfdata->data[8].dataset_no  = AM2_RES06_TB23V;
			hdfdata->data[9].dataset_no  = AM2_RES06_TB23H;
			hdfdata->data[10].dataset_no = AM2_RES06_TB36V;
			hdfdata->data[11].dataset_no = AM2_RES06_TB36H;
			hdfdata->data[12].dataset_no = AM2_RES06_TB89V;
			hdfdata->data[13].dataset_no = AM2_RES06_TB89H;
			break;
		case AM2_RES10_TB_ALL:
			hdfdata->datacnt = 10;
			hdfdata->data[0].dataset_no = AM2_RES10_TB10V;
			hdfdata->data[1].dataset_no = AM2_RES10_TB10H;
			hdfdata->data[2].dataset_no = AM2_RES10_TB18V;
			hdfdata->data[3].dataset_no = AM2_RES10_TB18H;
			hdfdata->data[4].dataset_no = AM2_RES10_TB23V;
			hdfdata->data[5].dataset_no = AM2_RES10_TB23H;
			hdfdata->data[6].dataset_no = AM2_RES10_TB36V;
			hdfdata->data[7].dataset_no = AM2_RES10_TB36H;
			hdfdata->data[8].dataset_no = AM2_RES10_TB89V;
			hdfdata->data[9].dataset_no = AM2_RES10_TB89H;
			break;
		case AM2_RES23_TB_ALL:
			hdfdata->datacnt = 8;
			hdfdata->data[0].dataset_no = AM2_RES23_TB18V;
			hdfdata->data[1].dataset_no = AM2_RES23_TB18H;
			hdfdata->data[2].dataset_no = AM2_RES23_TB23V;
			hdfdata->data[3].dataset_no = AM2_RES23_TB23H;
			hdfdata->data[4].dataset_no = AM2_RES23_TB36V;
			hdfdata->data[5].dataset_no = AM2_RES23_TB36H;
			hdfdata->data[6].dataset_no = AM2_RES23_TB89V;
			hdfdata->data[7].dataset_no = AM2_RES23_TB89H;
			break;
		case AM2_RES36_TB_ALL:
			hdfdata->datacnt = 4;
			hdfdata->data[0].dataset_no = AM2_RES36_TB36V;
			hdfdata->data[1].dataset_no = AM2_RES36_TB36H;
			hdfdata->data[2].dataset_no = AM2_RES36_TB89V;
			hdfdata->data[3].dataset_no = AM2_RES36_TB89H;
			break;
		case AM2_GRID_TB:
			hdfdata->datacnt = 2;
			hdfdata->data[0].dataset_no = AM2_GRID_TBV;
			hdfdata->data[1].dataset_no = AM2_GRID_TBH;
			break;
		case AM2_GRID_TB_STD:
			hdfdata->datacnt = 2;
			hdfdata->data[0].dataset_no = AM2_GRID_TBV_STD;
			hdfdata->data[1].dataset_no = AM2_GRID_TBH_STD;
			break;
		case AM2_GRID_TB_ANUM:
			hdfdata->datacnt = 2;
			hdfdata->data[0].dataset_no = AM2_GRID_TBV_ANUM;
			hdfdata->data[1].dataset_no = AM2_GRID_TBH_ANUM;
			break;
		case AM2_GRID_TB_TNUM:
			hdfdata->datacnt = 2;
			hdfdata->data[0].dataset_no = AM2_GRID_TBV_TNUM;
			hdfdata->data[1].dataset_no = AM2_GRID_TBH_TNUM;
			break;
		default:
			hdfdata->datacnt = 1;
			hdfdata->data[0].dataset_no = dataset_no;
			break;
	}
	return;
}

/** @file amtk_hdf.c
 * Get Meta data (AMTK inside).
 * Notify the user the error number.
 *
 * @author     NEC
 * @date       2010/12/27
 */
int getMetaDataInside(hid_t HDF_file_id, char *p_name, char **p_value, int open_err_no, int hdf_err_no)
{
	int ret = 0;
	
	ret = AMTK_getMetaDataName(HDF_file_id, p_name, p_value);
	if (0 > ret)
	{
		/* Error case */
		if (ERROR_HDF_ATTR_NAME_OPEN == ret)
		{
			/* open error */
			ret = open_err_no;
		}
		else if ((-500 >= ret) && (-600 < ret))
		{
			/* -500 ... -599 : HDF error */
			ret = hdf_err_no;
		}
		/*
		 * other:
		 * ERROR_ILLEGAL_HDF_FILEID
		 * ERROR_NULL_ADDRESS
		 * ERROR_MALLOC
		 */
	}
	return ret;
}


/** @file amtk_hdf.c
 * Get overlap scan
 *
 * @author     NEC
 * @date       2010/06/30
 *
 * 2014/05     RESTEC/NEC modified
 *
 */
int	getOverLapScans(hid_t HDF_file_id, int *overlapscan)
{
	char	value[80];
	char	name[80];
	int		ret = 0;
	int		ret_l3check = 0;
    char    *p_dmybuf = NULL;

	memset(&value, 0x00, sizeof(value));

	strcpy(name, AM2_DEF_META_OVERLAPSCANS);
	p_dmybuf = &value[0];
	
	/* Get "OverlapScans" */
	ret = getMetaDataInside(HDF_file_id, &name[0], &p_dmybuf,
		ERROR_HDF_METAOP_OVERLAP, ERROR_HDF_METANG_OVERLAP);
	if(ret < 0)
	{
		strcpy(name, AM2_DEF_META_PRODUCTNAME);
	    p_dmybuf = &value[0];
		
		/* Check L3 */
		ret_l3check = getMetaDataInside(HDF_file_id, &name[0], &p_dmybuf,
			ERROR_HDF_METAOP_PRDCTNAME, ERROR_HDF_METANG_PRDCTNAME);
		if (ret_l3check > 0)
		{
			if (0 == strncmp(PRODUCTNAME_L3, value, strlen(PRODUCTNAME_L3)) ||
                            0 == strncmp("L3", &value[strlen(value)-2], 2))                /* 2014/05  RESTEC/NEC modified */
			{
				*overlapscan = 0;
				return SUCCESS;
			}
		}
		return ret;
	}
	*overlapscan = atoi(value);
	return SUCCESS;
}

/** @file amtk_hdf.c
 * Get attribute.
 *
 * @author     NEC
 * @date       2011/06/13
 */
int getAttribute(hid_t attr, char **value)
{
	/* Attribute, datatype, dataspace identifiers. */
	hid_t  atype = 0, space = 0;  
	/* Datatype class. */
	hid_t  tclass = 0;
	/* Return code. */
	herr_t ret = 0;
	/* flag indicating if datatype is valiable-length strings. */
	htri_t is_val_string;
	/* Metadata dimension. */
	hsize_t meta_length = 0;
	/* Memory Allocation Flag. */
	char   mallocflg = IOTK_OFF;
	/* valiable-length Buffer. */
	char   *p_val_value[1] = {0};

	/* Get datatype. */
	atype = H5Aget_type(attr);
	if (ret < 0)
	{
		IOTK_printf("getMetaDataName ERROR - 2\n");
		/* Close Datatype. */
		H5Tclose(atype);
		return ERROR_HDF_DT_SET_SIZE;
	}
	
	/* Get valiable-length strings Flag. */
	is_val_string = H5Tis_variable_str(atype);
	if (is_val_string < 0)
	{
		IOTK_printf("getMetaDataName ERROR - 3\n");
		/* Close Datatype. */
		H5Tclose(atype);
		return ERROR_HDF_DT_SET_SIZE;
	}
	
	
	/* Check class */
	tclass = H5Tget_class(atype);
	if (H5T_STRING == tclass)
	{
		/* Get Dataspace. */
		space = H5Aget_space(attr);
		if (space < 0)
		{
			IOTK_printf("getAttribute ERROR - 4\n");
			/* Close Datatype. */
			H5Tclose(atype);
			return ERROR_HDF_ATTR_READ;
		}
		
		if (is_val_string)
		{
			/* Read valiable-length strings. */
			ret = H5Aread(attr, atype, p_val_value);
		}
		else
		{
			/* Read strings. */
			if (*value == NULL)
			{
				/* Memory Allocation. */
				meta_length = H5Tget_size(atype);
				
				*value = (char *) malloc(meta_length + 1);
				if (*value == NULL)
				{
					IOTK_printf("getAttribute ERROR - 5\n");
					/* Close Dataspece. */
					H5Sclose(space);
					/* Close Datatype. */
					H5Tclose(atype);
					return ERROR_MALLOC;
				}
				/* malloc flag */
				mallocflg = IOTK_ON;
			}
			ret = H5Aread(attr, atype, *value);
		}
		
		if (ret < 0)
		{
			IOTK_printf("getAttribute ERROR - 6\n");
			/* Close Dataspece. */
			H5Sclose(space);
			/* Close Datatype. */
			H5Tclose(atype);
			/* Free memory. */
			if (mallocflg == IOTK_ON)
			{
				free(*value);
				*value = NULL;
			}
			return ERROR_HDF_ATTR_READ;
		}
		
		if (is_val_string)
		{
			if (*value == NULL)
			{
				/* Memory Allocation(valiable-length). */
				meta_length = strlen(p_val_value[0]);
				
				*value = (char *) malloc(meta_length + 1);
				if (*value == NULL)
				{
					IOTK_printf("getAttribute ERROR - 7\n");
					/* Close Dataspece. */
					H5Sclose(space);
					/* Close Datatype. */
					H5Tclose(atype);
					return ERROR_MALLOC;
				}
				/* malloc flag */
				mallocflg = IOTK_ON;
			}
			
			/* Copy String */
			strcpy(*value, p_val_value[0]);
			
			/* Free valiable-length strings.  */
			ret = H5Dvlen_reclaim(atype, space, H5P_DEFAULT, p_val_value);
			if (ret < 0)
			{
				IOTK_printf("getAttribute ERROR - 8\n");
				/* Close Dataspece. */
				H5Sclose(space);
				/* Close Datatype. */
				H5Tclose(atype);
				/* Free memory. */
				if (mallocflg == IOTK_ON)
				{
					free(*value);
					*value = NULL;
				}
				return ERROR_HDF_ATTR_READ;
			}
		}
		
		/* Close Dataspece. */
		H5Sclose(space);
		
	}
	else
	{
		IOTK_printf("getAttribute ERROR - 9\n");
		/* Close Datatype. */
		H5Tclose(atype);
		return ERROR_HDF_DT_GET_CLASS;
	}
	
	/* Close Datatype. */
	H5Tclose(atype);
	
	return SUCCESS;
}

/** @file amtk_hdf.c
 * Get dataset
 *
 * @author     NEC
 * @date       2011/06/13
 */
int getDataSet(hid_t HDF_file_id, void **data, int from_scan, int to_scan, int dataset_no, int datatype)
{

	ACCESS_LABEL	*p_table=NULL;
	int			ret = 0;
	int			table_num = 0;
	int			dataset = 0;
	int			overlap = 0;
	void		*p_dmydata = NULL;
	void		*p_rdata = NULL;
	int			cnt = 0;
	int			cnt1 = 0;
	int			cnt2 = 0;
	GET_HDFDATA hdfdata;
	long		mallocsize = 0L;
	long		writePnt = 0L;
	char		shiftCnt = 0;
	char		mallocflg = IOTK_OFF;
	hid_t   	filespace = 0;
	hid_t   	memspace = 0;
	hid_t   	getdatatype = 0;
	hid_t   	ndatatype = 0;
	int			readDataCnt = 0;
	hid_t   	chgdatatype = 0;
	int			datasize = 0;
	int			writeCnt = 0;
	int			getmemsize = 0;
	int			chkflg = 0;
	int			allDataCnt = 0;
	float		scale = 0.0;
	float		fans = 0.0;
	double		dans = 0.0;

	ret = AccessLabelTypeCheck(HDF_file_id, datatype, dataset_no);
	if(ret < 0)
	{
		return ret;
	}

	switch(dataset_no)
	{
		case AM2_SWATH_GEO1:
		case AM2_SWATH_GEO2:
		case AM2_SWATH_GEO3:
		case AM2_SWATH_GEOA:
		case AM2_SWATHA_GEO1:
		case AM2_SWATHA_GEO2:
		case AM2_SWATHA_GEO3:
		case AM2_SWATHA_GEOA:
		case AM2_SWATHB_GEO1:
		case AM2_SWATHB_GEO2:
		case AM2_SWATHB_GEO3:
		case AM2_SWATHB_GEOA:
			ret = getGridDataSet(HDF_file_id, data, from_scan, to_scan, dataset_no, datatype);
			return ret;

		default:
			break;
	}

	checkAccessLabel(dataset_no, &hdfdata);

	for(cnt = 0; cnt < hdfdata.datacnt; cnt++)
	{
		hdfdata.data[cnt].rank = AMTK_getDimSize( HDF_file_id, hdfdata.data[cnt].dataset_no , &hdfdata.data[cnt].size[0]);
		if(hdfdata.data[cnt].rank < 0)
		{
			return hdfdata.data[cnt].rank;
		}
		if(datatype == AM2_DTYPE_INT)
		{
			if( dataset_no == AM2_PIX_QUAL_LO)
			{
				hdfdata.data[cnt].datasize = (sizeof(short)*16);
			}
			else if( dataset_no == AM2_PIX_QUAL_HI)
			{
				hdfdata.data[cnt].datasize = (sizeof(short)*8);
			}
			else
			{
				hdfdata.data[cnt].datasize = sizeof(short);
				hdfdata.data[cnt].ntype = H5T_NATIVE_SHORT;
			}
		}
		else if(datatype == AM2_DTYPE_FLOAT)
		{
			hdfdata.data[cnt].datasize = sizeof(float);
			hdfdata.data[cnt].ntype = H5T_NATIVE_FLOAT;
		}
		else if(datatype == AM2_DTYPE_DOUBLE)
		{
			hdfdata.data[cnt].datasize = sizeof(double);
			hdfdata.data[cnt].ntype = H5T_NATIVE_DOUBLE;
		}
		else
		{
			hdfdata.data[cnt].datasize = sizeof(unsigned char);
			hdfdata.data[cnt].ntype = H5T_NATIVE_UCHAR;
		}
		ret = getOverLapScans( HDF_file_id, &overlap);
		if(ret != SUCCESS)
		{
			return ret;
		}
		hdfdata.data[cnt].datasetsize = getMemorySize(&hdfdata.data[cnt].size[0],
														hdfdata.data[cnt].rank,
														hdfdata.data[cnt].datasize,
														from_scan,
														to_scan);
		if(hdfdata.data[cnt].datasetsize == 0)
		{
			return ERROR_MEMSIZE;
		}
		mallocsize += hdfdata.data[cnt].datasetsize;
	}
	if(*data == NULL)
	{
		*data = (void *)malloc(mallocsize);
		if(*data == NULL)
		{
			return ERROR_MALLOC;
		}
		mallocflg = IOTK_ON;
	}

	for(cnt = 0; cnt < hdfdata.datacnt; cnt++)
	{
		p_table = search_table(HDF_file_id, hdfdata.data[cnt].dataset_no, &table_num);
		if(p_table == NULL)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			return ERROR_NOT_ACCESSLABEL;
		}

		ret = getOverLapScans( HDF_file_id, &overlap);
		if(ret != SUCCESS)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			return ret;
		}

		dataset = H5Dopen(HDF_file_id, name_table[table_num][p_table->dataset_no], H5P_DEFAULT);
		if(dataset < 0)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			return ERROR_HDF_DATASET_OPEN;
		}
		ret = getScaleFactor(dataset, &scale);
		if(ret != SUCCESS)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			H5Dclose(dataset);
			return ret;
		}

		getdatatype = H5Dget_type (dataset);
		if(getdatatype < 0)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			H5Dclose(dataset);
			return ERROR_HDF_DATASET_GET_TYPE;
		}
		ndatatype= H5Tget_native_type(getdatatype, H5T_DIR_ASCEND);
		
		/* close datatype */
		H5Tclose(getdatatype);
		
	    if(ndatatype < 0)
	    {
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			H5Dclose(dataset);
		    return ERROR_HDF_DT_GET_TYPE;
		}
		if( H5Tequal( ndatatype, H5T_NATIVE_FLOAT))
		{
			chgdatatype = H5T_NATIVE_FLOAT;
			datasize = sizeof(float);
			chkflg = 1;
		}
		else if( H5Tequal( ndatatype, H5T_NATIVE_DOUBLE))
		{
			chgdatatype = H5T_NATIVE_DOUBLE;
			datasize = sizeof(double);
			chkflg = 2;
		}
		else if( H5Tequal( ndatatype, H5T_NATIVE_SHORT))
		{
			chgdatatype = H5T_NATIVE_SHORT;
			datasize = sizeof(short);
			chkflg = 3;
		}
		else if( H5Tequal( ndatatype, H5T_NATIVE_USHORT))
		{
			chgdatatype = H5T_NATIVE_USHORT;
			datasize = sizeof(unsigned short);
			chkflg = 4;
		}
		else if( H5Tequal( ndatatype, H5T_NATIVE_SCHAR))
		{
			chgdatatype = H5T_NATIVE_SCHAR;
			datasize = sizeof(char);
			chkflg = 5;
		}
		else if( H5Tequal( ndatatype, H5T_NATIVE_UCHAR))
		{
			chgdatatype = H5T_NATIVE_UCHAR;
			datasize = sizeof(unsigned char);
			chkflg = 6;
		}
		else if( H5Tequal( ndatatype, H5T_NATIVE_INT))
		{
			chgdatatype = H5T_NATIVE_INT;
			datasize = sizeof(int);
			chkflg = 7;
		}
		else if( H5Tequal( ndatatype, H5T_NATIVE_UINT))
		{
			chgdatatype = H5T_NATIVE_UINT;
			datasize = sizeof(unsigned int);
			chkflg = 8;
		}
		
		/* close datatype */
		H5Tclose(ndatatype);
		
		p_rdata  = (void *)getMemoryAlloc(&hdfdata.data[cnt].size[0], hdfdata.data[cnt].rank, datasize, from_scan, to_scan, &getmemsize);
		if(p_rdata == NULL)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			H5Dclose(dataset);
			return ERROR_MALLOC;
		}
		ret = setHyperslab(dataset, from_scan, to_scan, &hdfdata.data[cnt].size[0], hdfdata.data[cnt].rank, overlap, &memspace, &filespace);
		if(ret < 0)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			free(p_rdata);
			H5Dclose(dataset);
			return ret;
	    }
		ret = H5Dread(dataset,		    		/* hid_t dset_id        */
					  chgdatatype,				/* hid_t mem_type_id    */
					  memspace,					/* hid_t mem_space_id   */
					  filespace,				/* hid_t file_space_id  */
					  H5P_DEFAULT,				/* hid_t plist_id       */
					  p_rdata);					/* void *buf   *out*    */
		
		/* close dataspace */
		H5Sclose(memspace);
		H5Sclose(filespace);
		
		if(ret < 0)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			free(p_rdata);
			H5Dclose(dataset);
			return ERROR_HDF_DATASET_READ;
		}

		readDataCnt = (getmemsize / datasize);

		if(datatype == AM2_DTYPE_INT)
		{
			p_dmydata = ((int *)*data+writePnt);
		}
		else if(datatype == AM2_DTYPE_FLOAT)
		{
			p_dmydata = ((float *)*data+writePnt);
		}
		else if(datatype == AM2_DTYPE_DOUBLE)
		{
			p_dmydata = ((double *)*data+writePnt);
		}
		else if(datatype == AM2_DTYPE_UCHAR)
		{
			p_dmydata = ((unsigned char *)*data+writePnt);
		}

		if( ( (dataset_no == AM2_PIX_QUAL_HI)
			||(dataset_no == AM2_PIX_QUAL_LO)
			||(dataset_no == AM2_PIX_QUAL)
			||(dataset_no == AM2_PIX_QUAL_A)
			||(dataset_no == AM2_PIX_QUAL_B) )
			&& (datatype == AM2_DTYPE_INT))
		{
			for(cnt1 = 0; cnt1 < readDataCnt; cnt1++)
			{
				if( (chgdatatype == H5T_NATIVE_SHORT)
					|| (chgdatatype == H5T_NATIVE_USHORT) )
				{
					shiftCnt = 16;
					for(cnt2 = 0; cnt2 < shiftCnt; cnt2++)
					{
						if (*((short *)p_rdata+cnt1) & (1<<cnt2))
						{
							*((int *)p_dmydata+writeCnt) = 1;
						}
						else
						{
							*((int *)p_dmydata+writeCnt) = 0;
						}
						writeCnt++;
					}
				}
				else
				{
					shiftCnt = 8;
					for(cnt2 = 0; cnt2 < shiftCnt; cnt2++)
					{
						if (*((char *)p_rdata+cnt1) & (1<<cnt2))
						{
							*((int *)p_dmydata+writeCnt) = 1;
						}
						else
						{
							*((int *)p_dmydata+writeCnt) = 0;
						}
						writeCnt++;
					}
				}
			}
			H5Dclose(dataset);

			free(p_rdata);
			return writeCnt;
		}

		for(cnt1 = 0; cnt1 < readDataCnt; cnt1++)
		{
			switch(chkflg)
			{
				case 1:	/* FLOAT */
					if(datatype == AM2_DTYPE_INT)
					{
						if( *((float *)p_rdata+cnt1) < (AM2_DEF_R_INIT) )
						{
							*((int *)p_dmydata+cnt1) =  *((float *)p_rdata+cnt1);
							continue;
						}
						fans = (float )(*((float *)p_rdata+cnt1)*scale);
						*((int *)p_dmydata+cnt1) = (int )floor(fans+0.5);
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((float *)p_rdata+cnt1)< (AM2_DEF_R_INIT) )
						{
							*((float *)p_dmydata+cnt1) =  *((float *)p_rdata+cnt1);
							continue;
						}
						fans = (float )(*((float *)p_rdata+cnt1)*scale);
						*((float *)p_dmydata+cnt1) = fans;
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((float *)p_rdata+cnt1) < (AM2_DEF_R_INIT) )
						{
							*((double *)p_dmydata+cnt1) =  *((float *)p_rdata+cnt1);
							continue;
						}
						fans = (float )(*((float *)p_rdata+cnt1)*scale);
						*((double *)p_dmydata+cnt1) = fans;
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((float *)p_rdata+cnt1)< (AM2_DEF_R_INIT) )
						{
							*((unsigned char *)p_dmydata+cnt1) =  *((float *)p_rdata+cnt1);
							continue;
						}
						fans = (float )(*((float *)p_rdata+cnt1)*scale);
						if(floor(fans) < 256)
						{
							*((unsigned char *)p_dmydata+cnt1) = (unsigned char )floor(fans+0.5);
						}
						else
						{
							*((unsigned char *)p_dmydata+cnt1) = *((float *)p_rdata+cnt1);
						}
					}
					break;
				case 2:	/* DOUBLE */
					if(datatype == AM2_DTYPE_INT)
					{
						if( *((double *)p_rdata+cnt1)< (AM2_DEF_R_INIT) )
						{
							*((int *)p_dmydata+cnt1) =  *((double *)p_rdata+cnt1);
							continue;
						}
						dans = (double )(*((double *)p_rdata+cnt1)*scale);
						*((int *)p_dmydata+cnt1) = (int )floor(dans+0.5);
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((double *)p_rdata+cnt1)< (AM2_DEF_R_INIT) )
						{
							*((float *)p_dmydata+cnt1) =  *((double *)p_rdata+cnt1);
							continue;
						}
						dans = (double )(*((double *)p_rdata+cnt1)*scale);
						*((float *)p_dmydata+cnt1) = dans;
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((double *)p_rdata+cnt1)< (AM2_DEF_R_INIT) )
						{
							*((double *)p_dmydata+cnt1) =  *((double *)p_rdata+cnt1);
							continue;
						}
						dans = (double )(*((double *)p_rdata+cnt1)*scale);
						*((double *)p_dmydata+cnt1) = dans;
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((double *)p_rdata+cnt1)< (AM2_DEF_R_INIT) )
						{
							*((unsigned char *)p_dmydata+cnt1) =  *((double *)p_rdata+cnt1);
							continue;
						}
						dans = (double )(*((double *)p_rdata+cnt1)*scale);
						if(floor(dans) < 256)
						{
							*((unsigned char *)p_dmydata+cnt1) = (unsigned char )floor(dans+0.5);
						}
						else
						{
							*((unsigned char *)p_dmydata+cnt1) = *((double *)p_rdata+cnt1);
						}
					}
					break;
				case 3:	/* SHORT */
					if(datatype == AM2_DTYPE_INT)
					{
						if( *((short *)p_rdata+cnt1) <= (AM2_DEF_I_INIT) )
						{
							*((int *)p_dmydata+cnt1) =  *((short *)p_rdata+cnt1);
							continue;
						}
						*((int *)p_dmydata+cnt1) = floor(((*((short *)p_rdata+cnt1)*scale)+0.5));
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((short *)p_rdata+cnt1) <= (AM2_DEF_I_INIT) )
						{
							*((float *)p_dmydata+cnt1) =  *((short *)p_rdata+cnt1);
							continue;
						}
					    fans = (float )(*((short *)p_rdata+cnt1)*scale);
						*((float *)p_dmydata+cnt1) = (float )fans;
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((short *)p_rdata+cnt1) <= (AM2_DEF_I_INIT) )
						{
							*((double *)p_dmydata+cnt1) =  *((short *)p_rdata+cnt1);
							continue;
						}
					   dans = (double )(*((short *)p_rdata+cnt1)*scale);
						*((double *)p_dmydata+cnt1) = (double )dans;
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((short *)p_rdata+cnt1) <= (AM2_DEF_I_INIT) )
						{
							*((unsigned char *)p_dmydata+cnt1) =  *((short *)p_rdata+cnt1);
							continue;
						}
					    if((*((short *)p_rdata+cnt1)*scale) < 256)
					    {
							*((unsigned char *)p_dmydata+cnt1) = (unsigned char )floor((*((short *)p_rdata+cnt1)*scale)+0.5);
						}
						else
						{
							*((unsigned char *)p_dmydata+cnt1) = *((short *)p_rdata+cnt1);
						}
					}
					break;
				case 4:	/* USHORT */
					if(datatype == AM2_DTYPE_INT)
					{
						if( *((unsigned short *)p_rdata+cnt1) >= (AM2_DEF_UI_INIT) )
						{
							*((int *)p_dmydata+cnt1) =  *((unsigned short *)p_rdata+cnt1);
							continue;
						}
						*((int *)p_dmydata+cnt1) = floor(((*((unsigned short *)p_rdata+cnt1)*scale)+0.5));
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((unsigned short *)p_rdata+cnt1) >= (AM2_DEF_UI_INIT) )
						{
							*((float *)p_dmydata+cnt1) =  *((unsigned short *)p_rdata+cnt1);
							continue;
						}
					    fans = (float )(*((unsigned short *)p_rdata+cnt1)*scale);
						*((float *)p_dmydata+cnt1) = (float )fans;
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((unsigned short *)p_rdata+cnt1) >= (AM2_DEF_UI_INIT) )
						{
							*((double *)p_dmydata+cnt1) =  *((unsigned short *)p_rdata+cnt1);
							continue;
						}
					   dans = (double )(*((unsigned short *)p_rdata+cnt1)*scale);
						*((double *)p_dmydata+cnt1) = (double )dans;
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((unsigned short *)p_rdata+cnt1) >= (AM2_DEF_UI_INIT) )
						{
							*((unsigned char *)p_dmydata+cnt1) =  *((unsigned short *)p_rdata+cnt1);
							continue;
						}
					    if((*((unsigned short *)p_rdata+cnt1)*scale) < 256)
					    {
							*((unsigned char *)p_dmydata+cnt1) = (unsigned char )floor((*((unsigned short *)p_rdata+cnt1)*scale)+0.5);
						}
						else
						{
							*((unsigned char *)p_dmydata+cnt1) = *((unsigned short *)p_rdata+cnt1);
						}
					}
					break;
				case 5:	/* CHAR */
				case 6:	/* UCHAR */
					if(datatype == AM2_DTYPE_INT)
					{
						if( *((unsigned char *)p_rdata+cnt1) >= AM2_DEF_C_INIT )
						{
							*((int *)p_dmydata+cnt1) =  *((unsigned char *)p_rdata+cnt1);
							continue;
						}
						*((int *)p_dmydata+cnt1) = floor((*((unsigned char *)p_rdata+cnt1)*scale)+0.5);
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((unsigned char *)p_rdata+cnt1) >= AM2_DEF_C_INIT )
						{
							*((float *)p_dmydata+cnt1) =  *((unsigned char *)p_rdata+cnt1);
							continue;
						}
						fans = (float )(*((unsigned char *)p_rdata+cnt1)*scale);
						*((float *)p_dmydata+cnt1) = (float )fans;
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((unsigned char *)p_rdata+cnt1) >= AM2_DEF_C_INIT )
						{
							*((double *)p_dmydata+cnt1) =  *((unsigned char *)p_rdata+cnt1);
							continue;
						}
						dans = (double )(*((unsigned char *)p_rdata+cnt1)*scale);
						*((double *)p_dmydata+cnt1) = (double )dans;
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((unsigned char *)p_rdata+cnt1) >= AM2_DEF_C_INIT )
						{
							*((unsigned char *)p_dmydata+cnt1) =  *((unsigned char *)p_rdata+cnt1);
							continue;
						}
					    if((*((char *)p_rdata+cnt1)*scale) < 256)
					    {
							*((unsigned char *)p_dmydata+cnt1) = *((unsigned char *)p_rdata+cnt1);
						}
						else
						{
							*((unsigned char *)p_dmydata+cnt1) = *((unsigned char *)p_rdata+cnt1);
						}
					}
					break;

				default:
					break;
			}
		}

		writePnt += (hdfdata.data[cnt].datasetsize / hdfdata.data[cnt].datasize);

		H5Dclose(dataset);

		free(p_rdata);
		p_rdata = NULL;
	}
	allDataCnt = readDataCnt * hdfdata.datacnt;
	return allDataCnt;
}

/** @file amtk_hdf.c
 * Set dataset
 *
 * @author     NEC
 * @date       2011/06/13
 */
int setDataSet(hid_t HDF_file_id, void *data, int from_scan, int to_scan, int dataset_no, int datatype)
{

	ACCESS_LABEL	*p_table=NULL;
	int				ret = 0;
	int				ntype = 0;
	int				table_num = 0;
	int				size[3] = {0,0,0};
	int				rank = 0;
	hid_t			dataset = 0;
	int				overlap = 0;
	hid_t			filespace = 0;
	hid_t			memspace = 0;
	float			scale = 0;
	void 			*p_wdata = NULL;
	int				datasize = 0;
	hid_t			getdatatype = 0;
	int				cnt1 = 0;
	int				cnt2 = 0;
	float			fans = 0;
	double			dans = 0;
	hid_t			ndatatype = 0;
	int				chkflg=0;
	int				getmemsize=0;
	int				writeDataCnt=0;
	int				writePnt=0;
	char			warning_flg=IOTK_OFF;

	ret = AccessLabelTypeCheck(HDF_file_id, datatype, dataset_no);
	if(ret < 0)
	{
		return ret;
	}

	switch(dataset_no)
	{
		case AM2_SWATH_GEO1:
		case AM2_SWATH_GEO2:
		case AM2_SWATH_GEO3:
		case AM2_SWATH_GEOA:
		case AM2_SWATHA_GEO1:
		case AM2_SWATHA_GEO2:
		case AM2_SWATHA_GEO3:
		case AM2_SWATHA_GEOA:
		case AM2_SWATHB_GEO1:
		case AM2_SWATHB_GEO2:
		case AM2_SWATHB_GEO3:
		case AM2_SWATHB_GEOA:
			ret = setGridDataSet(HDF_file_id, data, from_scan, to_scan, dataset_no, datatype);
			return ret;

		default:
			break;
	}
	p_table = search_table(HDF_file_id, dataset_no, &table_num);
	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	ret = getOverLapScans( HDF_file_id, &overlap);
	if(ret != SUCCESS)
	{
		return ret;
	}

	rank = AMTK_getDimSize( HDF_file_id, dataset_no, &size[0]);
	if(rank < 0)
	{
		return rank;
	}

	dataset = H5Dopen(HDF_file_id, name_table[table_num][p_table->dataset_no], H5P_DEFAULT);
	if(dataset < 0)
	{
		return ERROR_HDF_DATASET_OPEN;
	}

	ret = getScaleFactor(dataset, &scale);
	if(ret != SUCCESS)
	{
		/* close dataset */
	    H5Dclose(dataset);
		return ret;
	}

	ret = setHyperslab(dataset, from_scan, to_scan, &size[0], rank, overlap, &memspace, &filespace);
	if(ret < 0)
	{
		/* close dataset */
		H5Dclose(dataset);
		return ret;
	}
 
	if(datatype == AM2_DTYPE_INT)
	{
		ntype = H5T_NATIVE_SHORT;
		datasize = sizeof(short);
	}
	else if(datatype == AM2_DTYPE_FLOAT)
	{
		ntype = H5T_NATIVE_FLOAT;
		datasize = sizeof(float);
	}
	else if(datatype == AM2_DTYPE_DOUBLE)
	{
		ntype = H5T_NATIVE_DOUBLE;
		datasize = sizeof(double);
	}
	else 
	{
		ntype = H5T_NATIVE_UCHAR;
		datasize = sizeof(unsigned char);
	}

	getdatatype = H5Dget_type(dataset);
	if(getdatatype < 0)
	{
		/* close dataspace */
		H5Sclose(memspace);
		H5Sclose(filespace);
		/* close dataset */
		H5Dclose(dataset);
		return ERROR_HDF_DATASET_GET_TYPE;
	}
	ndatatype= H5Tget_native_type(getdatatype, H5T_DIR_ASCEND);
	
	/* close datatype */
	H5Tclose(getdatatype);
	
	if(ndatatype < 0)
	{
		/* close dataspace */
		H5Sclose(memspace);
		H5Sclose(filespace);
		/* close dataset */
		H5Dclose(dataset);
		return ERROR_HDF_DT_GET_TYPE;
	}

	if( H5Tequal( ndatatype, H5T_NATIVE_FLOAT))
	{
		ntype = H5T_NATIVE_FLOAT;
		datasize = sizeof(float);
		chkflg = 1;
	}
	else if( H5Tequal( ndatatype, H5T_NATIVE_DOUBLE))
	{
		ntype = H5T_NATIVE_DOUBLE;
		datasize = sizeof(double);
		chkflg = 2;
	}
	else if( H5Tequal( ndatatype, H5T_NATIVE_SHORT))
	{
		ntype = H5T_NATIVE_SHORT;
		datasize = sizeof(short);
		chkflg = 3;
	}
	else if( H5Tequal( ndatatype, H5T_NATIVE_USHORT))
	{
		ntype = H5T_NATIVE_USHORT;
		datasize = sizeof(unsigned short);
		chkflg = 4;
	}
	else if( H5Tequal( ndatatype, H5T_NATIVE_SCHAR))
	{
		ntype = H5T_NATIVE_SCHAR;
		datasize = sizeof(char);
		chkflg = 5;
	}
	else if( H5Tequal( ndatatype, H5T_NATIVE_UCHAR))
	{
		ntype = H5T_NATIVE_UCHAR;
		datasize = sizeof(unsigned char);
		chkflg = 6;
	}
	else if( H5Tequal( ndatatype, H5T_NATIVE_INT))
	{
		ntype = H5T_NATIVE_INT;
		datasize = sizeof(int);
		chkflg = 7;
	}
	else if( H5Tequal( ndatatype, H5T_NATIVE_UINT))
	{
		ntype = H5T_NATIVE_UINT;
		datasize = sizeof(unsigned int);
		chkflg = 8;
	}
	
	/* close datatype */
	H5Tclose(ndatatype);

	p_wdata  = (void *)getMemoryAlloc(&size[0], rank, datasize, from_scan, to_scan, &getmemsize);
	if(p_wdata == NULL)
	{
		/* close dataspace */
		H5Sclose(memspace);
		H5Sclose(filespace);
		/* close dataset */
		H5Dclose(dataset);
		return ERROR_MALLOC;
	}

	writeDataCnt = getmemsize/datasize;

	writePnt = 0;

	if( ( (dataset_no == AM2_PIX_QUAL_HI)
		||(dataset_no == AM2_PIX_QUAL_LO)
		||(dataset_no == AM2_PIX_QUAL)
		||(dataset_no == AM2_PIX_QUAL_A)
		||(dataset_no == AM2_PIX_QUAL_B) )
		&& (datatype == AM2_DTYPE_INT))
	{
		for(cnt1 = 0; cnt1 < writeDataCnt; cnt1++)
		{
			if( (ntype == H5T_NATIVE_SHORT)
				|| (ntype == H5T_NATIVE_USHORT) )
			{
			    *((short *)p_wdata+cnt1) = 0;
				for(cnt2 = 0; cnt2 < 16; cnt2++)
				{
					if(*((int *)data+writePnt) == 1)
					{
					    *((short *)p_wdata+cnt1) |= (1<<cnt2);
					}
					writePnt++;
				}
			}
			else
			{
			    *((char *)p_wdata+cnt1) = 0;
				for(cnt2 = 0; cnt2 < 8; cnt2++)
				{
					if(*((int *)data+writePnt) == 1)
					{
					    *((char *)p_wdata+cnt1) |= (1<<cnt2);
					}
					writePnt++;
				}
			}
		}
	}
	else
	{
		for(cnt1 = 0; cnt1 < writeDataCnt; cnt1++)
		{
			switch(chkflg)
			{
				case 1:	/* FLOAT */
					if(datatype == AM2_DTYPE_INT)
					{
						if( *((int *)data+cnt1) < (AM2_DEF_I_ABNML) )
						{
							*((float *)p_wdata+cnt1) = (float )(*((int *)data+cnt1));
							continue;
						}
						fans = (float )(*((int *)data+cnt1)/scale);
						*((float *)p_wdata+cnt1) = (float )fans;
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((float *)data+cnt1) < (AM2_DEF_R_ABNML) )
						{
							*((float *)p_wdata+cnt1) = *((float *)data+cnt1);
							continue;
						}
						*((float *)p_wdata+cnt1) = (*((float *)data+cnt1)/scale);
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((double *)data+cnt1) < (AM2_DEF_R_ABNML) )
						{
							*((float *)p_wdata+cnt1) = AM2_DEF_RMISS;
							continue;
						}
						*((float *)p_wdata+cnt1) = (*((double *)data+cnt1)/scale);
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((unsigned char *)data+cnt1) >= (AM2_DEF_C_ABNML) )
						{
							*((float *)p_wdata+cnt1) = (float )(*((unsigned char *)data+cnt1));
							continue;
						}
						fans = (float )(*((unsigned char *)data+cnt1)/scale);
						*((float *)p_wdata+cnt1) = (float )fans;
					}
					break;
				case 2:	/* DOUBLE */
					if(datatype == AM2_DTYPE_INT)
					{
						if( *((int *)data+cnt1) < (AM2_DEF_I_ABNML) )
						{
							*((double *)p_wdata+cnt1) = (double )(*((int *)data+cnt1));
							continue;
						}
						dans = (double )(*((int *)data+cnt1)/scale);
						*((double *)p_wdata+cnt1) = (double )dans;
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((float *)data+cnt1)<(AM2_DEF_R_ABNML) )
						{
							*((double *)p_wdata+cnt1) = (double )(*((float *)data+cnt1));
							continue;
						}
						*((double *)p_wdata+cnt1) = (*((float *)data+cnt1)/scale);
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((double *)data+cnt1) < (AM2_DEF_R_ABNML)  )
						{
							*((double *)p_wdata+cnt1) = *((double *)data+cnt1);
							continue;
						}
						*((double *)p_wdata+cnt1) = (*((double *)data+cnt1)/scale);
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((unsigned char *)data+cnt1) == (AM2_DEF_C_ABNML) )
						{
							*((double *)p_wdata+cnt1) = *((unsigned char *)data+cnt1);
							continue;
						}
						dans = (double )(*((unsigned char *)data+cnt1)/scale);
						*((double *)p_wdata+cnt1) = (double )dans;
					}
					break;
				case 3:	/* SHORT */
					if(datatype == AM2_DTYPE_INT)
					{
						if( (*((int *)data+cnt1)) < (AM2_DEF_I_ABNML) )
						{
							if( (*((int *)data+cnt1)) < (AM2_DEF_IMISS))
							{
								*((short *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
								warning_flg = IOTK_ON;
							}else
							{
								*((short *)p_wdata+cnt1) = (short )*((int *)data+cnt1);
							}
							continue;
						}

						if(floor((*((int *)data+cnt1)/scale)+0.5) < 32768)
						{
							*((short *)p_wdata+cnt1) = (short )floor((*((int *)data+cnt1)/scale)+0.5);
						}
						else
						{
							*((short *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((float *)data+cnt1) <= (AM2_DEF_I_ABNML) )
						{
							if( (*((float *)data+cnt1)) < (AM2_DEF_IMISS))
							{
								*((short *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
								warning_flg = IOTK_ON;
							}else
							{
								*((short *)p_wdata+cnt1) = (short )*((float *)data+cnt1);
							}
							continue;
						}
						fans = (float )(*((float *)data+cnt1)/scale);
						if(floor(fans+0.5) < 32768)
						{
						    *((short *)p_wdata+cnt1) = (short )floor(fans+0.5);
						}
						else
						{
						    *((short *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((double *)data+cnt1)< (AM2_DEF_I_ABNML) )
						{
							if( (*((float *)data+cnt1)) < (AM2_DEF_IMISS))
							{
								*((short *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
								warning_flg = IOTK_ON;
							}else
							{
								*((short *)p_wdata+cnt1) = (short )*((double *)data+cnt1);
							}
							continue;
						}
						dans = (double )(*((double *)data+cnt1)/scale);
						if(floor(dans+0.5) < 32768)
						{
							*((short *)p_wdata+cnt1) = (short )floor(dans+0.5);
						}
						else
						{
						    *((short *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((unsigned char *)data+cnt1) >= (AM2_DEF_C_ABNML) )
						{
							*((short *)p_wdata+cnt1) = *((unsigned char *)data+cnt1);
							continue;
						}
						if(floor((*((unsigned char *)data+cnt1)/scale)+0.5) < 32768)
						{
							*((short *)p_wdata+cnt1) = (short )floor((*((unsigned char *)data+cnt1)/scale)+0.5);
						}
						else
						{
							*((short *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					break;
				case 4:	/* USHORT */
					if(datatype == AM2_DTYPE_INT)
					{
						if( *((int *)data+cnt1) >= AM2_DEF_UI_ABNML )
						{
							if( (*((int *)data+cnt1)) > (AM2_DEF_UIMISS))
							{
								*((unsigned short *)p_wdata+cnt1) = AM2_DEF_UI_ABNML;
								warning_flg = IOTK_ON;
							}else
							{
								*((unsigned short *)p_wdata+cnt1) = (unsigned short )*((int *)data+cnt1);
							}
							continue;
						}
						if( *((int *)data+cnt1) < 0 )
						{
							*((unsigned short *)p_wdata+cnt1) = AM2_DEF_UI_ABNML;
							warning_flg = IOTK_ON;
							continue;
						}
						if(floor((*((int *)data+cnt1)/scale)+0.5) < 65536)
						{
							*((unsigned short *)p_wdata+cnt1) = (unsigned short )floor((*((int *)data+cnt1)/scale)+0.5);
						}
						else
						{
							*((unsigned short *)p_wdata+cnt1) = AM2_DEF_UI_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( *((float *)data+cnt1) >= AM2_DEF_UI_ABNML )
						{
							if( (*((float *)data+cnt1)) > (AM2_DEF_UIMISS))
							{
								*((unsigned short *)p_wdata+cnt1) = AM2_DEF_UI_ABNML;
								warning_flg = IOTK_ON;
							}else
							{
								*((unsigned short *)p_wdata+cnt1) = (unsigned short )*((float *)data+cnt1);
							}
							continue;
						}
						if( *((float *)data+cnt1) < 0.0 )
						{
							*((unsigned short *)p_wdata+cnt1) = AM2_DEF_UI_ABNML;
							warning_flg = IOTK_ON;
							continue;
						}
						fans = (float )(*((float *)data+cnt1)/scale);
						if(floor(fans+0.5) < 65536)
						{
						    *((unsigned short *)p_wdata+cnt1) = (unsigned short )floor(fans+0.5);
						}
						else
						{
						    *((unsigned short *)p_wdata+cnt1) = AM2_DEF_UI_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if( *((double *)data+cnt1)< (AM2_DEF_R_ABNML) )
						{
							*((unsigned short *)p_wdata+cnt1) = (unsigned short )*((double *)data+cnt1);
							continue;
						}
						dans = (double )(*((double *)data+cnt1)/scale);
						if(floor(dans+0.5) < 65536)
						{
							*((unsigned short *)p_wdata+cnt1) = (unsigned short )floor(dans+0.5);
						}
						else
						{
						    *((unsigned short *)p_wdata+cnt1) = AM2_DEF_UI_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((unsigned char *)data+cnt1) >= (AM2_DEF_C_ABNML) )
						{
							*((unsigned short *)p_wdata+cnt1) = *((unsigned char *)data+cnt1);
							continue;
						}
						if(floor((*((unsigned char *)data+cnt1)/scale)+0.5) < 65536)
						{
							*((unsigned short *)p_wdata+cnt1) = (unsigned short )floor((*((unsigned char *)data+cnt1)/scale)+0.5);
						}
						else
						{
							*((unsigned short *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					break;
				case 5:	/* CHAR */
				case 6:	/* UCHAR */
					if(datatype == AM2_DTYPE_INT)
					{
						if( (*((int *)data+cnt1)) == (AM2_DEF_CMISS) )
						{
							*((unsigned char *)p_wdata+cnt1) = AM2_DEF_CMISS;
							continue;
						}
						if( (*((int *)data+cnt1)) < 0 )
						{
							*((unsigned char *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
							continue;
						}
						if(floor((*((int *)data+cnt1)/scale)+0.5) < 256)
						{
							*((unsigned char *)p_wdata+cnt1) = (char )floor((*((int *)data+cnt1)/scale)+0.5);
						}
						else
						{
						    *((unsigned char *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_FLOAT)
					{
						if( fabs(*((float *)data+cnt1) - (AM2_DEF_CMISS))<DBL_EPSILON)
						{
							*((unsigned char *)p_wdata+cnt1) = AM2_DEF_CMISS;
							continue;
						}
						if( (*((float *)data+cnt1)) < 0.0 )
						{
							*((unsigned char *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
							continue;
						}
						fans = (float )(*((float *)data+cnt1)/scale);
						if(floor(fans+0.5) < 256)
						{
						    *((unsigned char *)p_wdata+cnt1) = (unsigned char)floor(fans+0.5);
						}
						else
						{
						    *((unsigned char *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_DOUBLE)
					{
						if(  fabs(*((double *)data+cnt1) - (AM2_DEF_CMISS))<DBL_EPSILON )
						{
							*((unsigned char *)p_wdata+cnt1) = AM2_DEF_CMISS;
							continue;
						}
						if( *((double *)data+cnt1) < 0.0 )
						{
							*((unsigned char *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
							continue;
						}
						dans = (double )(*((double *)data+cnt1)/scale);
						if(floor(dans+0.5) < 256)
						{
						    *((unsigned char *)p_wdata+cnt1) = (unsigned char)floor(dans+0.5);
						}
						else
						{
						    *((unsigned char *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					else if(datatype == AM2_DTYPE_UCHAR)
					{
						if( *((unsigned char *)data+cnt1) >= (AM2_DEF_C_ABNML) )
						{
							*((unsigned char *)p_wdata+cnt1) = (unsigned char)(*((unsigned char *)data+cnt1));
							continue;
						}
					    if(floor((*((unsigned char *)data+cnt1)/scale)+0.5) < 256)
					    {
						    *((unsigned char *)p_wdata+cnt1) = (char )floor((*((unsigned char *)data+cnt1)/scale)+0.5);
						}
						else
						{
						    *((unsigned char *)p_wdata+cnt1) = AM2_DEF_C_ABNML;
							warning_flg = IOTK_ON;
						}
					}
					break;
				default:
					break;
			}
		}
	}

	ret = H5Dwrite(dataset,              /* hid_t dset_id        */
					 ntype,   			 /* hid_t mem_type_id    */
					 memspace,           /* hid_t mem_space_id   */
					 filespace,          /* hid_t file_space_id  */
					 H5P_DEFAULT,        /* hid_t plist_id       */
					 p_wdata);             /* void *buf   *out*    */
	
	/* close dataspace */
	H5Sclose(memspace);
	H5Sclose(filespace);
	/* close dataset */
	H5Dclose(dataset);
	/* free area */
	free(p_wdata);
	
	if(ret  < 0 )
	{
		return ERROR_HDF_DATASET_WRITE;
	}
	
	if(warning_flg == IOTK_ON)
	{
		ret = WARNING_DATA;
	}
	else
	{
		ret = SUCCESS;
	}
	return ret;
}

/** @file amtk_hdf.c
 * Get memory size
 *
 * @author     NEC
 * @date       2010/06/30
 */
int getMemorySize(int *size, int rank, int datasize, int from_scan, int to_scan)
{
	int	memsize = 0;

	switch(rank)
	{
		case 1:
			if((from_scan == AM2_START_SCANNUM_ALL) && (to_scan == AM2_END_SCANNUM_ALL))
			{
				memsize = (int )size[0] * datasize;
			}
			else
			{
				if(from_scan < 0)
				{
					memsize = (to_scan + ((from_scan*-1)+1)) * datasize;
				}
				else
				{
					memsize = ((to_scan - from_scan)+1) * datasize;
				}
			}
			break;
		case 2:
			if((from_scan == AM2_START_SCANNUM_ALL) && (to_scan == AM2_END_SCANNUM_ALL))
			{
				memsize = (int )size[0] * (int )size[1] * datasize;
			}
			else
			{
				if(from_scan < 0)
				{
					memsize = (to_scan + ((from_scan*-1)+1)) * (int )size[1] * datasize;
				}
				else
				{
					memsize = ((to_scan - from_scan)+1) * (int )size[1] * datasize;
				}
			}
			break;
		case 3:
			if((from_scan == AM2_START_SCANNUM_ALL) && (to_scan == AM2_END_SCANNUM_ALL))
			{
				memsize = (int )size[0] * (int )size[1] * (int )size[2] * datasize;
			}
			else
			{
				if(from_scan < 0)
				{
					memsize = (int )size[0] * (to_scan + ((from_scan*-1)+1)) * (int )size[2] * datasize;
				}
				else 
				{
					memsize = (int )size[0] * ((to_scan - from_scan)+1) * (int )size[2] * datasize;
				}
			}
			break;
		default:
			memsize = 0;
			break;
	}
    return memsize;
}

/** @file amtk_hdf.c
 * Get memory allocation
 *
 * @author     NEC
 * @date       2010/06/30
 */
void *getMemoryAlloc(int *size, int rank, int datasize, int from_scan, int to_scan, int *getmemsize)
{
	int		ret = 0;
	void 	*p_buff = NULL;

	*getmemsize = 0;

	ret = getMemorySize(size, rank, datasize, from_scan, to_scan);
	if(ret == 0)
	{
		return NULL;
	}
	p_buff = (void *)malloc(ret);
	if(p_buff == NULL)
	{
		return NULL;
	}
	*getmemsize = ret;
    return p_buff;
}

/** @file amtk_hdf.c
 * Get Geophysical Information
 *
 * @author     NEC
 * @date       2010/06/30
 */
int getGeophysicalInfo(char* geophysicalName,  float* scale, char* unit)
{
	FILE	*p_fp = NULL;
	char	buf[256];
	char	*ptn = NULL;
	char    *p_str = NULL;
	int    ret=0;

	p_str = getenv("GEOPHYSICALFILE");
	if(p_str != NULL)
	{
		p_fp = fopen(p_str,"r");
	}
	else
	{
		p_fp = fopen("geophysical_file","r");
	}
	if(p_fp == NULL)
	{
		return ERROR_GEOPHSICAL_FILE;
	}
	while(NULL != fgets(buf,256,p_fp))
	{
		if( buf[0] == '#')
		{
			continue;
		}
		ptn = strtok(buf,",");
		ret = strcmp( ptn, geophysicalName);
		if( ret == 0 )
		{
			ptn = strtok(NULL,",");
			if( ptn == NULL)
			{
				break;
			}
			*scale = atof(ptn);

			ptn = strtok(NULL,",");
			if( ptn == NULL)
			{
				break;
			}
			strcpy( unit, ptn );
			fclose(p_fp);
			return SUCCESS;
		}
	}
	fclose(p_fp);

	return ERROR_GEOPHSICAL_FILE;
}

/** @file amtk_hdf.c
 * Get scale factor for L3
 *
 * @author     NEC
 * @date       2010/11/30
 */
int getL3Scalefactor(hid_t HDF_file_id, int dataset_no, float* scale, char* unit)
{
	int       ret = 0;
	char		buf[80];
	char      *p_dmybuf = NULL;

	/* L2,L3 Scale factor depends on Geophysical data */
	switch(dataset_no)
	{
		case AM2_SWATH_GEO1:
		case AM2_SWATH_GEO2:
		case AM2_SWATH_GEO3:
		case AM2_SWATH_GEOA:
		case AM2_SWATHA_GEO1:
		case AM2_SWATHA_GEO2:
		case AM2_SWATHA_GEO3:
		case AM2_SWATHA_GEOA:
		case AM2_SWATHB_GEO1:
		case AM2_SWATHB_GEO2:
		case AM2_SWATHB_GEO3:
		case AM2_SWATHB_GEOA:
		case AM2_GRID_GEO1:
		case AM2_GRID_GEO2:
		case AM2_GRID_GEO3:
		case AM2_GRID_GEOA:

			p_dmybuf = &buf[0];
			
			/* Get meta data (AMTK inside) */
			ret = getMetaDataInside(HDF_file_id, AM2_DEF_META_GEOPHYSICALNAME, &p_dmybuf,
				ERROR_HDF_METAOP_GEONAME, ERROR_HDF_METANG_GEONAME);
			if(ret < 0)
			{
				return ret;
			}
			ret = getGeophysicalInfo(p_dmybuf,  scale, unit);
			if(ret < 0)
			{
				return ret;
			}
		
		default:
			break;
	}
	return SUCCESS ;
}

/** @file amtk_hdf.c
 * Set scale factor
 *
 * @author     NEC
 * @date       2011/06/13
 */
int setScaleFactor(hid_t dataset, float value)
{
	hid_t	attr = 0;
	hid_t	filespace = 0;
	hid_t	atype = 0;
	hsize_t	size[3]={0,0,0};
	int		ret = 0;


	attr = H5Aopen_name(dataset, "SCALE FACTOR");
	if(attr < 0)
	{
		size[0] = 1;
		filespace = H5Screate_simple(1, &size[0], NULL);
		if(filespace < 0)
		{
			return ERROR_HDF_CREATE_SPACE;
		}
		attr = H5Acreate(dataset, "SCALE FACTOR", H5T_NATIVE_FLOAT, filespace, H5P_DEFAULT, H5P_DEFAULT);
		
		/* close dataspace */
		H5Sclose(filespace);
		
		if(attr < 0)
		{
			return ERROR_HDF_ATTR_CREATE;
		}
	}

	atype = H5Aget_type(attr);
	if(atype < 0)
	{
		H5Aclose(attr);
		return ERROR_HDF_ATTR_GET_TYPE;
	}
	ret = H5Awrite(attr, atype, &value);
	
	/* close datatype */
	H5Tclose(atype);
	
	if(ret < 0)
	{
		H5Aclose(attr);
		return ERROR_HDF_ATTR_WRITE;
	}
	H5Aclose(attr);

    return SUCCESS;
}

/** @file amtk_hdf.c
 * Set unit
 *
 * @author     NEC
 * @date       2011/06/13
 */
int setUnit(hid_t dataset, char* unit)
{
	hid_t	attr = 0;
	hid_t	filespace = 0;
	hid_t	atype = 0;
	hsize_t	size[3]={0,0,0};
	hid_t	memtype = 0;
	int		ret = 0;
	long		len = 0L;

	attr = H5Aopen_name(dataset, "UNIT");
	if(attr < 0)
	{
		memtype = H5Tcopy (H5T_C_S1);
		if(memtype < 0)
		{
			IOTK_printf("setUnit ERROR - 1\n");
			return ERROR_HDF_DT_COPY;
		}
		len = strlen(unit);
		if(len == 0)
		{
			len = 1L;
		}
		else
		{
			/* NULL+1 */
			len+=1L;
		}
		ret = H5Tset_size (memtype, len);
		if(ret < 0)
		{
			IOTK_printf("setUnit ERROR - 2\n");
			/* close datatype */
			H5Tclose(memtype);
			return ERROR_HDF_DT_SET_SIZE;
		}
		
		size[0] = 1;
		filespace = H5Screate_simple(1, &size[0], NULL);
		if(filespace < 0)
		{
			/* close datatype */
			H5Tclose(memtype);
			return ERROR_HDF_CREATE_SPACE;
		}
		attr = H5Acreate(dataset, "UNIT", memtype, filespace, H5P_DEFAULT, H5P_DEFAULT);
		
		/* close dataspace */
		H5Sclose(filespace);
		/* close datatype */
		H5Tclose(memtype);
		
		if(attr < 0)
		{
			return ERROR_HDF_ATTR_CREATE;
		}
	}
	atype = H5Aget_type(attr);
	if(atype < 0)
	{
		H5Aclose(attr);
		return ERROR_HDF_ATTR_GET_TYPE;
	}
	ret = H5Awrite(attr, atype, unit);
	
	/* close datatype */
	H5Tclose(atype);
	/* close attribute */
	H5Aclose(attr);
	
	if(ret < 0)
	{
		return ERROR_HDF_ATTR_WRITE;
	}

    return SUCCESS;
}

/** @file amtk_hdf.c
 * Set Hyperslab for HDF
 * FAIL: *memspace and *filespace is negative value.
 *
 * @author     NEC
 * @date       2011/06/13
 */
int	setHyperslab(hid_t dataset, int from_scan, int to_scan, int	*size, int rank, int overlap, hid_t *memspace, hid_t *filespace)
{
	int		readStartNo = 0;
	int		readEndNo = 0;
	hsize_t	offset_out[3] = { 0, 0, 0 };
	hsize_t	count_out[3] = { 0, 0, 0 };
	hsize_t	stride[3] = { 1, 1, 1};
	int		ret = 0;
	int		chkdata = 0;

	if( (from_scan == AM2_START_SCANNUM_ALL) && (to_scan == AM2_END_SCANNUM_ALL) )
	{
		*memspace = H5S_ALL;
		*filespace = H5S_ALL;
		return SUCCESS;
	}

	readStartNo = ((overlap + from_scan)-1);
	if(readStartNo < 0)
	{
	    return ERROR_HDF_HYPERSLAB;
	}
	if(rank == 3)
	{
	    chkdata = size[1];
	}
	else
	{
	    chkdata = size[0];
	}
    if((to_scan+overlap) > chkdata)
    {
		readEndNo = chkdata;
    }
    else
    {
        if(from_scan == to_scan)
        {
    		readEndNo = readStartNo+1;
        }
        else
        {
    		readEndNo = (to_scan+overlap);
        }
    }
	*filespace = H5Dget_space(dataset);
	if(*filespace < 0)
	{
		return ERROR_HDF_DATASET_GET_SPACE;
	}

	if(rank == 3)
	{
		offset_out[0] = 0;
		offset_out[1] = readStartNo;
		offset_out[2] = 0;
		ret = H5Sget_simple_extent_dims(*filespace, count_out, NULL);
		if(ret < 0)
		{
			/* close dataspace */
			H5Sclose(*filespace);
			*filespace = -1;
			*memspace = -1;
			return ERROR_HDF_DATASET_GET_DIMS;
		}

		count_out[1] = (readEndNo - readStartNo);
		ret = H5Sselect_hyperslab(*filespace, H5S_SELECT_SET, offset_out, stride, count_out, NULL );
		if(ret < 0)
		{
			/* close dataspace */
			H5Sclose(*filespace);
			*filespace = -1;
			*memspace = -1;
			return ERROR_HDF_HYPERSLAB;
		}

		offset_out[0] = 0;
		offset_out[1] = 0;
		offset_out[2] = 0;
		*memspace = H5Screate_simple( rank, count_out, NULL);
		if(*memspace < 0)
		{
			/* close dataspace */
			H5Sclose(*filespace);
			*filespace = -1;
			*memspace = -1;
			return ERROR_HDF_CREATE_SPACE;
		}
	}
	else
	{
		offset_out[0] = readStartNo;
		offset_out[1] = 0;
		offset_out[2] = 0;
		ret = H5Sget_simple_extent_dims(*filespace, count_out, NULL);
		if(ret < 0)
		{
			/* close dataspace */
			H5Sclose(*filespace);
			*filespace = -1;
			*memspace = -1;
			return ERROR_HDF_DATASET_GET_DIMS;
		}

		count_out[0] = (readEndNo - readStartNo);
		ret = H5Sselect_hyperslab(*filespace, H5S_SELECT_SET, offset_out, NULL, count_out, NULL );
		if(ret < 0)
		{
			/* close dataspace */
			H5Sclose(*filespace);
			*filespace = -1;
			*memspace = -1;
			return ERROR_HDF_HYPERSLAB;
		}

		offset_out[0] = 0;
		offset_out[1] = 0;
		offset_out[2] = 0;
		*memspace = H5Screate_simple( rank, count_out, NULL);
		if(*memspace < 0)
		{
			/* close dataspace */
			H5Sclose(*filespace);
			*filespace = -1;
			*memspace = -1;
			return ERROR_HDF_CREATE_SPACE;
		}
	}
	ret = H5Sselect_hyperslab(*memspace, H5S_SELECT_SET, offset_out, NULL, count_out, NULL );
	if(ret < 0)
	{
		/* close dataspace */
		H5Sclose(*memspace);
		*memspace = -1;
		H5Sclose(*filespace);
		*filespace = -1;
		return ERROR_HDF_HYPERSLAB;
	}

	return SUCCESS;
}

/** @file amtk_hdf.c
 * Get dimension size
 *
 * @author     NEC
 * @date       2010/06/30
 */
int getDimSize(hid_t dataset, int *size)
{
	int		status = 0;
	int		rank = 0;
	hid_t	space = 0;
	hsize_t	sIZE1[3] = {0,0,0};
	hsize_t	sIZE2[3] = {0,0,0};

	space  = H5Dget_space(dataset);
	if(space < 0)
	{
		IOTK_printf("getDimSize ERROR - 2\n");
		return ERROR_HDF_DATASET_GET_SPACE;
	}

	rank   = H5Sget_simple_extent_ndims(space);
	if(rank < 0)
	{
		IOTK_printf("getDimSize ERROR - 3\n");
		/* close dataspace */
	    H5Sclose(space);
		return ERROR_HDF_DATASET_GET_RANK;
	}

	status = H5Sget_simple_extent_dims(space, (hsize_t *)&sIZE1, (hsize_t *)&sIZE2);

	/* close dataspace */
	H5Sclose(space);
    
	if(status < 0)
	{
		IOTK_printf("getDimSize ERROR - 4\n");
		return ERROR_HDF_DATASET_GET_DIMS;
	}
	size[0] = (int)sIZE1[0];
	size[1] = (int)sIZE1[1];
	size[2] = (int)sIZE1[2];

	return rank;
}

/** @file amtk_hdf.c
 * Get grid hyperlab
 * FAIL: *memspace and *filespace is negative value.
 *
 * @author     NEC
 * @date       2011/06/13
 */
int	setGridHyperslab(hid_t dataset, int from_scan, int to_scan, int overlap, int rank, int getGrid, hid_t *memspace, hid_t *filespace)
{
	int		readStartNo = 0;
	int		readEndNo = 0;
	hsize_t	offset_out[3] = { 0, 0, 0 };
	hsize_t	count_out[3] = { 0, 0, 0 };
	hsize_t	stride[3] = { 1, 1, 1};
	int		ret = 0;
	hsize_t	size[3] = { 0, 0, 0 };

	*filespace = H5Dget_space(dataset);
	if(*filespace < 0)
	{
		*filespace = -1;
		*memspace = -1;
		return ERROR_HDF_DATASET_GET_SPACE;
	}
	ret = H5Sget_simple_extent_dims(*filespace, size, NULL);

	if((from_scan == AM2_START_SCANNUM_ALL) && (to_scan == AM2_END_SCANNUM_ALL))
	{
		readStartNo = 0;
	}
	else
	{
		readStartNo = ((overlap + from_scan)-1);

		if((to_scan+overlap) > size[0])
		{
			readEndNo = size[0];
	    }
		else
		{
			if(from_scan == to_scan)
			{
				readEndNo = readStartNo+1;
			}
			else
			{
				readEndNo = (to_scan+overlap);
			}
		}
	}
	offset_out[0] = readStartNo;
	offset_out[1] = 0;
	offset_out[2] = getGrid;
	ret = H5Sget_simple_extent_dims(*filespace, count_out, NULL);
	if(ret < 0)
	{
		/* close dataspace */
		H5Sclose(*filespace);
		*filespace = -1;
		*memspace = -1;
		return ERROR_HDF_DATASET_GET_DIMS;
	}

	if((from_scan == AM2_START_SCANNUM_ALL) && (to_scan == AM2_END_SCANNUM_ALL))
	{
		;
	}
	else
	{
		if(readEndNo < readStartNo )
		{
			/* close dataspace */
			H5Sclose(*filespace);
			*filespace = -1;
			*memspace = -1;
			return ERROR_HDF_HYPERSLAB;
		}
		count_out[0] = (readEndNo - readStartNo);
	}
	count_out[2] = 1;
	ret = H5Sselect_hyperslab(*filespace, H5S_SELECT_SET, offset_out, stride, count_out, NULL );
	if(ret < 0)
	{
		/* close dataspace */
		H5Sclose(*filespace);
		*filespace = -1;
		*memspace = -1;
		return ERROR_HDF_HYPERSLAB;
	}

	offset_out[0] = 0;
	offset_out[1] = 0;
	offset_out[2] = 0;
	*memspace = H5Screate_simple( rank, count_out, NULL);
	if(*memspace < 0)
	{
		/* close dataspace */
		H5Sclose(*filespace);
		*filespace = -1;
		*memspace = -1;
		return ERROR_HDF_CREATE_SPACE;
	}
	ret = H5Sselect_hyperslab(*memspace, H5S_SELECT_SET, offset_out, NULL, count_out, NULL );
	if(ret < 0)
	{
		/* close dataspace */
		H5Sclose(*filespace);
		*filespace = -1;
		H5Sclose(*memspace);
		*memspace = -1;
		return ERROR_HDF_HYPERSLAB;
	}

	return SUCCESS;
}

/** @file amtk_hdf.c
 * Get grid dataset
 *
 * @author     NEC
 * @date       2011/06/13
 */
int getGridDataSet(hid_t HDF_file_id, void **data, int from_scan, int to_scan, int dataset_no, int datatype)
{

	ACCESS_LABEL	*p_table=NULL;
	int			ret = 0;
	int			table_num = 0;
	int			dataset = 0;
	int			overlap = 0;
	void		*p_dmydata = NULL;
	void		*p_rdata = NULL;
	int			cnt = 0;
	int			cnt1 = 0;
	int			writePnt=0;
	char		mallocflg = IOTK_OFF;
	hid_t   	filespace = 0;
	hid_t   	memspace = 0;
	int			readDataCnt = 0;
	int			datasize = 0;
	int			datasize1 = 0;
	int			getmemsize = 0;
	int			getmemsize1 = 0;
	float		scale = 0.0;
	float		fans = 0.0;
	int			rank = 0;
	int			rrank = 0;
	int			chgrank = 0;
	int			size[3] = {0,0,0};
	int			rsize[3] = {0,0,0};
	int			readPnt[3];
	int			chg_dataset_no = 0;
	int			readData = 0;

	switch(dataset_no)
	{
		case AM2_GRID_GEO1:
			chg_dataset_no = AM2_GRID_GEOA;
			readPnt[0] = 0;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEO2:
			chg_dataset_no = AM2_GRID_GEOA;
			readPnt[0] = 1;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEO3:
			chg_dataset_no = AM2_GRID_GEOA;
			readPnt[0] = 2;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEOA:
			chg_dataset_no = AM2_GRID_GEOA;
			readPnt[0] = 0;
			readPnt[1] = 1;
			readPnt[2] = 2;
			readData = 3;
			break;

		case AM2_GRID_GEO1_ANUM:
			chg_dataset_no = AM2_GRID_GEOA_ANUM;
			readPnt[0] = 0;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEO2_ANUM:
			chg_dataset_no = AM2_GRID_GEOA_ANUM;
			readPnt[0] = 1;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEO3_ANUM:
			chg_dataset_no = AM2_GRID_GEOA_ANUM;
			readPnt[0] = 2;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEOA_ANUM:
			chg_dataset_no = AM2_GRID_GEOA_ANUM;
			readPnt[0] = 0;
			readPnt[1] = 1;
			readPnt[2] = 2;
			readData = 3;
			break;

		case AM2_GRID_GEO1_TNUM:
			chg_dataset_no = AM2_GRID_GEOA_TNUM;
			readPnt[0] = 0;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEO2_TNUM:
			chg_dataset_no = AM2_GRID_GEOA_TNUM;
			readPnt[0] = 1;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEO3_TNUM:
			chg_dataset_no = AM2_GRID_GEOA_TNUM;
			readPnt[0] = 2;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEOA_TNUM:
			chg_dataset_no = AM2_GRID_GEOA_TNUM;
			readPnt[0] = 0;
			readPnt[1] = 1;
			readPnt[2] = 2;
			readData = 3;
			break;

		case AM2_GRID_GEO1_STD:
			chg_dataset_no = AM2_GRID_GEOA_STD;
			readPnt[0] = 0;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEO2_STD:
			chg_dataset_no = AM2_GRID_GEOA_STD;
			readPnt[0] = 1;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEO3_STD:
			chg_dataset_no = AM2_GRID_GEOA_STD;
			readPnt[0] = 2;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_GRID_GEOA_STD:
			chg_dataset_no = AM2_GRID_GEOA_STD;
			readPnt[0] = 0;
			readPnt[1] = 1;
			readPnt[2] = 2;
			readData = 3;
			break;

		case AM2_SWATH_GEO1:
			chg_dataset_no = AM2_SWATH_GEOA;
			readPnt[0] = 0;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATH_GEO2:
			chg_dataset_no = AM2_SWATH_GEOA;
			readPnt[0] = 1;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATH_GEO3:
			chg_dataset_no = AM2_SWATH_GEOA;
			readPnt[0] = 2;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATH_GEOA:
			chg_dataset_no = AM2_SWATH_GEOA;
			readPnt[0] = 0;
			readPnt[1] = 1;
			readPnt[2] = 2;
			readData = 3;
			break;

		case AM2_SWATHA_GEO1:
			chg_dataset_no = AM2_SWATHA_GEOA;
			readPnt[0] = 0;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATHA_GEO2:
			chg_dataset_no = AM2_SWATHA_GEOA;
			readPnt[0] = 1;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATHA_GEO3:
			chg_dataset_no = AM2_SWATHA_GEOA;
			readPnt[0] = 2;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATHA_GEOA:
			chg_dataset_no = AM2_SWATHA_GEOA;
			readPnt[0] = 0;
			readPnt[1] = 1;
			readPnt[2] = 2;
			readData = 3;
			break;

		case AM2_SWATHB_GEO1:
			chg_dataset_no = AM2_SWATHB_GEOA;
			readPnt[0] = 0;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATHB_GEO2:
			chg_dataset_no = AM2_SWATHB_GEOA;
			readPnt[0] = 1;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATHB_GEO3:
			chg_dataset_no = AM2_SWATHB_GEOA;
			readPnt[0] = 2;
			readPnt[1] = 0;
			readPnt[2] = 0;
			readData = 1;
			break;
		case AM2_SWATHB_GEOA:
			chg_dataset_no = AM2_SWATHB_GEOA;
			readPnt[0] = 0;
			readPnt[1] = 1;
			readPnt[2] = 2;
			readData = 3;
			break;

		default:
			break;
	}

	p_table = search_table(HDF_file_id, chg_dataset_no, &table_num);
	if(p_table == NULL)
	{
		if(mallocflg == IOTK_ON)
		{
			free(*data);
			*data = NULL;
		}
		return ERROR_NOT_ACCESSLABEL;
	}

	rank = AMTK_getDimSize(HDF_file_id, chg_dataset_no , &size[0]);
	if(rank < 0)
	{
		return rank;
	}
	
	/* Check layer size */
	if ((readData == 3) && (readData != size[2]))
	{
		/* Get readData:3 > File dimsize  */
		readData = size[2];
	}

	dataset = H5Dopen(HDF_file_id, name_table[table_num][p_table->dataset_no], H5P_DEFAULT);
	if(dataset < 0)
	{
		if(mallocflg == IOTK_ON)
		{
			free(*data);
			*data = NULL;
		}
		return ERROR_HDF_DATASET_OPEN;
	}

	ret = getScaleFactor(dataset, &scale);
	if(ret != SUCCESS)
	{
		if(mallocflg == IOTK_ON)
		{
			free(*data);
			*data = NULL;
		}
		H5Dclose(dataset);
		return ret;
	}

	if(readData != 3)
	{
	    chgrank = 2;
		size[2] = 0;
	}
    else
    {
	    chgrank = 3;
    }
    rrank = 2;
    rsize[0] = size[0];
    rsize[1] = size[1];
    rsize[2] = 0;
	datasize = sizeof(short);
	p_rdata  = (void *)getMemoryAlloc(&rsize[0], rrank, datasize, from_scan, to_scan, &getmemsize);
	if(p_rdata == NULL)
	{
		H5Dclose(dataset);
		return ERROR_MALLOC;
	}

	if(*data == NULL)
	{
		if(datatype == AM2_DTYPE_INT)
		{
			datasize1 = sizeof(int);
		}
		else
		{
			datasize1 = sizeof(float);
		}
		*data = (void *)(void *)getMemoryAlloc(&size[0], chgrank, datasize1, from_scan, to_scan, &getmemsize1);
		if(*data == NULL)
		{
			free(p_rdata);
			return ERROR_MALLOC;
		}
		mallocflg = IOTK_ON;
	}

	switch(chg_dataset_no)
	{
		case AM2_SWATH_GEOA:
		case AM2_SWATHA_GEOA:
		case AM2_SWATHB_GEOA:
			ret = getOverLapScans( HDF_file_id, &overlap);
			if(ret != SUCCESS)
			{
			    if(mallocflg == IOTK_ON)
			    {
				    free(*data);
				    *data = NULL;
			    }
			    free(p_rdata);
			    H5Dclose(dataset);
				return ret;
			}
			break;
		default:
			overlap = 0;
			break;
	}
	for(cnt = 0; cnt < readData; cnt++)
	{
		ret = setGridHyperslab(dataset, from_scan, to_scan, overlap, rank, readPnt[cnt], &memspace, &filespace);
		if(ret < 0)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			free(p_rdata);
			H5Dclose(dataset);
			return ret;
	    }
		ret = H5Dread(dataset,		    		/* hid_t dset_id        */
					  H5T_NATIVE_SHORT,			/* hid_t mem_type_id    */
					  memspace,					/* hid_t mem_space_id   */
					  filespace,				/* hid_t file_space_id  */
					  H5P_DEFAULT,				/* hid_t plist_id       */
					  p_rdata);					/* void *buf   *out*    */
		
		/* close dataspace */
		H5Sclose(memspace);
		H5Sclose(filespace);
		
		if(ret < 0)
		{
			if(mallocflg == IOTK_ON)
			{
				free(*data);
				*data = NULL;
			}
			free(p_rdata);
			H5Dclose(dataset);
			return ERROR_HDF_DATASET_READ;
		}

		readDataCnt = (getmemsize / datasize);

		if(datatype == AM2_DTYPE_INT)
		{
			p_dmydata = ((int *)*data+writePnt);
		}
		else if(datatype == AM2_DTYPE_FLOAT)
		{
			p_dmydata = ((float *)*data+writePnt);
		}

		for(cnt1 = 0; cnt1 < readDataCnt; cnt1++)
		{
			if(datatype == AM2_DTYPE_INT)
			{
			    if( *((short *)p_rdata+cnt1) < (AM2_DEF_I_INIT) )
			    {
			         *((int *)p_dmydata+cnt1) =  *((short *)p_rdata+cnt1);
			         continue;
			    }
			    *((int *)p_dmydata+cnt1) = floor((*((short *)p_rdata+cnt1)*scale)+0.5);
			}
			else if(datatype == AM2_DTYPE_FLOAT)
			{
			    if( *((short *)p_rdata+cnt1) < (AM2_DEF_I_INIT) )
			    {
			        *((float *)p_dmydata+cnt1) =  *((short *)p_rdata+cnt1);
			        continue;
			    }
			    fans = (float )(*((short *)p_rdata+cnt1)*scale);
			    *((float *)p_dmydata+cnt1) = (float )fans;
			}
		}
		writePnt += readDataCnt;
	}
	H5Dclose(dataset);

	readDataCnt *= readData;
	free(p_rdata);
	p_rdata = NULL;
	return readDataCnt;
}

/** @file amtk_hdf.c
 * Set grid dataset
 *
 * @author     NEC
 * @date       2011/06/13
 */
int setGridDataSet(hid_t HDF_file_id, void *data, int from_scan, int to_scan, int dataset_no, int datatype)
{

	ACCESS_LABEL	*p_table=NULL;
	int				ret = 0;
	int				table_num = 0;
	int				dataset = 0;
	int				overlap = 0;
	void			*p_dmydata = NULL;
	void			*p_wdata = NULL;
	int				cnt = 0;
	int				cnt1 = 0;
	hid_t   		filespace = 0;
	hid_t   		memspace = 0;
	int				writeDataCnt = 0;
	int				datasize = 0;
	int				getmemsize = 0;
	float			scale = 0.0;
	float			fans = 0.0;
	int				rank = 0;
	int				rrank = 0;
	int				size[3] = {0,0,0};
	int				rsize[3] = {0,0,0};
	int				writePnt[3];
	int				readPnt=0;
	int				chg_dataset_no = 0;
	int				writeData = 0;
	int				warning_flg = 0;

	switch(dataset_no)
	{
		case AM2_GRID_GEO1:
			chg_dataset_no = AM2_GRID_GEOA;
			writePnt[0] = 0;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEO2:
			chg_dataset_no = AM2_GRID_GEOA;
			writePnt[0] = 1;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEO3:
			chg_dataset_no = AM2_GRID_GEOA;
			writePnt[0] = 2;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEOA:
			chg_dataset_no = AM2_GRID_GEOA;
			writePnt[0] = 0;
			writePnt[1] = 1;
			writePnt[2] = 2;
			writeData = 3;
			break;

		case AM2_GRID_GEO1_ANUM:
			chg_dataset_no = AM2_GRID_GEOA_ANUM;
			writePnt[0] = 0;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEO2_ANUM:
			chg_dataset_no = AM2_GRID_GEOA_ANUM;
			writePnt[0] = 1;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEO3_ANUM:
			chg_dataset_no = AM2_GRID_GEOA_ANUM;
			writePnt[0] = 2;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEOA_ANUM:
			chg_dataset_no = AM2_GRID_GEOA_ANUM;
			writePnt[0] = 0;
			writePnt[1] = 1;
			writePnt[2] = 2;
			writeData = 3;
			break;

		case AM2_GRID_GEO1_TNUM:
			chg_dataset_no = AM2_GRID_GEOA_TNUM;
			writePnt[0] = 0;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEO2_TNUM:
			chg_dataset_no = AM2_GRID_GEOA_TNUM;
			writePnt[0] = 1;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEO3_TNUM:
			chg_dataset_no = AM2_GRID_GEOA_TNUM;
			writePnt[0] = 2;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEOA_TNUM:
			chg_dataset_no = AM2_GRID_GEOA_TNUM;
			writePnt[0] = 0;
			writePnt[1] = 1;
			writePnt[2] = 2;
			writeData = 3;
			break;

		case AM2_GRID_GEO1_STD:
			chg_dataset_no = AM2_GRID_GEOA_STD;
			writePnt[0] = 0;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEO2_STD:
			chg_dataset_no = AM2_GRID_GEOA_STD;
			writePnt[0] = 1;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEO3_STD:
			chg_dataset_no = AM2_GRID_GEOA_STD;
			writePnt[0] = 2;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_GRID_GEOA_STD:
			chg_dataset_no = AM2_GRID_GEOA_STD;
			writePnt[0] = 0;
			writePnt[1] = 1;
			writePnt[2] = 2;
			writeData = 3;
			break;

		case AM2_SWATH_GEO1:
			chg_dataset_no = AM2_SWATH_GEOA;
			writePnt[0] = 0;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATH_GEO2:
			chg_dataset_no = AM2_SWATH_GEOA;
			writePnt[0] = 1;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATH_GEO3:
			chg_dataset_no = AM2_SWATH_GEOA;
			writePnt[0] = 2;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATH_GEOA:
			chg_dataset_no = AM2_SWATH_GEOA;
			writePnt[0] = 0;
			writePnt[1] = 1;
			writePnt[2] = 2;
			writeData = 3;
			break;

		case AM2_SWATHA_GEO1:
			chg_dataset_no = AM2_SWATHA_GEOA;
			writePnt[0] = 0;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATHA_GEO2:
			chg_dataset_no = AM2_SWATHA_GEOA;
			writePnt[0] = 1;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATHA_GEO3:
			chg_dataset_no = AM2_SWATHA_GEOA;
			writePnt[0] = 2;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATHA_GEOA:
			chg_dataset_no = AM2_SWATHA_GEOA;
			writePnt[0] = 0;
			writePnt[1] = 1;
			writePnt[2] = 2;
			writeData = 3;
			break;

		case AM2_SWATHB_GEO1:
			chg_dataset_no = AM2_SWATHB_GEOA;
			writePnt[0] = 0;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATHB_GEO2:
			chg_dataset_no = AM2_SWATHB_GEOA;
			writePnt[0] = 1;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATHB_GEO3:
			chg_dataset_no = AM2_SWATHB_GEOA;
			writePnt[0] = 2;
			writePnt[1] = 0;
			writePnt[2] = 0;
			writeData = 1;
			break;
		case AM2_SWATHB_GEOA:
			chg_dataset_no = AM2_SWATHB_GEOA;
			writePnt[0] = 0;
			writePnt[1] = 1;
			writePnt[2] = 2;
			writeData = 3;
			break;

		default:
			break;
	}

	p_table = search_table(HDF_file_id, chg_dataset_no, &table_num);
	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	rank = AMTK_getDimSize(HDF_file_id, chg_dataset_no , &size[0]);
	if(rank < 0)
	{
		return rank;
	}

	dataset = H5Dopen(HDF_file_id, name_table[table_num][p_table->dataset_no], H5P_DEFAULT);
	if(dataset < 0)
	{
		return ERROR_HDF_DATASET_OPEN;
	}

	ret = getScaleFactor(dataset, &scale);
	if(ret != SUCCESS)
	{
		H5Dclose(dataset);
		return ret;
	}

	if(writeData != 3)
	{
		size[2] = 0;
	}

	rrank = 2;
	rsize[0] = size[0];
	rsize[1] = size[1];
	rsize[2] = 0;
	datasize = sizeof(short);
	p_wdata  = (void *)getMemoryAlloc(&rsize[0], rrank, datasize, from_scan, to_scan, &getmemsize);
	if(p_wdata == NULL)
	{
		H5Dclose(dataset);
		return ERROR_MALLOC;
	}

	switch(chg_dataset_no)
	{
		case AM2_SWATH_GEOA:
		case AM2_SWATHA_GEOA:
		case AM2_SWATHB_GEOA:
			ret = getOverLapScans( HDF_file_id, &overlap);
			if(ret != SUCCESS)
			{
			    free(p_wdata);
			    H5Dclose(dataset);
				return ret;
			}
			break;
		default:
			overlap = 0;
			break;
	}
	for(cnt = 0; cnt < writeData; cnt++)
	{
		ret = setGridHyperslab(dataset, from_scan, to_scan, overlap, rank, writePnt[cnt], &memspace, &filespace);
		if(ret < 0)
		{
			free(p_wdata);
			H5Dclose(dataset);
			return ret;
		}

		if(datatype == AM2_DTYPE_INT)
		{
			p_dmydata = ((int *)data+readPnt);
		}
		else if(datatype == AM2_DTYPE_FLOAT)
		{
			p_dmydata = ((float *)data+readPnt);
		}

		writeDataCnt = (getmemsize / datasize);
		for(cnt1 = 0; cnt1 < writeDataCnt; cnt1++)
		{
		    if(datatype == AM2_DTYPE_INT)
		    {
				if( *((int *)p_dmydata+cnt1) <= AM2_DEF_I_ABNML )
				{
					if( (*((int *)p_dmydata+cnt1)) < (AM2_DEF_IMISS))
					{
						*((short  *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
						warning_flg = IOTK_ON;
					}else
					{
						*((short  *)p_wdata+cnt1) = (short )*((int *)p_dmydata+cnt1);
					}
					continue;
				}
				if(floor((*((int *)p_dmydata+cnt1)/scale)+0.5) < 32768)
				{
					*((short  *)p_wdata+cnt1) = (short )floor((*((int *)p_dmydata+cnt1)/scale)+0.5);
				}
				else
				{
					*((short  *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
					warning_flg = IOTK_ON;
				}
			}
			else if(datatype == AM2_DTYPE_FLOAT)
			{
				if( ((*((float *)p_dmydata+cnt1)) - (AM2_DEF_I_ABNML)) <= 0.0)
				{
				    if( (*((float *)p_dmydata+cnt1)) < (AM2_DEF_IMISS))
				    {
						*((short  *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
						warning_flg = IOTK_ON;
				    }else
				    {
						*((short  *)p_wdata+cnt1) = (short )*((float *)p_dmydata+cnt1);
				    }
				    continue;
				}
				fans = (float )(*((float *)p_dmydata+cnt1)/scale);
				if(floor(fans+0.5) < 32768.0 )
				{
				    *((short *)p_wdata+cnt1) = (short )floor(fans+0.5);
				}
				else
				{
				    *((short *)p_wdata+cnt1) = AM2_DEF_I_ABNML;
				    warning_flg = IOTK_ON;
				}
			}
		}
		ret = H5Dwrite( dataset,		    		/* hid_t dset_id        */
						H5T_NATIVE_SHORT,			/* hid_t mem_type_id    */
						memspace,					/* hid_t mem_space_id   */
						filespace,					/* hid_t file_space_id  */
						H5P_DEFAULT,				/* hid_t plist_id       */
						p_wdata);						/* void *buf   *out*    */
		
		/* close dataspace */
		H5Sclose(memspace);
		H5Sclose(filespace);
		
		if(ret < 0)
		{
			free(p_wdata);
			H5Dclose(dataset);
			return ERROR_HDF_DATASET_WRITE;
		}

		readPnt += writeDataCnt;
	}
	H5Dclose(dataset);

	free(p_wdata);
	p_wdata = NULL;
	
	if(warning_flg == IOTK_ON)
	{
		ret = WARNING_DATA;
	}
	else
	{
		ret = SUCCESS;
	}
	return ret;
}

